#define MAX_SPRITES	150
#define MAX_SHOTS 20

#define NTSC 0
#define PAL 1
#define SECAM 2

// object color table indexes
#define COLOR_ASTEROID          0   //  0x28  // orange
#define COLOR_MINE              1   //  0x88  // blue
#define COLOR_SHIP              2   //  0x0c  // white, subtract 2 due to lumaboost
#define COLOR_I                 3   //  0x58  // pink
#define COLOR_P                 4   //  0xb8  // cyan
#define COLOR_E                 5   //  0x38  // reddish orange
#define COLOR_SPY               6   //  0x18  // yellow
#define COLOR_STATION           7   //  0xc8  // green
#define COLOR_CONDITION_GREEN   8
#define COLOR_CONDITION_YELLOW  9
#define COLOR_CONDITION_RED     10

#define STARFIELD_FRAC_BITS 2

#define SHOT_SHAPE_MASK         3
#define SHOT_SHAPE_SHIFT        0
#define SHOT_DIRECTION_MASK     0x1C
#define SHOT_DIRECTION_SHIFT    2

// station direction, used in level layout
#define HS              0
#define VS              1024
#define VS_SHIFT        10

// RADAR_PIXEL is how many screen pixels each dot on the radar represents
// use either 64 or 128
#define RADAR_PIXEL     64
#define WORLD_WIDTH     512
#define WORLD_HEIGHT    1024
#define X_FRACTION_BITS 7
#define Y_FRACTION_BITS 6
#define RADAR_SHIFT     6
#define STATION_X_MASK  511

//#define RADAR_PIXEL     128
//#define WORLD_WIDTH     1024
//#define WORLD_HEIGHT    2048
//#define X_FRACTION_BITS 6
//#define Y_FRACTION_BITS 5
//#define RADAR_SHIFT     7
//#define STATION_X_MASK  1023

#define PLAYER_SPRITE           8
#define FORMATION_SPRITE        9
#define FIRST_STATIONARY_SPRITE 22
#define STATIONARY_DENSITY      4

#define PLAYER_BUFFER_SPACE     25

// For gun cool down use
// *2 = 20
// *3 = 15
// *4 = 10
#define GUN_COOL_DOWN   15
#define STATION_COOL_DOWN 15

// constants for gSpriteState
#define SS_SPRITE_ACTIVE        0x80
#define SS_STATION_ONSCREEN     0x40
#define SS_EXPLOSION_NOT_DEADLY 0x20
#define SS_EXPLOSION_TIME_MASK  0x0f
#define EXPLOSION_TIME          15

// constants for gStationState
#define SS_STATION_OPEN         0x40
#define SS_ENERGY_FRAME         0x80
#define SS_ENERGY_FRAME_SHIFT   7

//#define QUAD_TEST               1

// copy-paste the echo'd defines from the 6507 code

#define FRAME                      (*(unsigned char *)(0x40000C00 + 2693 ))

// data from 6507 to ARM
#define SUB                        (*(unsigned char *)(0x40000C00 + 2699 ))
#define SWCHA                      (*(unsigned char *)(0x40000C00 + 2700 ))
#define SWCHB                      (*(unsigned char *)(0x40000C00 + 2701 ))
#define INPT4                      (*(unsigned char *)(0x40000C00 + 2702 ))
#define VB_TIME                    (*(unsigned char *)(0x40000C00 + 2703 ))
#define OS_TIME                    (*(unsigned char *)(0x40000C00 + 2704 ))

// data from ARM to 6507 - from overscan call
#define MODE                       (*(unsigned char *)(0x40000C00 + 2705 ))
#define AUDV                       ((unsigned char *)(0x40000C00 + 2710 ))
#define AUDF                       ((unsigned char *)(0x40000C00 + 2708 ))
#define AUDC                       ((unsigned char *)(0x40000C00 + 2706 ))
#define AUDV0                      (*(unsigned char *)(0x40000C00 + 2710 ))
#define AUDF0                      (*(unsigned char *)(0x40000C00 + 2708 ))
#define AUDC0                      (*(unsigned char *)(0x40000C00 + 2706 ))
#define AUDV1                      (*(unsigned char *)(0x40000C00 + 2711 ))
#define AUDF1                      (*(unsigned char *)(0x40000C00 + 2709 ))
#define AUDC1                      (*(unsigned char *)(0x40000C00 + 2707 ))
#define PLAYER_X                   ((unsigned char *)(0x40000C00 + 2686 ))
#define PLAYER_0_X                 (*(unsigned char *)(0x40000C00 + 2687 ))
#define PLAYER_1_X                 (*(unsigned char *)(0x40000C00 + 2686 ))
#define MISSILE_X                  ((unsigned char *)(0x40000C00 + 2684 ))
#define MISSILE_0_X                (*(unsigned char *)(0x40000C00 + 2685 ))
#define MISSILE_1_X                (*(unsigned char *)(0x40000C00 + 2684 ))
#define BALL_X                     (*(unsigned char *)(0x40000C00 + 2683 ))
#define FIRST_NUSIZ                ((unsigned char *)(0x40000C00 + 2688 ))
#define FIRST_NUSIZ0               (*(unsigned char *)(0x40000C00 + 2688 ))
#define FIRST_NUSIZ1               (*(unsigned char *)(0x40000C00 + 2689 ))
#define FIRST_COLUP                ((unsigned char *)(0x40000C00 + 2690 ))
#define FIRST_COLUP0               (*(unsigned char *)(0x40000C00 + 2690 ))
#define FIRST_COLUP1               (*(unsigned char *)(0x40000C00 + 2691 ))
#define FIRST_COLUPF               (*(unsigned char *)(0x40000C00 + 2692 ))

// kernel datastreams
#define DATASTREAM_SIZE             160
#define P0_DATASTREAM               322
#define P1_DATASTREAM               503
#define MISSILE_BALL_DATASTREAM     684
#define HMP0_DATASTREAM             844
#define HMP1_DATASTREAM             1004
#define COLOR_HM_DATASTREAM         1164
#define SIZE_BALL_COLOR_DATASTREAM  1324
#define JUMP_DATASTREAM             0
#define MISSILE_REPOSITION_ACTIVE  ((unsigned char *)(0x40000C00 + 1484 ))

// jump data
#define NORMAL_KERNEL               61896
#define KERNEL_DONE                 61952
#define FIRST_P0_JUMP               62278
#define LAST_P0_JUMP                62721
#define FIRST_P1_JUMP               62765
#define LAST_P1_JUMP                63208
#define FIRST_M0_JUMP               63252
#define LAST_M0_JUMP                63696
#define FIRST_M1_JUMP               63740
#define LAST_M1_JUMP                64184
#define RESP0_STROBE_TABLE         ((unsigned short int *)(0x0C00 + 18432 ))
#define RESP1_STROBE_TABLE         ((unsigned short int *)(0x0C00 + 18454 ))
#define RESM0_STROBE_TABLE         ((unsigned short int *)(0x0C00 + 18476 ))
#define RESM1_STROBE_TABLE         ((unsigned short int *)(0x0C00 + 18498 ))
#define RESBL_STROBE_TABLE         ((unsigned short int *)(0x0C00 + 18520 ))
#define INIT_LEVEL_KERNEL           65395

// Image Data
#define IMAGE_HEIGHTS               18612
#define IMAGE_SHIFT                 18608
#define IMAGE_GRAPHICS              18542
#define STATION_IMAGES              1764
#define STATION_HEIGHT              65
#define OBJECT_COLORS               2288
#define OBJECT_COLOR_SIZE           11

// Image IDs
#define QUAD_SPRITES                1
#define DOUBLE_SPRITES              5
#define SHIFTING_SPRITES            1
#define STATION_HORIZONTAL_ID       0
#define STATION_VERTICAL_ID         1
#define ASTEROID_ID                 6
#define MINE_ID                     9
#define SHIP_ID                     10
#define SHIP_HEIGHT                 16
#define ETYPE_ID                    18
#define ITYPE_ID                    20
#define PTYPE_ID                    22
#define SPY_ID                      24
#define SHOT_HORIZONTAL_ID          26
#define SHOT_DIAGONAL_ID            27
#define SHOT_VERTICAL_ID            28
#define SHOT_STATION_ID             29
#define EXPLOSION_ID                2
#define LAST_EXPLOSION_ID           5
#define POD_ID                      30
#define HORIZONTAL_ENERGY_POD_ID    31
#define VERTICAL_ENERGY_POD_ID      32

// starfield
#define MAX_STARS                   128
#define STARFIELD_X                ((unsigned char *)(0x40000C00 + 2299 ))
#define STARFIELD_Y                ((unsigned char *)(0x40000C00 + 2427 ))
#define STARFIELD_COLOR            ((unsigned char *)(0x40000C00 + 2555 ))

// score, radar, status
#define RADAR_HEIGHT                20
#define PLAYFIELD_DATA              1644
#define PF0L                        0
#define PF1L                        20
#define PF2L                        40
#define PF0R                        60
#define PF1R                        80
#define PF2R                        100
#define PF_DIGIT0                  ((unsigned char *)(0x0C00 + 18645 ))
#define PF_DIGIT_HEIGHT             15
#define PF_DIGIT_X                  16
#define PF_DIGIT_G                  17
#define PF_DIGIT_Y                  18
#define PF_DIGIT_R                  19
#define PF_DIGIT_V                  20
#define PF_DIGIT_O                  21
#define RADAR_CONTROL              ((unsigned char *)(0x40000C00 + 2284 ))

// Sound Effects
#define ACTIVE_SOUND               ((unsigned char *)(0x40000C00 + 2695 ))
#define ACTIVE_SOUND_0             (*(unsigned char *)(0x40000C00 + 2695 ))
#define ACTIVE_SOUND_1             (*(unsigned char *)(0x40000C00 + 2696 ))
#define SOUND_DURATION             ((unsigned char *)(0x40000C00 + 2697 ))
#define SOUND_DURATION_0           (*(unsigned char *)(0x40000C00 + 2697 ))
#define SOUND_DURATION_1           (*(unsigned char *)(0x40000C00 + 2698 ))
#define SOUND_FREQ                  27293
#define SOUND_VOL                   27352
#define SOUND_PLAYER_SHOOTS_ID      13
#define SOUND_ASTEROID_ID           29
#define SOUND_MINE_ID               42
#define SOUND_PLAYER_HIT_ID         56
#define SOUND_MENU_BLIP_ID          58

// Main Menu
#define MENU_OPTIONS_DS             1488
#define MAIN_MENU_GRAPHICS          0
#define MAIN_MENU_GFX_SIZE          1724
#define MAIN_MENU_COLORS            1548
#define MAIN_MENU_CLR_SIZE          176
#define MENU_SELECTED_COLOR         1688
#define MENU_FIRST_OPTION_ID        0
#define MENU_LEVEL_ID               0
#define MENU_TV_TYPE_ID             1
#define MENU_START_ID               2
#define OPTION_PAL_A                1250
#define OPTION_PAL_B                1258
#define OPTION_SECAM_A              1266
#define OPTION_SECAM_B              1274
#define OPTION_SECAM_C              1282
#define MM_JOYSTICK_TIMER          (*(unsigned char *)(0x40000C00 + 2712 ))
#define MM_FIRE_DOWN               (*(unsigned char *)(0x40000C00 + 2713 ))
#define MM_SELECTED_OPTION         (*(unsigned char *)(0x40000C00 + 2714 ))
#define MM_INITIAL_LEVEL           (*(unsigned char *)(0x40000C00 + 2715 ))
#define MM_TV_TYPE                 (*(unsigned char *)(0x40000C00 + 2716 ))
#define MENU_DIGITS                ((unsigned char *)(0x0C00 + 25922 ))
#define LEVEL_DIGITS_A             ((unsigned char *)(0x40000C00 + 1346 ))
#define LEVEL_DIGITS_B             ((unsigned char *)(0x40000C00 + 1354 ))