    PROCESSOR 6502
; Draconian
; Last Update:  May 3, 2014
; Progam by:    Darrell Spice, Jr.

TIA_BASE_READ_ADDRESS = $30
        include VCS.H
        include macro.h
        include graphics.h
        include DPCplus.h
;        include speakjet.inc

        
; TIA object usage
;----------------------------------------
;   P0 = All Sprites
;   P1 = All Sprites
;   M0 = All Shots
;   M1 = All Shots
;   BL = All Stars
;   PF = Score/Radar
;----------------------------------------        
        
;----------------------------------------
; Data Fetcher usage, game screen  
;----------------------------------------
;   DF0DATA     = Player 0 Graphics
;   DF1DATA     = Player 1 Graphics
;   DF2DATA     = ENABL, ENAM0 and ENAM1
;   DF3DATA     = HMP0
;   DF4DATA     = HMP1
;   DF5DATA     = COLUP or HMxx data for Missile/Ball, used during reposition kernel
;   DF6DATA     = NUSIZx or COLUPF data used during reposition kernel
;   DF7DATA     = Jump to Next Kernel
;
;   DF0FRACDATA = PF0 left,  used for score/radar
;   DF1FRACDATA = PF1 left,  used for score/radar
;   DF2FRACDATA = PF2 left,  used for score/radar
;   DF3FRACDATA = PF0 right, used for score/radar
;   DF4FRACDATA = PF1 right, used for score/radar
;   DF5FRACDATA = PF2 right, used for score/radar
;   DF6FRACDATA = RadarControl, holds colors and HMBL adjust for ball position

DS_GRP0             = DF0DATA
DS_GRP1             = DF1DATA
DS_M1M0BL           = DF2DATA
DS_HMP0             = DF3DATA
DS_HMP1             = DF4DATA
DS_COLUP_HMMB       = DF5DATA
DS_NUSIZ_COLUPF     = DF6DATA
DS_JUMP             = DF7DATA

DS_PF0L             = DF0FRACDATA
DS_PF1L             = DF1FRACDATA
DS_PF2L             = DF2FRACDATA
DS_PF0R             = DF3FRACDATA
DS_PF1R             = DF4FRACDATA
DS_PF2R             = DF5FRACDATA
DS_RADAR_CONTROL    = DF6FRACDATA

; datastream usage to display two-color 96 pixel menu title
DS_GRP0A            = DF0DATA
DS_GRP0B            = DF1DATA
DS_GRP0C            = DF2DATA
DS_GRP1A            = DF3DATA
DS_GRP1B            = DF4DATA
DS_GRP1C            = DF5DATA
DS_COLUBK           = DF6DATA
DS_PF2              = DF7DATA


HMOVE_L7          =  $70
HMOVE_L6          =  $60
HMOVE_L5          =  $50
HMOVE_L4          =  $40
HMOVE_L3          =  $30
HMOVE_L2          =  $20
HMOVE_L1          =  $10
HMOVE_0           =  $00
HMOVE_R1          =  $F0
HMOVE_R2          =  $E0
HMOVE_R3          =  $D0
HMOVE_R4          =  $C0
HMOVE_R5          =  $B0
HMOVE_R6          =  $A0
HMOVE_R7          =  $90
HMOVE_R8          =  $80

; Color constants.  Only need NTSC, will do on the fly conversion for
; PAL and SECAM
GREY        = $00
YELLOW      = $10
ORANGE      = $20
RED_ORANGE  = $30
RED         = $40
PURPLE      = $50
VIOLET      = $60
BLUE        = $80
GREEN       = $C0
WHITE       = $0E


;MM_TITLE_GAP    = 47
MM_OPTION_GAP   = 2
MM_START_GAP    = 15
MM_LOGO_GAP     = 16

; set these to adjust storage allocation for the ARM code
DATASTREAM_SIZE = 160
RADAR_HEIGHT = 20       ; radar is 16 pixels up/down with 2 extra at top/bottom for spacing
MAX_STARS = 128         ; needs to be a power of 2

;----------------------------------------
; Zero Page RAM  
;----------------------------------------
; ZP RAM can only be seen by the 6507.  Any values that the ARM needs to see
; must be stored in the 4K Display Data bank
;----------------------------------------
        SEG.U VARS
        ORG $80
NextKernel:     ds 2        
Mode:           ds 1
TimeLeftOS:     ds 1
TimeLeftVB:     ds 1
Sleep5:         ds 1
LoopCounter:    ds 1
Frame:          ds 1

 echo "----",($00FE - *) , "bytes of RAM left"
        
;----------------------------------------
; Bank 0-4 - ARM user code
;----------------------------------------
; The ARM code must start at the beginning of the ROM space in bank 0.
; Do NOT make space for the 128 DPC+ registers like is done in banks
; for 6507 code. 
;----------------------------------------
        SEG CODE               
        ORG $0000
        ; include the Harmony/Melody driver so we don't have to merge it later
HM_DRIVER        
        INCBIN DPC+.arm
        ; all following ORGs need to compensate for the driver size
HM_DRIVER_SIZE = * - HM_DRIVER      
        
        ORG $0000 + HM_DRIVER_SIZE   
        RORG $0000
        ; include the custom ARM code.  This must be compiled before compiling
        ; the 6507 code by executing the MAKE command in the custom directory.
        
        INCBIN custom/bin/custom2.bin   
        
AsteroidA:
        .byte zz__XX____ ; 0
        .byte zz__XXX_X_ ; 1
        .byte zz_XXXXXXX ; 2
        .byte zz_XX_XXXX ; 3
        .byte zzXXXX_X_X ; 4
        .byte zz_XXX_X_X ; 5
        .byte zzX_XX__X_ ; 6
        .byte zzXX_X_XX_ ; 7
        .byte zzXX__X___ ; 8
        .byte zz_XXXXX__ ; 9
        .byte zz__XXX___ ; 10
        .byte zz___XX___ ; 11
AsteroidAHeight = * - AsteroidA

AsteroidB:
        .byte zz___X__X_ ; 0
        .byte zz__XX_XX_ ; 1
        .byte zz___XX_XX ; 2
        .byte zz__X_XX_X ; 3
        .byte zz_XXXXX_X ; 4
        .byte zz_XXXX_XX ; 5
        .byte zz_XX_X__X ; 6
        .byte zz_X_X_X__ ; 7
        .byte zzXX__XXX_ ; 8
        .byte zzXX_XXXX_ ; 9
        .byte zzX_XXXX__ ; 10
        .byte zzX_XX_X__ ; 11
        .byte zz_X_X____ ; 12
AsteroidBHeight = * - AsteroidB

AsteroidC:
        .byte zz___XX___ ; 0
        .byte zz___XX_XX ; 1
        .byte zz__XXXXXX ; 2
        .byte zz__XXXXX_ ; 3
        .byte zz_XX_XXXX ; 4
        .byte zz_X_XXXXX ; 5
        .byte zzXX_XXX_X ; 6
        .byte zzXX_XX___ ; 7
        .byte zzXXX___X_ ; 8
        .byte zzXX_XXXX_ ; 9
        .byte zz__XXXXX_ ; 10
        .byte zz_XXX_X__ ; 11
        .byte zz__X_____ ; 12
AsteroidCHeight = * - AsteroidC
      
Mine:   
        .byte zz___X____ ; 0
        .byte zz___X____ ; 1
        .byte zz___X____ ; 2
        .byte zzX_____X_ ; 3
        .byte zz_X_X_X__ ; 4
        .byte zz__XXX___ ; 5
        .byte zz__XXX___ ; 6
        .byte zzX_XXX_X_ ; 7
        .byte zz__XX____ ; 8
        .byte zz__X_X___ ; 9
        .byte zz_X_X_X__ ; 10
        .byte zzX_____X_ ; 11
        .byte zz___X____ ; 12
        .byte zz___X____ ; 13
        .byte zz___X____ ; 14     
MineHeight = * - Mine        

Ship0:
        .byte zz____X___ ; 0
        .byte zz____X___ ; 1
        .byte zz____X___ ; 2
        .byte zz____X___ ; 3
        .byte zz____X___ ; 4
        .byte zz___XXX__ ; 5
        .byte zz___XXX__ ; 6
        .byte zz_X_X_X_X ; 7
        .byte zz_X_X_X_X ; 8
        .byte zz_X_X_X_X ; 9
        .byte zz_X_XXX_X ; 10
        .byte zz_XXXXXXX ; 11
        .byte zz_XXXXXXX ; 12
        .byte zz_X__X__X ; 13
        .byte zz___XXX__ ; 14
        .byte zz____X___ ; 15
;        .byte zz________ ; 16
Ship0Height = * - Ship0


Ship1:      
        .byte zz________ ; 0
        .byte zz___X____ ; 1
        .byte zz__X_____ ; 2
        .byte zz_X______ ; 3
        .byte zzX______X ; 4
        .byte zzXX___XX_ ; 5
        .byte zz_XXXXXX_ ; 6
        .byte zz_XXX_X__ ; 7
        .byte zz_XX_XX__ ; 8
        .byte zz_XX_XX__ ; 9
        .byte zzXXXXX___ ; 10
        .byte zzXXXXX___ ; 11
        .byte zz_XXXX___ ; 12
        .byte zz_X_X__X_ ; 13
        .byte zz___X_X__ ; 14
        .byte zz___XX___ ; 15
        .byte zz___X____ ; 16        
Ship1Height = * - Ship1


Ship2:
        .byte zz__X_____ ; 0
        .byte zz_XXXX___ ; 1
        .byte zz__X_____ ; 2
        .byte zz__XX____ ; 3
        .byte zz__XX____ ; 4
        .byte zzX_XXX___ ; 5
        .byte zzXXX_XXX_ ; 6
        .byte zzXXX__XXX ; 7
        .byte zzXXX_XXX_ ; 8
        .byte zzX_XXX___ ; 9
        .byte zz__XX____ ; 10
        .byte zz__XX____ ; 11
        .byte zz__X_____ ; 12
        .byte zz_XXXX___ ; 13
        .byte zz__X_____ ; 14
;        .byte zz________ ; 15
Ship2Height = * - Ship2


Ship3:       
        .byte zz___X____ ; 0
        .byte zz___XX___ ; 1
        .byte zz___X_X__ ; 2
        .byte zz_X_X__X_ ; 3
        .byte zz_XXXX___ ; 4
        .byte zzXXXXX___ ; 5
        .byte zzXXXXX___ ; 6
        .byte zz_XX_XX__ ; 7
        .byte zz_XX__X__ ; 8
        .byte zz_XXXXX__ ; 9
        .byte zz_XXXXXX_ ; 10
        .byte zzXX___XX_ ; 11
        .byte zzX______X ; 12
        .byte zz_X______ ; 13
        .byte zz__X_____ ; 14
        .byte zz___X____ ; 15        
Ship3Height = * - Ship3


Ship4:
        .byte zz____X___ ; 0
        .byte zz___XXX__ ; 1
        .byte zz_X__X__X ; 2
        .byte zz_XXXXXXX ; 3
        .byte zz_XXXXXXX ; 4
        .byte zz_X_XXX_X ; 5
        .byte zz_X_X_X_X ; 6
        .byte zz_X_X_X_X ; 7
        .byte zz_X_X_X_X ; 8
        .byte zz___XXX__ ; 9
        .byte zz___XXX__ ; 10
        .byte zz____X___ ; 11
        .byte zz____X___ ; 12
        .byte zz____X___ ; 13
        .byte zz____X___ ; 14
        .byte zz____X___ ; 15
Ship4Height = * - Ship4


Ship5:
        .byte zz___XX___ ; 0
        .byte zz__X_X___ ; 1
        .byte zz_X__X_X_ ; 2
        .byte zz___XXXX_ ; 3
        .byte zz___XXXXX ; 4
        .byte zz___XXXXX ; 5
        .byte zz__XX_XX_ ; 6
        .byte zz__X__XX_ ; 7
        .byte zz__XXXXX_ ; 8
        .byte zz_XXXXXX_ ; 9
        .byte zz_XX___XX ; 10
        .byte zzX______X ; 11
        .byte zz______X_ ; 12
        .byte zz_____X__ ; 13
Ship5Height = * - Ship5


Ship6:
        .byte zz_____X__ ; 0
        .byte zz___XXXX_ ; 1
        .byte zz_____X__ ; 2
        .byte zz____XX__ ; 3
        .byte zz____XX__ ; 4
        .byte zz___XXX_X ; 5
        .byte zz_XXX_XXX ; 6
        .byte zzXXX__XXX ; 7
        .byte zz_XXX_XXX ; 8
        .byte zz___XXX_X ; 9
        .byte zz____XX__ ; 10
        .byte zz____XX__ ; 11
        .byte zz_____X__ ; 12
        .byte zz___XXXX_ ; 13
        .byte zz_____X__ ; 14
Ship6Height = * - Ship6


Ship7:
        .byte zz____X___ ; 0
        .byte zz_____X__ ; 1
        .byte zz______X_ ; 2
        .byte zzX______X ; 3
        .byte zz_XX___XX ; 4
        .byte zz_XXXXXX_ ; 5
        .byte zz__X_XXX_ ; 6
        .byte zz__XX_XX_ ; 7
        .byte zz__XX_XX_ ; 8
        .byte zz___XXXXX ; 9
        .byte zz___XXXXX ; 10
        .byte zz___XXXX_ ; 11
        .byte zz_X__X_X_ ; 12
        .byte zz__X_X___ ; 13
        .byte zz___XX___ ; 14
        .byte zz____X___ ; 15        
Ship7Height = * - Ship7       


        .byte 0         ; part of StationHorizontal
StationHorizontal:
        .byte zz___X____ ; 0
        .byte zz___X____ ; 1
        .byte zz__XX____ ; 2
        .byte zz__XX____ ; 3
        .byte zz__XX____ ; 4
        .byte zz__X_____ ; 5
        .byte zz__XX____ ; 6
        .byte zz__X_____ ; 7
        .byte zz__XX____ ; 8
        .byte zz_XX_X___ ; 9
        .byte zz__XX____ ; 10
        .byte zz_XX_X___ ; 11
        .byte zz__XX____ ; 12
        .byte zzX____X__ ; 13
        .byte zz___X____ ; 14
        .byte zzXX___XX_ ; 15
        .byte zzXX_X_XX_ ; 16
        .byte zzXX_X_XX_ ; 17
        .byte zzX__X_X__ ; 18
        .byte zzXX___XX_ ; 19
        .byte zzX__X_X__ ; 20
        .byte zzXX_X_XX_ ; 21
        .byte zzX_XXXX__ ; 22
        .byte zzXX_X_XX_ ; 23
        .byte zzX_XXXX__ ; 24
        .byte zzXX___XX_ ; 25
        .byte zz___X____ ; 26
        .byte zzX____X__ ; 27
        .byte zz__XX____ ; 28
        .byte zz__XX____ ; 29
        .byte zz__XX____ ; 30
        .byte zz__X_____ ; 31
        .byte zz__XX____ ; 32
        .byte zz__X_____ ; 33
        .byte zz__XX____ ; 34
        .byte zz__X_____ ; 35
        .byte zz__XX____ ; 36
        .byte zzX____X__ ; 37
        .byte zz___X____ ; 38
        .byte zzXX___XX_ ; 39
        .byte zzXXXXXXX_ ; 40
        .byte zzXX_X_XX_ ; 41
        .byte zzX_XXXX__ ; 42
        .byte zzXX_X_XX_ ; 43
        .byte zzX__X_X__ ; 44
        .byte zzXX___XX_ ; 45
        .byte zzX__X_X__ ; 46
        .byte zzXX_X_XX_ ; 47
        .byte zzX__X_X__ ; 48
        .byte zzXX___XX_ ; 49
        .byte zz___X____ ; 50
        .byte zzX____X__ ; 51
        .byte zz__XX____ ; 52
        .byte zz_XXXX___ ; 53
        .byte zz__XX____ ; 54
        .byte zz_XX_X___ ; 55
        .byte zz__XX____ ; 56
        .byte zz__X_____ ; 57
        .byte zz__XX____ ; 58
        .byte zz__X_____ ; 59
        .byte zz__XX____ ; 60
        .byte zz__X_____ ; 61
        .byte zz__XX____ ; 62
        .byte zz________ ; 63
        .byte zz___X____ ; 64
StationHorizontalHeight = * - StationHorizontal
        .byte 0         ; part of StationHorizontal


Shift_StationHorizontal:
        .byte HMOVE_0  |  0  |  8   ; bit 3 on for "in use" test in DataStream()
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_R2 |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_L2 |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_R2 |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_L2 |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_R2 |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_L2 |  0  |  8
        .byte HMOVE_0  |  0  |  8
        
; in quad collision test, need to do something about the gaps in the image. 
; idea is to use something like this:
;    QuadBits(image2[j]) | (QuadBits(image2[j-1]) & QuadBits(image2[j+1]))
; which will require 0 padding on either side of the station graphic.
 
        .byte 0         ; part of StationVertical        
StationVertical:
        .byte zz__X__X__ ; 0
        .byte zz__X__X__ ; 1
        .byte zz_XX_XX__ ; 2
        .byte zz_XX_XX__ ; 3
        .byte zz_XX_XX__ ; 4
        .byte zz_X__X___ ; 5
        .byte zz_XX_XX__ ; 6
        .byte zz_X__X___ ; 7
        .byte zz_XX_XX__ ; 8
        .byte zz_X__X___ ; 9
        .byte zz_XX_XX__ ; 10
        .byte zz_X__X___ ; 11
        .byte zz_XX_XX__ ; 12
        .byte zz________ ; 13
        .byte zz__X__X__ ; 14
        .byte zz________ ; 15
        .byte zz__X__X__ ; 16
        .byte zz_X____X_ ; 17
        .byte zz__X_X___ ; 18
        .byte zz_X____X_ ; 19
        .byte zz__X_X___ ; 20
        .byte zzX__X__X_ ; 21
        .byte zzX__X__X_ ; 22
        .byte zzXX_XX_XX ; 23
        .byte zzXX_XX_XX ; 24
        .byte zzXX_XX_XX ; 25
        .byte zzX_XX_XX_ ; 26
        .byte zzXXXXXXXX ; 27
        .byte zzX_XX_XX_ ; 28
        .byte zzXX_XX_XX ; 29
        .byte zzX_XX_XX_ ; 30
        .byte zzXX_XX_XX ; 31
        .byte zzX__X__X_ ; 32
        .byte zzXX_XX_XX ; 33
        .byte zz________ ; 34
        .byte zzX__X__X_ ; 35
        .byte zz__X_X___ ; 36
        .byte zz_X____X_ ; 37
        .byte zz__X_X___ ; 38
        .byte zz_X____X_ ; 39
        .byte zz__X__X__ ; 40
        .byte zz________ ; 41
        .byte zz__X__X__ ; 42
        .byte zz__X__X__ ; 43
        .byte zz_XX_XX__ ; 44
        .byte zz_XX_XX__ ; 45
        .byte zz_XX_XX__ ; 46
        .byte zz_X__X___ ; 47
        .byte zz_XX_XX__ ; 48
        .byte zz_X__X___ ; 49
        .byte zz_XX_XX__ ; 50
        .byte zz_X__X___ ; 51
        .byte zz_XX_XX__ ; 52
        .byte zz_X__X___ ; 53
        .byte zz_XX_XX__ ; 54
        .byte zz________ ; 55
        .byte zz__X__X__ ; 56
StationVerticalHeight = * - StationVertical
        .byte 0         ; part of StationVertical        


Shift_StationVertical:
        .byte HMOVE_0  |  0  |  8   ; bit 3 on for "in use" test in DataStream()
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_R2 |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_L2 |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_R2 |  2  |  8
        .byte HMOVE_L2 |  0  |  8
        .byte HMOVE_R2 |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_L2 |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_R2 |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_L2 |  0  |  8
        .byte HMOVE_R2 |  2  |  8
        .byte HMOVE_L2 |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_0  |  0  |  8
        .byte HMOVE_R2 |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_0  |  2  |  8
        .byte HMOVE_L2 |  0  |  8
        .byte HMOVE_0  |  0  |  8
        
EType0:
        .byte zz___X____ ; 0
        .byte zz___X____ ; 1
        .byte zz__XXX___ ; 2
        .byte zz__XXX___ ; 3
        .byte zz__X_X___ ; 4
        .byte zz__X_X___ ; 5
        .byte zz__XXX___ ; 6
        .byte zz__XXX___ ; 7
        .byte zz_X_X_X__ ; 8
        .byte zz_X_X_X__ ; 9
        .byte zz_X_X_X__ ; 10
        .byte zz_X_X_X__ ; 11
        .byte zz_X_X_X__ ; 12
        .byte zz__XXX___ ; 13
        .byte zz__XXX___ ; 14
        .byte zz___X____ ; 15        
EType0Height = * - EType0

EType1:
        .byte zz__XX____ ; 0
        .byte zz_X__XX__ ; 1
        .byte zz_X__XXX_ ; 2
        .byte zzXXXXX_XX ; 3
        .byte zzXXXXX_XX ; 4
        .byte zz_X__XXX_ ; 5
        .byte zz_X__XX__ ; 6
        .byte zz__XX____ ; 7
EType1Height = * - EType1

IType0:
        .byte zz___X____ ; 0
        .byte zz___X____ ; 1
        .byte zz___X____ ; 2
        .byte zz___X____ ; 3
        .byte zz__XXX___ ; 4
        .byte zz__X_X___ ; 5
        .byte zz__X_X___ ; 6
        .byte zz__X_X___ ; 7
        .byte zz_XXXXX__ ; 8
        .byte zz_XXXXX__ ; 9
        .byte zzXXXXXXX_ ; 10
        .byte zzXXXXXXX_ ; 11
        .byte zz_XXXXX__ ; 12
        .byte zz__XXX___ ; 13
        .byte zz__X_X___ ; 14
        .byte zz__X_X___ ; 15
IType0Height = * - IType0

IType1:
        .byte zz__X_____ ; 0
        .byte zz_XX_____ ; 1
        .byte zz_XX_____ ; 2
        .byte zz_XXX____ ; 3
        .byte zzXXXXX___ ; 4
        .byte zzXXXXXX__ ; 5
        .byte zz_XX__XXX ; 6
        .byte zz_XX__XXX ; 7
        .byte zzXXXXXX__ ; 8
        .byte zzXXXXX___ ; 9
        .byte zz_XXX____ ; 10
        .byte zz_XX_____ ; 11
        .byte zz_XX_____ ; 12
        .byte zz__X_____ ; 13
IType1Height = * - IType1

PType0:
        .byte zz___X____ ; 0
        .byte zz__XXX___ ; 1
        .byte zz__X_X___ ; 2
        .byte zz_XX_XX__ ; 3
        .byte zz_XXXXX__ ; 4
        .byte zz_XXXXX__ ; 5
        .byte zzXXX_XXX_ ; 6
        .byte zzXX___XX_ ; 7
        .byte zzXX___XX_ ; 8
        .byte zzX_____X_ ; 9
        .byte zzX_____X_ ; 10
        .byte zzX_____X_ ; 11
        .byte zzX_____X_ ; 12
PType0Height = * - PType0
        
PType1:
        .byte zzXXX_____ ; 0
        .byte zz_XXX____ ; 1
        .byte zz__XXX___ ; 2
        .byte zz___XXX__ ; 3
        .byte zz___XXX__ ; 4
        .byte zz____XXX_ ; 5
        .byte zz____X_X_ ; 6
        .byte zz____X_X_ ; 7
        .byte zz____XXX_ ; 8
        .byte zz___XXX__ ; 9
        .byte zz___XXX__ ; 10
        .byte zz__XXX___ ; 11
        .byte zz_XXX____ ; 12
        .byte zzXXX_____ ; 13
PType1Height = * - PType1

Spy0:
        .byte zz___X____ ; 0
        .byte zz___X____ ; 1
        .byte zz__XXX___ ; 2
        .byte zz__XXX___ ; 3
        .byte zz_X_X_X__ ; 4
        .byte zz_X_X_X__ ; 5
        .byte zz_XXXXX__ ; 6
        .byte zz_XXXXX__ ; 7
        .byte zzXXXXXXX_ ; 8
        .byte zzXXXXXXX_ ; 9
        .byte zzXXXXXXX_ ; 10
        .byte zzXX_X_XX_ ; 11
        .byte zzXX_X_XX_ ; 12
        .byte zzX__X__X_ ; 13
        .byte zzX__X__X_ ; 14
        .byte zz___X____ ; 15
Spy0Height = * - Spy0

Spy1:
        .byte zzXXX_____ ; 0
        .byte zz_XXXX___ ; 1
        .byte zz__XXXX__ ; 2
        .byte zz___XXX__ ; 3
        .byte zz___X_XX_ ; 4
        .byte zz__XX_XX_ ; 5
        .byte zzXXXXXXXX ; 6
        .byte zzXXXXXXXX ; 7
        .byte zz__XX_XX_ ; 8
        .byte zz___X_XX_ ; 9
        .byte zz___XXX__ ; 10
        .byte zz__XXXX__ ; 11
        .byte zz_XXXX___ ; 12
        .byte zzXXX_____ ; 13
Spy1Height = * - Spy1

ShotHorizontal:
        .byte zzXXXX____
ShotHorizontalHeight = * - ShotHorizontal

ShotDiagonal:
        .byte zzXX______
        .byte zzXX______
ShotDiagonalHeight = * - ShotDiagonal

ShotVertical:
        .byte zzX_______
        .byte zzX_______
        .byte zzX_______
        .byte zzX_______
        .byte zzX_______
ShotVerticalHeight = * - ShotVertical   

ShotStation:
        .byte zzXX______
        .byte zzXX______
        .byte zzXX______
ShotStationHeight = * - ShotStation

ExplosionImage0:
        .byte zz________ ;-4
        .byte zz________ ;-3
        .byte zz________ ;-2
        .byte zz________ ;-1
        .byte zz___X____ ; 0
        .byte zz__X_X___ ; 1
        .byte zz___X____ ; 2
        .byte zz__X_X___ ; 3
        .byte zz___X____ ; 4
        .byte zz__X_X___ ; 5
        .byte zz___X____ ; 6
        .byte zz__X_X___ ; 7
        .byte zz___X____ ; 8
        .byte zz__X_X___ ; 9
        .byte zz___X____ ;10
        .byte zz__X_X___ ;11
        .byte zz___X____ ;12
;        .byte zz________ ;+1
;        .byte zz________ ;+2
;        .byte zz________ ;+3
;        .byte zz________ ;+4
ExplosionImage0Height = * - ExplosionImage0   

ExplosionImage1:
        .byte zz________ ;-4
        .byte zz________ ;-3
        .byte zz________ ;-2
        .byte zz__X_X___ ;-1
        .byte zz___X____ ; 0
        .byte zz_X___X__ ; 1
        .byte zz__X_X___ ; 2
        .byte zz___X_X__ ; 3
        .byte zz__X__X__ ; 4
        .byte zz___X____ ; 5
        .byte zz_X_X_X__ ; 6
        .byte zz__XXX___ ; 7
        .byte zz__X_X___ ; 8
        .byte zz_X__X___ ; 9
        .byte zz___XX___ ;10
        .byte zz__X_X___ ;11
        .byte zz__X_X___ ;12
        .byte zz___X____ ;+1
;        .byte zz________ ;+2
;        .byte zz________ ;+3
;        .byte zz________ ;+4
ExplosionImage1Height = * - ExplosionImage1
        
ExplosionImage2:
        .byte zz________ ;-4
        .byte zz________ ;-3
        .byte zz_X_X_X__ ;-2
        .byte zz________ ;-1
        .byte zz__X__X__ ; 0
        .byte zzX____X__ ; 1
        .byte zz__X__X__ ; 2
        .byte zz__X_X___ ; 3
        .byte zz______X_ ; 4
        .byte zz__X_X___ ; 5
        .byte zzX__X__X_ ; 6
        .byte zz__X_____ ; 7
        .byte zz_X__X___ ; 8
        .byte zz_X____X_ ; 9
        .byte zz__X__X__ ;10
        .byte zz_X__X___ ;11
        .byte zz__X___X_ ;12
        .byte zz__X__X__ ;+1
        .byte zz__X__X__ ;+2
        .byte zz___X____ ;+3
;        .byte zz________ ;+4         
ExplosionImage2Height = * - ExplosionImage2        
        
ExplosionImage3:
        .byte zz_X___X__ ;-4
        .byte zzX__X___X ;-3
        .byte zz________ ;-2
        .byte zz_X____X_ ;-1
        .byte zzX_____X_ ; 0
        .byte zz____X___ ; 1
        .byte zz_X____X_ ; 2
        .byte zz_X_____X ; 3
        .byte zz______X_ ; 4
        .byte zz__X___X_ ; 5
        .byte zz_______X ; 6
        .byte zz________ ; 7
        .byte zzX_______ ; 8
        .byte zz_X_____X ; 9
        .byte zz________ ;10
        .byte zzX_____X_ ;11
        .byte zz_______X ;12
        .byte zzX_____X_ ;+1
        .byte zz________ ;+2
        .byte zzX___X___ ;+3
        .byte zz_X___X__ ;+4  	
ExplosionImage3Height = * - ExplosionImage3

Pod:
        .byte zz__XXXX__ ; 0
        .byte zz__XXXX__ ; 1
        .byte zzXXXXXXXX ; 2
        .byte zzXXXXXXXX ; 3
        .byte zzXXXXXXXX ; 4
        .byte zzXXXXXXXX ; 5
        .byte zzXXXXXXXX ; 6
        .byte zzXXXXXXXX ; 7
        .byte zzXXXXXXXX ; 8
        .byte zzXXXXXXXX ; 9
        .byte zzXXXXXXXX ; 10
        .byte zzXXXXXXXX ; 11
        .byte zzXXXXXXXX ; 12
        .byte zz__XXXX__ ; 13
        .byte zz__XXXX__ ; 14
PodHeight = * - Pod

HorizontalEnergyPod:
        .byte zzXXXXXXXX ; 0
        .byte zzXXXXXXXX ; 1
        .byte zzXXXXXXXX ; 2
        .byte zzXXXXXXXX ; 3
        .byte zzXXXXXXXX ; 4
        .byte zzXXXXXXXX ; 5
        .byte zzXXXXXXXX ; 6
        .byte zzXXXXXXXX ; 7
        .byte zzXXXXXXXX ; 8
HorizontalEnergyPodHeight = * - HorizontalEnergyPod

VerticalEnergyPod:
        .byte zz__XXXX__ ; 0
        .byte zz__XXXX__ ; 1
        .byte zz__XXXX__ ; 2
        .byte zz__XXXX__ ; 3
        .byte zz__XXXX__ ; 4
        .byte zz__XXXX__ ; 5
        .byte zz__XXXX__ ; 6
        .byte zz__XXXX__ ; 7
        .byte zz__XXXX__ ; 8
        .byte zz__XXXX__ ; 9
        .byte zz__XXXX__ ; 10
        .byte zz__XXXX__ ; 11
        .byte zz__XXXX__ ; 12
VerticalEnergyPodHeight = * - VerticalEnergyPod


        echo "------",($4800 - *) , "bytes of ARM/Graphics space left"       
        
; these are tables that we don't want to move around when the ARM code changes
; otherwise syncing the 6507 and ARM code becomes convoluted
        ORG $4800 + HM_DRIVER_SIZE
        RORG $4800

Resp0StrobeTable:
        .word Resp0Strobe23   ; positions for  -1 -  14 
        .word Resp0Strobe28   ; positions for  14 -  29
        .word Resp0Strobe33   ; positions for  29 -  44
        .word Resp0Strobe38   ; positions for  44 -  59
        .word Resp0Strobe43   ; positions for  59 -  74
        .word Resp0Strobe48   ; positions for  74 -  89
        .word Resp0Strobe53   ; positions for  89 - 104
        .word Resp0Strobe58   ; positions for 104 - 119
        .word Resp0Strobe63   ; positions for 119 - 134
        .word Resp0Strobe68   ; positions for 134 - 149
        .word Resp0Strobe73   ; positions for 149 - 164

Resp1StrobeTable:
        .word Resp1Strobe23   ; positions for  -1 -  14 
        .word Resp1Strobe28   ; positions for  14 -  29
        .word Resp1Strobe33   ; positions for  29 -  44
        .word Resp1Strobe38   ; positions for  44 -  59
        .word Resp1Strobe43   ; positions for  59 -  74
        .word Resp1Strobe48   ; positions for  74 -  89
        .word Resp1Strobe53   ; positions for  89 - 104
        .word Resp1Strobe58   ; positions for 104 - 119
        .word Resp1Strobe63   ; positions for 119 - 134
        .word Resp1Strobe68   ; positions for 134 - 149
        .word Resp1Strobe73   ; positions for 149 - 164
        
Resm0StrobeTable:      
        .word Resm0Strobe23   ; positions for  -1 -  14 
        .word Resm0Strobe28   ; positions for  14 -  29
        .word Resm0Strobe33   ; positions for  29 -  44
        .word Resm0Strobe38   ; positions for  44 -  59
        .word Resm0Strobe43   ; positions for  59 -  74
        .word Resm0Strobe48   ; positions for  74 -  89
        .word Resm0Strobe53   ; positions for  89 - 104
        .word Resm0Strobe58   ; positions for 104 - 119
        .word Resm0Strobe63   ; positions for 119 - 134
        .word Resm0Strobe68   ; positions for 134 - 149
        .word Resm0Strobe73   ; positions for 149 - 164      

Resm1StrobeTable:      
        .word Resm1Strobe23   ; positions for  -1 -  14 
        .word Resm1Strobe28   ; positions for  14 -  29
        .word Resm1Strobe33   ; positions for  29 -  44
        .word Resm1Strobe38   ; positions for  44 -  59
        .word Resm1Strobe43   ; positions for  59 -  74
        .word Resm1Strobe48   ; positions for  74 -  89
        .word Resm1Strobe53   ; positions for  89 - 104
        .word Resm1Strobe58   ; positions for 104 - 119
        .word Resm1Strobe63   ; positions for 119 - 134
        .word Resm1Strobe68   ; positions for 134 - 149
        .word Resm1Strobe73   ; positions for 149 - 164      
        
        
ResblStrobeTable:      
        .word ResblStrobe23   ; positions for  -1 -  14 
        .word ResblStrobe28   ; positions for  14 -  29
        .word ResblStrobe33   ; positions for  29 -  44
        .word ResblStrobe38   ; positions for  44 -  59
        .word ResblStrobe43   ; positions for  59 -  74
        .word ResblStrobe48   ; positions for  74 -  89
        .word ResblStrobe53   ; positions for  89 - 104
        .word ResblStrobe58   ; positions for 104 - 119
        .word ResblStrobe63   ; positions for 119 - 134
        .word ResblStrobe68   ; positions for 134 - 149
        .word ResblStrobe73   ; positions for 149 - 164      
        
; put stations and explosions first in ImageGraphics: so we can limit
; the size of ImageShift:
ImageGraphics:
        .word StationHorizontal
        .word StationVertical
        .word ExplosionImage0
        .word ExplosionImage1
        .word ExplosionImage2
        .word ExplosionImage3
        .word AsteroidA
        .word AsteroidB
        .word AsteroidC
        .word Mine
        .word Ship0
        .word Ship1
        .word Ship2
        .word Ship3
        .word Ship4
        .word Ship5
        .word Ship6
        .word Ship7
        .word EType0
        .word EType1
        .word IType0
        .word IType1
        .word PType0
        .word PType1
        .word Spy0
        .word Spy1
        .word ShotHorizontal 
        .word ShotDiagonal
        .word ShotVertical   
        .word ShotStation       
        .word Pod
        .word HorizontalEnergyPod
        .word VerticalEnergyPod
        
ImageShift:
        .word Shift_StationHorizontal
        .word Shift_StationVertical
        ; .word Shift_Explosion_0
        ; .word Shift_Explosion_1
        ; .word ...
        
ImageHeights:
StationHorizontalID = * - ImageHeights
        .byte StationHorizontalHeight
StationVerticalID = * - ImageHeights
        .byte StationVerticalHeight
QuadSprites = * - ImageHeights - 1
ShiftingSprites = * - ImageHeights - 1
ExplosionID = * - ImageHeights
        .byte ExplosionImage0Height
        .byte ExplosionImage1Height
        .byte ExplosionImage2Height
        .byte ExplosionImage3Height
LastExplosionID = * - ImageHeights - 1        
DoubleSprites = * - ImageHeights - 1       


AsteroidID = * - ImageHeights
        .byte AsteroidAHeight
        .byte AsteroidBHeight
        .byte AsteroidCHeight
MineID = * - ImageHeights        
        .byte MineHeight
ShipID = * - ImageHeights
        .byte Ship0Height
        .byte Ship1Height
        .byte Ship2Height
        .byte Ship3Height
        .byte Ship4Height
        .byte Ship5Height
        .byte Ship6Height
        .byte Ship7Height
ETypeID = * - ImageHeights
        .byte EType0Height
        .byte EType1Height
ITypeID = * - ImageHeights
        .byte IType0Height
        .byte IType1Height
PTypeID = * - ImageHeights
        .byte PType0Height
        .byte PType1Height
SpyID = * - ImageHeights
        .byte Spy0Height
        .byte Spy1Height
ShotHorizontalID = * - ImageHeights
        .byte ShotHorizontalHeight
ShotDiagonalID = * - ImageHeights
        .byte ShotDiagonalHeight
ShotVerticalID = * - ImageHeights
        .byte ShotVerticalHeight
ShotStationID = * - ImageHeights
        .byte ShotStationHeight
PodID = * - ImageHeights
        .byte PodHeight
HorizontalEnergyPodID = * - ImageHeights        
        .byte HorizontalEnergyPodHeight
VerticalEnergyPodID = * - ImageHeights        
        .byte VerticalEnergyPodHeight
        
                
PFdigit0:
        .byte zz_____XXX ; 0
        .byte zz_____XXX ; 1
        .byte zz_____XXX ; 2
        .byte zz_____X_X ; 5
        .byte zz_____X_X ; 6
        .byte zz_____X_X ; 7
        .byte zz_____X_X ; 8
        .byte zz_____X_X ; 9
        .byte zz_____X_X ; 10
        .byte zz_____X_X ; 11
        .byte zz_____X_X ; 12
        .byte zz_____X_X ; 13
        .byte zz_____XXX ; 14
        .byte zz_____XXX ; 15
        .byte zz_____XXX ; 16
PFdigitHeight = * - PFdigit0      

PFdigit1:
        .byte zz_______X ; 0
        .byte zz_______X ; 1
        .byte zz_______X ; 2
        .byte zz_______X ; 3
        .byte zz_______X ; 4
        .byte zz_______X ; 5
        .byte zz_______X ; 6
        .byte zz_______X ; 7
        .byte zz_______X ; 10
        .byte zz_______X ; 11
        .byte zz_______X ; 12
        .byte zz_______X ; 13
        .byte zz_______X ; 14
        .byte zz_______X ; 15
        .byte zz_______X ; 16      
PFdigit2:
        .byte zz_____XXX ; 0
        .byte zz_____XXX ; 1
        .byte zz_____XXX ; 2
        .byte zz_______X ; 4
        .byte zz_______X ; 5
        .byte zz_______X ; 6
        .byte zz_____XXX ; 7
        .byte zz_____XXX ; 8
        .byte zz_____XXX ; 9
        .byte zz_____X__ ; 10
        .byte zz_____X__ ; 12
        .byte zz_____X__ ; 13
        .byte zz_____XXX ; 14
        .byte zz_____XXX ; 15
        .byte zz_____XXX ; 16    

PFdigit3:
        .byte zz_____XXX ; 0
        .byte zz_____XXX ; 1
        .byte zz_____XXX ; 2
        .byte zz_______X ; 3
        .byte zz_______X ; 5
        .byte zz_______X ; 6
        .byte zz_____XXX ; 7
        .byte zz_____XXX ; 8
        .byte zz_____XXX ; 9
        .byte zz_______X ; 10
        .byte zz_______X ; 12
        .byte zz_______X ; 13
        .byte zz_____XXX ; 14
        .byte zz_____XXX ; 15
        .byte zz_____XXX ; 16
        
PFdigit4:
        .byte zz_____X_X ; 0
        .byte zz_____X_X ; 1
        .byte zz_____X_X ; 2
        .byte zz_____X_X ; 3
        .byte zz_____X_X ; 5
        .byte zz_____X_X ; 6
        .byte zz_____XXX ; 7
        .byte zz_____XXX ; 8
        .byte zz_____XXX ; 9
        .byte zz_______X ; 10
        .byte zz_______X ; 12
        .byte zz_______X ; 13
        .byte zz_______X ; 14
        .byte zz_______X ; 15
        .byte zz_______X ; 16    
        
PFdigit5:
        .byte zz_____XXX ; 0
        .byte zz_____XXX ; 1
        .byte zz_____XXX ; 2
        .byte zz_____X__ ; 4
        .byte zz_____X__ ; 5
        .byte zz_____X__ ; 6
        .byte zz_____XXX ; 7
        .byte zz_____XXX ; 8
        .byte zz_____XXX ; 9
        .byte zz_______X ; 10
        .byte zz_______X ; 12
        .byte zz_______X ; 13
        .byte zz_____XXX ; 14
        .byte zz_____XXX ; 15
        .byte zz_____XXX ; 16        
        
PFdigit6:
        .byte zz_____X__ ; 0
        .byte zz_____X__ ; 1
        .byte zz_____X__ ; 2
        .byte zz_____X__ ; 4
        .byte zz_____X__ ; 5
        .byte zz_____X__ ; 6
        .byte zz_____XXX ; 7
        .byte zz_____XXX ; 8
        .byte zz_____XXX ; 9
        .byte zz_____X_X ; 11
        .byte zz_____X_X ; 12
        .byte zz_____X_X ; 13
        .byte zz_____XXX ; 14
        .byte zz_____XXX ; 15
        .byte zz_____XXX ; 16      
        
PFdigit7:
        .byte zz_____XXX ; 0
        .byte zz_____XXX ; 1
        .byte zz_____XXX ; 2
        .byte zz_______X ; 5
        .byte zz_______X ; 6
        .byte zz_______X ; 7
        .byte zz_______X ; 8
        .byte zz_______X ; 9
        .byte zz_______X ; 10
        .byte zz_______X ; 11
        .byte zz_______X ; 12
        .byte zz_______X ; 13
        .byte zz_______X ; 14
        .byte zz_______X ; 15
        .byte zz_______X ; 16        
        
PFdigit8:
        .byte zz_____XXX ; 0
        .byte zz_____XXX ; 1
        .byte zz_____XXX ; 2
        .byte zz_____X_X ; 3
        .byte zz_____X_X ; 4
        .byte zz_____X_X ; 6
        .byte zz_____XXX ; 7
        .byte zz_____XXX ; 8
        .byte zz_____XXX ; 9
        .byte zz_____X_X ; 11
        .byte zz_____X_X ; 12
        .byte zz_____X_X ; 13
        .byte zz_____XXX ; 14
        .byte zz_____XXX ; 15
        .byte zz_____XXX ; 16
        
PFdigit9:
        .byte zz_____XXX ; 0
        .byte zz_____XXX ; 1
        .byte zz_____XXX ; 2
        .byte zz_____X_X ; 3
        .byte zz_____X_X ; 5
        .byte zz_____X_X ; 6
        .byte zz_____XXX ; 7
        .byte zz_____XXX ; 8
        .byte zz_____XXX ; 9
        .byte zz_______X ; 10
        .byte zz_______X ; 12
        .byte zz_______X ; 13
        .byte zz_______X ; 14
        .byte zz_______X ; 15
        .byte zz_______X ; 16    
        
PFdigitA:
        .byte zz_____XXX ; 0
        .byte zz_____XXX ; 1
        .byte zz_____XXX ; 2
        .byte zz_____X_X ; 4
        .byte zz_____X_X ; 5
        .byte zz_____X_X ; 6
        .byte zz_____XXX ; 7
        .byte zz_____XXX ; 8
        .byte zz_____XXX ; 9
        .byte zz_____X_X ; 10
        .byte zz_____X_X ; 12
        .byte zz_____X_X ; 13
        .byte zz_____X_X ; 14
        .byte zz_____X_X ; 15
        .byte zz_____X_X ; 16

PFdigitB:
        .byte zz_____XX_ ; 0
        .byte zz_____XX_ ; 1
        .byte zz_____XX_ ; 2
        .byte zz_____X_X ; 4
        .byte zz_____X_X ; 5
        .byte zz_____X_X ; 6
        .byte zz_____XX_ ; 7
        .byte zz_____XX_ ; 8
        .byte zz_____XX_ ; 9
        .byte zz_____X_X ; 10
        .byte zz_____X_X ; 12
        .byte zz_____X_X ; 13
        .byte zz_____XX_ ; 14
        .byte zz_____XX_ ; 15
        .byte zz_____XX_ ; 16

PFdigitC:
        .byte zz_____XXX ; 0
        .byte zz_____XXX ; 1
        .byte zz_____XXX ; 2
        .byte zz_____X__ ; 5
        .byte zz_____X__ ; 6
        .byte zz_____X__ ; 7
        .byte zz_____X__ ; 8
        .byte zz_____X__ ; 9
        .byte zz_____X__ ; 10
        .byte zz_____X__ ; 11
        .byte zz_____X__ ; 12
        .byte zz_____X__ ; 13
        .byte zz_____XXX ; 14
        .byte zz_____XXX ; 15
        .byte zz_____XXX ; 16    
        
PFdigitD:
        .byte zz_____XX_ ; 0
        .byte zz_____XX_ ; 1
        .byte zz_____XX_ ; 2
        .byte zz_____X_X ; 3
        .byte zz_____X_X ; 4
        .byte zz_____X_X ; 5
        .byte zz_____X_X ; 8
        .byte zz_____X_X ; 9
        .byte zz_____X_X ; 10
        .byte zz_____X_X ; 11
        .byte zz_____X_X ; 12
        .byte zz_____X_X ; 13
        .byte zz_____XX_ ; 14
        .byte zz_____XX_ ; 15
        .byte zz_____XX_ ; 16

PFdigitE:
        .byte zz_____XXX ; 0
        .byte zz_____XXX ; 1
        .byte zz_____XXX ; 2
        .byte zz_____X__ ; 3
        .byte zz_____X__ ; 5
        .byte zz_____X__ ; 6
        .byte zz_____XXX ; 7
        .byte zz_____XXX ; 8
        .byte zz_____XXX ; 9
        .byte zz_____X__ ; 10
        .byte zz_____X__ ; 11
        .byte zz_____X__ ; 13
        .byte zz_____XXX ; 14
        .byte zz_____XXX ; 15
        .byte zz_____XXX ; 16        
        
PFdigitF:
        .byte zz_____XXX ; 0
        .byte zz_____XXX ; 1
        .byte zz_____XXX ; 2
        .byte zz_____X__ ; 3
        .byte zz_____X__ ; 5
        .byte zz_____X__ ; 6
        .byte zz_____XXX ; 7
        .byte zz_____XXX ; 8
        .byte zz_____XXX ; 9
        .byte zz_____X__ ; 10
        .byte zz_____X__ ; 12
        .byte zz_____X__ ; 13
        .byte zz_____X__ ; 14
        .byte zz_____X__ ; 15
        .byte zz_____X__ ; 16
        
PFdigitX:
        .byte zz_____X_X ; 0
        .byte zz_____X_X ; 1
        .byte zz_____X_X ; 2
        .byte zz_____X_X ; 4
        .byte zz_____X_X ; 5
        .byte zz_____X_X ; 6
        .byte zz______X_ ; 7
        .byte zz______X_ ; 8
        .byte zz______X_ ; 9
        .byte zz_____X_X ; 10
        .byte zz_____X_X ; 11
        .byte zz_____X_X ; 13
        .byte zz_____X_X ; 14
        .byte zz_____X_X ; 15
        .byte zz_____X_X ; 16       

PFdigitG:
        .byte zz_____XXX ; 0
        .byte zz_____XXX ; 1
        .byte zz_____XXX ; 2
        .byte zz_____X__ ; 3
        .byte zz_____X__ ; 5
        .byte zz_____X__ ; 6
        .byte zz_____X_X ; 7
        .byte zz_____X_X ; 8
        .byte zz_____X_X ; 10
        .byte zz_____X_X ; 11
        .byte zz_____X_X ; 12
        .byte zz_____X_X ; 13
        .byte zz_____XXX ; 14
        .byte zz_____XXX ; 15
        .byte zz_____XXX ; 16
        
PFdigitY
        .byte zz_____X_X ; 0
        .byte zz_____X_X ; 2
        .byte zz_____X_X ; 3
        .byte zz_____X_X ; 4
        .byte zz_____X_X ; 5
        .byte zz_____X_X ; 6
        .byte zz_____XXX ; 7
        .byte zz_____XXX ; 8
        .byte zz_____XXX ; 9
        .byte zz_______X ; 10
        .byte zz_______X ; 12
        .byte zz_______X ; 13
        .byte zz_____XXX ; 14
        .byte zz_____XXX ; 15
        .byte zz_____XXX ; 16

PFdigitR:
        .byte zz_____XX_ ; 0
        .byte zz_____XX_ ; 1
        .byte zz_____XX_ ; 2
        .byte zz_____X_X ; 3
        .byte zz_____X_X ; 5
        .byte zz_____X_X ; 6
        .byte zz_____XX_ ; 7
        .byte zz_____XX_ ; 8
        .byte zz_____XX_ ; 9
        .byte zz_____X_X ; 10
        .byte zz_____X_X ; 12
        .byte zz_____X_X ; 13
        .byte zz_____X_X ; 14
        .byte zz_____X_X ; 15
        .byte zz_____X_X ; 16       
        
PFdigitV:
        .byte zz_____X_X ; 0
        .byte zz_____X_X ; 1
        .byte zz_____X_X ; 2
        .byte zz_____X_X ; 3
        .byte zz_____X_X ; 4
        .byte zz_____X_X ; 7
        .byte zz_____X_X ; 8
        .byte zz_____X_X ; 9
        .byte zz_____X_X ; 10
        .byte zz_____X_X ; 11
        .byte zz_____X_X ; 12
        .byte zz_____X_X ; 13
        .byte zz______X_ ; 14
        .byte zz______X_ ; 15
        .byte zz______X_ ; 16   
        
PFdigitO:
        .byte zz________ ; 0
        .byte zz________ ; 2
        .byte zz________ ; 3
        .byte zz________ ; 4
        .byte zz________ ; 5
        .byte zz________ ; 6
        .byte zz_____XXX ; 7
        .byte zz_____XXX ; 8
        .byte zz_____XXX ; 9
        .byte zz_____X_X ; 11
        .byte zz_____X_X ; 12
        .byte zz_____X_X ; 13
        .byte zz_____XXX ; 14
        .byte zz_____XXX ; 15
        .byte zz_____XXX ; 16        
        
        echo "------",($5000 - *) , "bytes of ARM stationary data space left"
;----------------------------------------
; Bank 5
;----------------------------------------
;       InitSystem
;       Game Game
;----------------------------------------
        ORG $5000 + HM_DRIVER_SIZE
        RORG $F000
        ; can store 128 bytes of data here that's accessible to the ARM
        ; chip, but not the 6507.
        ds 1        ; dummy allocation so align 128 works
        echo "------",($F080 - *), " bytes free for BANK 5 'DPC+ overlap' ARM data storage"
        align 128 ; makes space for DPC+ registers
        
; Two Color Graphic routine
; this works by using a "negative" image when drawing the sprites.
; The sprites are colored black while the background is the color
; that shows up as though it were the sprites.  The 2nd color is
; achieved by using the playfield and/or ball.
;
; The missiles (also black) and setting the screen & playfield to BLACK
; is used to hide the screen and playfield beyond the 48 pixel image.
ShowTwoColorGraphic:
        sty LoopCounter

S2CGloop:
;---------------------------------------
        sta WSYNC
        dec Sleep5      ; 5  5
        lda #<DF2DATA   ; 2  7
        sta GRP0        ; 3 10
        lda #<DF3DATA   ; 2 12
        sta GRP1        ; 3 15
        lda #<DF4DATA   ; 2 17
        sta GRP0        ; 3 20
        ldx DF6DATA     ; 4 24
        ldy DF7DATA     ; 4 28
        lda #<DF1DATA   ; 2 30
        sta.w COLUPF    ; 4 34
        lda #<DF1DATA   ; 2 36
        sta COLUBK      ; 3 39
        lda #<DF5DATA   ; 2 41
        sta GRP1        ; 3 44
        stx GRP0        ; 3 47
        sty GRP1        ; 3 50
        sty GRP0        ; 3 53
        ldx #0          ; 2 55
        stx COLUBK      ; 3 58
        stx COLUPF      ; 3 61
        dec LoopCounter ; 5 66
        bne S2CGloop    ; 3 69      <- OK if page crossed
        stx PF0         ; 3 72
        stx PF1         ; 3 75
;---------------------------------------
        stx GRP0        ; 3 78/2
        stx GRP1        ; 3  5
        stx GRP0        ; 3  8
        rts             ; 6 14

PosObject:              ; A holds X value
        sec             ; 2  
        sta WSYNC       ; X holds object, 0=P0, 1=P1, 2=M0, 3=M1, 4=Ball
DivideLoop
        sbc #15         ; 2  
        bcs DivideLoop  ; 2  4
        eor #7          ; 2  6
        asl             ; 2  8
        asl             ; 2 10
        asl             ; 2 12
        asl             ; 2 14
        sta.wx HMP0,X   ; 5 19
        sta RESP0,X     ; 4 23 <- set object position
SLEEP12: rts             ; 6 29
        
InitSystem:             ; DPC+ always boots in bank 5, no need to select it
        CLEAN_START
        stx FASTFETCH   ; X=0 turns on fast fetch mode
        ; call ARM Int() here
        jsr CallArmSub  ; X=0, 
        jmp VerticalBlank
        
CallArmSub:
        ldy #<ARMsub
        sty DF0LOW
        ldy #>ARMsub
        sty DF0HI
        stx DF0WRITE
        ldy #$FF                ; $FF = w/out digitized audio
        sty CALLFUNCTION        ; triggers ARM routine
        rts       
        
OverScan:
        tsx                     ; X = $ff
        stx CALLFUNCTION
        lda #<DF0DATA
        sta Mode
        
        lda INTIM
        sta TimeLeftOS        
OSwait:
        sta WSYNC
        lda INTIM
        bpl OSwait
        
VerticalBlank:
        ldx #2
        stx WSYNC
        stx VSYNC
        stx VBLANK
        ldy #$2B + 6 + 4
        sty TIM64T
        ldx #0          ; 2 12
AudioLoop:        
        lda #<DF0DATA   ; 2  2
        sta AUDC0,x     ; 4  6
        inx             ; 2  8
        cpx #6          ; 2 10
        bne AudioLoop   ; 3 13 * 6 - 1 = 77  89/13        
;        sta WSYNC       ; line 1 of VSYNC AUDC0 loop runs longer, so remove a WSYNC
        inc Frame
        sta WSYNC       ; line 2 of VSYNC
        ldx #0
ZERO = *-1              ; used to load A with 0 as we can't use LDA #0        
        stx GRP0
        stx ENABL
        stx ENAM1
        stx ENAM0
        stx PF0
        stx PF1
        stx PF2
        stx GRP1
        stx COLUBK
        stx COLUPF
        stx CTRLPF
        stx VDELP0
        stx VDELP1
        stx WSYNC       ; line 3 of VSYNC
        stx VSYNC       ; turn off VSYNC
        
        ; call ARM VerticalBlank() here
        ldx Mode
        beq VBM
        ldx #2          ; 2 = run VerticalBlank();
        .byte $2c
VBM:    ldx #4          ; 4 = run VerticalBlankMenu()        
        jsr CallArmSub
        lda Mode
        bne Game
        jmp MainMenu
        
Game:   ldx #3          
        stx VDELP0  ; turn on vertical delay for player 0
        stx VDELBL  ; turn on vertical delay for ball

PositionAll:        
        stx HMCLR
        ldx #<BallX
        stx DF0LOW
        ldx #>BallX
        stx DF0HI       
        ldx #4
PAloop:    
        lda #<DF0DATA     
        jsr PosObject
        dex
        bpl PAloop
        lda #<DF0DATA
        sta NUSIZ0        ; set player/missile 0 sizes
        lda #<DF0DATA
        sta NUSIZ1        ; set player/missile 1 sizes
        lda #<DF0DATA
        sta COLUP0        ; first sprite color
        lda #<DF0DATA
        sta COLUP1        ; first sprite color
        lda #<DF0DATA
        sta COLUPF        ; first star color
        sta WSYNC
        sta HMOVE        
        
        ldx #7
InitDS: lda InitDSlow,x
        sta DF0LOW,x        
        lda InitDShi,x
        sta DF0HI,x
        dex
        bpl InitDS
        
        ; x now 255
        ldy #7
InitPF: lda InitPFlow-1,y
        sta DF0FRACLOW-1,y
        lda InitPFhi-1,y
        sta DF0FRACHI-1,y
        txa
        sta DF0FRACINC-1,y
        lda DF0FRACDATA-1,y   ; priming read, required when using fractional data
        dey                 ; fetchers with 1 increment per read
        bne InitPF
        ; y now 0, ready for use after VBwait       
        
        lda INTIM
        sta TimeLeftVB           
VBwait:
        sta WSYNC
        lda INTIM               ; 4  4
        bpl VBwait              ; 2  6
        sty VBLANK              ; 3  9
                
NormalKernel:                   ;   19  
        lda #<DS_NUSIZ_COLUPF   ; 2 21 ' resize players/missiles, color for ball
        lda #<DS_COLUP_HMMB     ; 2 23 ' use for missile/ball HMxx
        ldx DS_GRP1             ; 4 27
        lda #<DS_JUMP           ; 2 29
        sta NextKernel          ; 3 32
        lda #<DS_JUMP           ; 2 34
        sta NextKernel+1        ; 3 37
        sta HMCLR               ; 3 40 ' reset missile/ball HMOVE
        lda #<DS_HMP0           ; 2 42
        sta HMP0                ; 3 45
        lda #<DS_HMP1           ; 2 47
        sta HMP1                ; 3 50
        lda #<DS_GRP0           ; 2 52
        sta GRP0                ; 3 55 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 57 <- on VDEL, for next scanline
        sta ENABL               ; 3 60
        lsr                     ; 2 62 
        sta WSYNC               ; 3 65/0
R76:        
        ; A holds M0 and M1
        ; X holds GRP1
        ; GRP0 and BL on VDEL
        sta HMOVE               ; 3  3
        stx GRP1                ; 3  6
        sta ENAM0               ; 3  9
        lsr                     ; 2 11
        sta ENAM1               ; 3 14
        jmp (NextKernel)        ; 5 19
        
;--------------------------------------------------------------        
; KernelDone displays score, radar, reserve lives and stats
;--------------------------------------------------------------
 if (* & $ff)
    echo "------", ($100 - (* & $ff)), "bytes free before KernelDone"
    align 256
 endif
        
KernelDone:                     ;   19 - prep for blank line before condition color line
        lda #<DS_GRP0           ; 2 21 - should be 0
        sta GRP0                ; 3 24 - on VDEL, turns player0 off on next line
        sta ENABL               ; 3 27 - on VDEL, turns ball off on next line
        jsr SLEEP12             ;12 39
        sta HMCLR               ; 3 42
        ldy #HMOVE_R4           ; 2 44
        sty HMP0                ; 3 47
        ldy #HMOVE_L1           ; 2 49
        sty HMP1                ; 3 52
        SLEEP 4                 ; 4 56
        sta RESP0               ; 3 59 - need to adjust cycle time 108
        sta RESP1               ; 3 62 - need to adjust cycle time 117
        sta WSYNC               ; 3 76/0
        
        ; blank line before condition color line
        lda #<DS_GRP1           ; 2  2 - should be 0
        sta GRP1                ; 3  5
        sta ENAM0               ; 3  8
        sta ENAM1               ; 3 11  
        sta NUSIZ0              ; 3 14
        sta NUSIZ1              ; 3 17
        ldy #HMOVE_R7           ; 2 19
        sty HMBL                ; 3 22
        ldx DS_RADAR_CONTROL    ; 4 26 - X holds status color
        stx COLUPF              ; 3 29 - color of status line
        stx COLUP0              ; 3 32 - color of radar frame
        stx COLUP1              ; 3 35 - color of radar frame
        jsr SLEEP12             ;12 47
        ldy #4                  ; 2 49 
        sty CTRLPF              ; 3 52 - show ball over player
        SLEEP 4
        sta RESBL               ; 3 55 - need to adjust cycle time 101
        sta WSYNC               ; 3 76/0
        
        ; condition line color
        sta HMOVE               ; 3  3
        ldy #zr______XX         ; 2  5
        sty PF0                 ; 3  8
        ldy #zzXXXXXXXX         ; 2 10
        sty PF1                 ; 3 13
        sty PF2                 ; 3 16
        jsr SLEEP12             ;12 28
        sty PF0                 ; 3 31 - PF0R, 28-49
        sta WSYNC               ; 3 76/0
        
        ; first line of top radar frame
        ldy #0                  ; 2  2
        sty PF0                 ; 3  5
        sty PF1                 ; 3  8
        sty PF2                 ; 3 11
        lda #<DS_GRP0           ; 2 13        
        sta GRP0                ; 3 16
        lda #<DS_GRP1           ; 2 18
        sta GRP1                ; 3 21
        ldy #RADAR_HEIGHT - 5   ; 2 23
        sty LoopCounter         ; 3 26
        lda #<DS_RADAR_CONTROL  ; 2 28 - position radar
        sta HMBL                ; 3 31        
        sta WSYNC
        
        ; second line of top radar frame
        lda #<DS_GRP0           ; 2  2
        sta GRP0                ; 3  5
        lda #<DS_GRP1           ; 2  7
        sta GRP1                ; 3 10
        ldy #HMOVE_R2           ; 2 12
        sty HMP0                ; 3 15
        ldy #HMOVE_L2           ; 2 17
        sty HMP1                ; 3 20
        ldy #WHITE              ; 2 22
        sty COLUPF              ; 3 25        
        sta WSYNC               ; 
        
        ; first line of score/radar
        sta HMOVE               ; 3  3        
        lda #<DS_PF0L           ; 2  5
        sta PF0                 ; 3  8
        sta ENABL               ; 3 11 - on VDEL
        lda #<DS_PF1L           ; 2 13
        sta PF1                 ; 3 16
        lda #<DS_PF2L           ; 2 18
        sta PF2                 ; 3 21
        lda #<DS_GRP0           ; 2 23
        sta GRP0                ; 3 26 - on VDEL
        lda #<DS_GRP1           ; 2 28
        sta GRP1                ; 3 31 - updates GRP0 and BL as well
        lda #<DS_RADAR_CONTROL  ; 2 33
        sta COLUP0              ; 3 36 - enemy color in radar
        lda #<DS_RADAR_CONTROL  ; 2 38
        sta COLUP1              ; 3 41 - station color in radar
        lda #<DS_PF0R           ; 2 43
        sta PF0                 ; 3 46 - PF0R, 28-49
        lda #<DS_PF1R           ; 2 48
        sta PF1                 ; 3 51
        lda #<DS_PF2R           ; 2 53
        sta PF2                 ; 3 56 
        ldy #HMOVE_L2           ; 2 58 - prep to shift players for
        sty HMP0                ; 3 61 - frame after score is displayed
        ldy #HMOVE_R2           ; 2 63
        sty HMP1                ; 3 66
        
        
ScoreLoop:
        sta WSYNC
        lda #<DS_PF0L           ; 2  2
        sta PF0                 ; 3  5
        sta ENABL               ; 3  8 - on VDEL
        lda #<DS_PF1L           ; 2 10
        sta PF1                 ; 3 13
        lda #<DS_PF2L           ; 2 15
        sta PF2                 ; 3 18
        lda #<DS_GRP0           ; 2 20
        sta GRP0                ; 3 23 - on VDEL
        lda #<DS_GRP1           ; 2 25
        sta GRP1                ; 3 28 - updates GRP0 and BL as well
        lda #<DS_PF0R           ; 2 30
        sta PF0                 ; 3 33 - PF0R, 28-49
        jsr SLEEP12             ;12 45
        lda #<DS_PF1R           ; 2 47
        sta PF1                 ; 3 50 - PF1R, 39-54
        lda #<DS_PF2R           ; 2 52
        sta PF2                 ; 3 55 - PF2R, 50-65
        dec LoopCounter         ; 5 60
        bne ScoreLoop           ; 2 62
        
        ; first line of bottom radar frame
        sta WSYNC
        sta HMOVE
        ldy #0
        sty PF0
        sty PF1
        sty PF2
        sty ENABL
        lda #<DS_GRP0
        sta GRP0
        lda #<DS_GRP1
        sta GRP1
        stx COLUP0
        stx COLUP1
        
        ; second line of bottom radar frame
        sta WSYNC
        lda #<DS_GRP0
        sta GRP0
        lda #<DS_GRP1
        sta GRP1


SetOStimer:        
        ldx #$2a - 4
        sta WSYNC
        ldy #2
        sty VBLANK
        stx TIM64T
        ; prep for call to OverScan()
        ldx #<ARMsub
        stx DF0LOW
        ldx #>ARMsub
        stx DF0HI
        ldx Mode
        beq OSM
        ldx #1              ; 1 = OverScan()
        .byte $2c           ; $2c = BIT $xxxx - skips over LDX #3
OSM     ldx #3              ; 3 = OverScanMenu()
        stx DF0WRITE
        ldx SWCHA
        stx DF0WRITE
        ldx SWCHB
        stx DF0WRITE
        ldx INPT4
        stx DF0WRITE        
        
        ldx TimeLeftVB          ; 3
        stx DF0WRITE            ; 4
        ldx TimeLeftOS          ; 3
        stx DF0WRITE            ; 4
        
        jmp OverScan

        
        
;----------------------------------------
; Reposition Player0 Kernels
;   all must be consecutive with
;   __Strobe23 first and __Strobe73 last
;----------------------------------------
Resp0Strobe23:                  ;   19
        sta.w RESP0             ; 4 23
        lda #<DS_NUSIZ_COLUPF   ; 2 25
        sta NUSIZ0              ; 3 28
        lda #<DS_COLUP_HMMB     ; 2 30
        sta COLUP0              ; 3 33
        ldx DS_GRP1             ; 4 37
        lda #<DS_JUMP           ; 2 39
        sta NextKernel          ; 3 42
        lda #<DS_JUMP           ; 2 44
        sta NextKernel+1        ; 3 47
        sta HMCLR               ; 3 50 ' reset missile/ball HMOVE
        lda #<DS_HMP0           ; 2 52
        sta HMP0                ; 3 55
        lda #<DS_HMP1           ; 2 57
        sta HMP1                ; 3 60
        lda #<DS_GRP0           ; 2 62
        sta GRP0                ; 3 65 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 67 <- on VDEL, for next scanline
        sta.w ENABL             ; 4 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76
Resp0Strobe28:                  ;   19
        lda #<DS_NUSIZ_COLUPF   ; 2 21 
        sta NUSIZ0              ; 3 24
        sta.w RESP0             ; 4 28
        lda #<DS_COLUP_HMMB     ; 2 30
        sta COLUP0              ; 3 33
        ldx DS_GRP1             ; 4 37
        lda #<DS_JUMP           ; 2 39
        sta NextKernel          ; 3 42
        lda #<DS_JUMP           ; 2 44
        sta NextKernel+1        ; 3 47
        sta HMCLR               ; 3 50 ' reset missile/ball HMOVE
        lda #<DS_HMP0           ; 2 52
        sta HMP0                ; 3 55
        lda #<DS_HMP1           ; 2 57
        sta HMP1                ; 3 60
        lda #<DS_GRP0           ; 2 62
        sta GRP0                ; 3 65 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 67 <- on VDEL, for next scanline
        sta.w ENABL             ; 4 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76
        
Resp0Strobe33:                  ;   19
        lda #<DS_NUSIZ_COLUPF   ; 2 21 
        sta NUSIZ0              ; 3 24
        lda #<DS_COLUP_HMMB     ; 2 26
        sta COLUP0              ; 3 29
        sta.w RESP0             ; 4 33
        ldx DS_GRP1             ; 4 37
        lda #<DS_JUMP           ; 2 39
        sta NextKernel          ; 3 42
        lda #<DS_JUMP           ; 2 44
        sta NextKernel+1        ; 3 47
        sta HMCLR               ; 3 50 ' reset missile/ball HMOVE
        lda #<DS_HMP0           ; 2 52
        sta HMP0                ; 3 55
        lda #<DS_HMP1           ; 2 57
        sta HMP1                ; 3 60
        lda #<DS_GRP0           ; 2 62
        sta GRP0                ; 3 65 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 67 <- on VDEL, for next scanline
        sta.w ENABL             ; 4 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76

Resp0Strobe38:                  ;   19
        lda #<DS_NUSIZ_COLUPF   ; 2 21 
        sta NUSIZ0              ; 3 24
        lda #<DS_COLUP_HMMB     ; 2 26
        sta COLUP0              ; 3 29
        ldx DS_GRP1             ; 4 33
        lda #<DS_JUMP           ; 2 35
        sta RESP0               ; 3 38
        sta NextKernel          ; 3 41
        lda #<DS_JUMP           ; 2 43
        sta NextKernel+1        ; 3 46
        sta HMCLR               ; 3 49 ' reset missile/ball HMOVE
        lda #<DS_HMP0           ; 2 51
        sta HMP0                ; 3 54
        lda #<DS_HMP1           ; 2 56
        sta HMP1                ; 3 59
        lda #<DS_GRP0           ; 2 61
        sta GRP0                ; 3 64 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 66 <- on VDEL, for next scanline
        sta ENABL               ; 3 69
        lsr                     ; 2 71
        SLEEP 2                 ; 2 73
        jmp R76                 ; 3 76

Resp0Strobe43:                  ;   19
        lda #<DS_NUSIZ_COLUPF   ; 2 21 
        sta NUSIZ0              ; 3 24
        lda #<DS_COLUP_HMMB     ; 2 26
        sta COLUP0              ; 3 29
        ldx DS_GRP1             ; 4 33
        lda #<DS_JUMP           ; 2 35
        sta NextKernel          ; 3 38
        lda #<DS_JUMP           ; 2 40
        sta RESP0               ; 3 43
        sta NextKernel+1        ; 3 46
        sta HMCLR               ; 3 49 ' reset missile/ball HMOVE
        lda #<DS_HMP0           ; 2 51
        sta HMP0                ; 3 54
        lda #<DS_HMP1           ; 2 56
        sta HMP1                ; 3 59
        lda #<DS_GRP0           ; 2 61
        sta GRP0                ; 3 64 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 66 <- on VDEL, for next scanline
        sta ENABL               ; 3 69
        lsr                     ; 2 71
        SLEEP 2                 ; 2 73
        jmp R76                 ; 3 76
        
Resp0Strobe48:                  ;   19
        lda #<DS_NUSIZ_COLUPF   ; 2 21 
        sta NUSIZ0              ; 3 24
        lda #<DS_COLUP_HMMB     ; 2 26
        sta COLUP0              ; 3 29
        ldx DS_GRP1             ; 4 33
        lda #<DS_JUMP           ; 2 35
        sta NextKernel          ; 3 38
        lda #<DS_JUMP           ; 2 40
        sta NextKernel+1        ; 3 43
        lda #<DS_HMP0           ; 2 45
        sta RESP0               ; 3 48
        sta HMCLR               ; 3 51
        sta HMP0                ; 3 54
        lda #<DS_HMP1           ; 2 56
        sta HMP1                ; 3 59
        lda #<DS_GRP0           ; 2 61
        sta GRP0                ; 3 64 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 66 <- on VDEL, for next scanline
        sta ENABL               ; 3 69
        lsr                     ; 2 71
        SLEEP 2                 ; 2 73
        jmp R76                 ; 3 76
        
Resp0Strobe53:                  ;   19
        lda #<DS_NUSIZ_COLUPF   ; 2 21 
        sta NUSIZ0              ; 3 24
        lda #<DS_COLUP_HMMB     ; 2 26
        sta COLUP0              ; 3 29
        ldx DS_GRP1             ; 4 33
        lda #<DS_JUMP           ; 2 35
        sta NextKernel          ; 3 38
        lda #<DS_JUMP           ; 2 40
        sta NextKernel+1        ; 3 43
        sta HMCLR               ; 3 46
        SLEEP 2                 ; 2 48
        lda #<DS_HMP0           ; 2 50
        sta RESP0               ; 3 53
        sta HMP0                ; 3 56
        lda #<DS_HMP1           ; 2 58
        sta HMP1                ; 3 61
        lda #<DS_GRP0           ; 2 63
        sta GRP0                ; 3 66 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 68 <- on VDEL, for next scanline
        sta ENABL               ; 3 71
        lsr                     ; 2 73
        jmp R76                 ; 3 76
        
Resp0Strobe58:                  ;   19
        lda #<DS_NUSIZ_COLUPF   ; 2 21 
        sta NUSIZ0              ; 3 24
        lda #<DS_COLUP_HMMB     ; 2 26
        sta COLUP0              ; 3 29
        ldx DS_GRP1             ; 4 33
        lda #<DS_JUMP           ; 2 35
        sta NextKernel          ; 3 38
        lda #<DS_JUMP           ; 2 40
        sta NextKernel+1        ; 3 43
        sta HMCLR               ; 3 46
        SLEEP 2                 ; 2 48
        lda #<DS_HMP0           ; 2 50
        sta HMP0                ; 3 53
        lda #<DS_HMP1           ; 2 55
        sta RESP0               ; 3 58        
        sta HMP1                ; 3 61
        lda #<DS_GRP0           ; 2 63
        sta GRP0                ; 3 66 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 68 <- on VDEL, for next scanline
        sta ENABL               ; 3 71
        lsr                     ; 2 73
        jmp R76                 ; 3 76
        
Resp0Strobe63:                  ;   19
        lda #<DS_NUSIZ_COLUPF   ; 2 21 
        sta NUSIZ0              ; 3 24
        lda #<DS_COLUP_HMMB     ; 2 26
        sta COLUP0              ; 3 29
        ldx DS_GRP1             ; 4 33
        lda #<DS_JUMP           ; 2 35
        sta NextKernel          ; 3 38
        lda #<DS_JUMP           ; 2 40
        sta NextKernel+1        ; 3 43
        sta HMCLR               ; 3 46
        SLEEP 2                 ; 2 48
        lda #<DS_HMP0           ; 2 50
        sta HMP0                ; 3 53
        lda #<DS_HMP1           ; 2 55
        sta HMP1                ; 3 58
        lda #<DS_GRP0           ; 2 60
        sta RESP0               ; 3 63        
        sta GRP0                ; 3 66
        lda #<DS_M1M0BL         ; 2 68
        sta ENABL               ; 3 71
        lsr                     ; 2 73
        jmp R76                 ; 3 76
        
Resp0Strobe68:                  ;   19
        lda #<DS_NUSIZ_COLUPF   ; 2 21 
        sta NUSIZ0              ; 3 24
        lda #<DS_COLUP_HMMB     ; 2 26
        sta COLUP0              ; 3 29
        ldx DS_GRP1             ; 4 33
        lda #<DS_JUMP           ; 2 35
        sta NextKernel          ; 3 38
        lda #<DS_JUMP           ; 2 40
        sta NextKernel+1        ; 3 43
        sta HMCLR               ; 3 46
        SLEEP 2                 ; 2 48
        lda #<DS_HMP0           ; 2 50
        sta HMP0                ; 3 53
        lda #<DS_HMP1           ; 2 55
        sta HMP1                ; 3 58
        lda #<DS_GRP0           ; 2 60
        sta GRP0                ; 3 63
        lda #<DS_M1M0BL         ; 2 65
        sta RESP0               ; 3 68
        sta ENABL               ; 3 71
        lsr                     ; 2 73
        jmp R76                 ; 3 76

Resp0Strobe73:                  ;   19
        lda #<DS_NUSIZ_COLUPF   ; 2 21 
        sta NUSIZ0              ; 3 24
        lda #<DS_COLUP_HMMB     ; 2 26
        sta COLUP0              ; 3 29
        ldx DS_GRP1             ; 4 33
        lda #<DS_JUMP           ; 2 35
        sta NextKernel          ; 3 38
        lda #<DS_JUMP           ; 2 40
        sta NextKernel+1        ; 3 43
        sta HMCLR               ; 3 46
        SLEEP 2                 ; 2 48
        lda #<DS_HMP0           ; 2 50
        sta HMP0                ; 3 53
        lda #<DS_HMP1           ; 2 55
        sta HMP1                ; 3 58
        lda #<DS_GRP0           ; 2 60
        sta GRP0                ; 3 63
        lda #<DS_M1M0BL         ; 2 65
        sta ENABL               ; 3 68
        lsr                     ; 2 70
        sta RESP0               ; 3 73
        jmp R76                 ; 3 76
        
;----------------------------------------
; Reposition Player1 Kernels
;   all must be consecutive with
;   __Strobe23 first and __Strobe73 last
; MUST FOLLOW Player0 Kernels
;----------------------------------------
Resp1Strobe23:                  ;   19
        sta.w RESP1             ; 4 23
        lda #<DS_NUSIZ_COLUPF   ; 2 25
        sta NUSIZ1              ; 3 28
        lda #<DS_COLUP_HMMB     ; 2 30
        sta COLUP1              ; 3 33
        ldx DS_GRP1             ; 4 37
        lda #<DS_JUMP           ; 2 39
        sta NextKernel          ; 3 42
        lda #<DS_JUMP           ; 2 44
        sta NextKernel+1        ; 3 47
        sta HMCLR               ; 3 50 ' reset missile/ball HMOVE
        lda #<DS_HMP0           ; 2 52
        sta HMP0                ; 3 55
        lda #<DS_HMP1           ; 2 57
        sta HMP1                ; 3 60
        lda #<DS_GRP0           ; 2 62
        sta GRP0                ; 3 65 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 67 <- on VDEL, for next scanline
        sta.w ENABL             ; 4 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76
        
Resp1Strobe28:                  ;   19
        lda #<DS_NUSIZ_COLUPF   ; 2 21 
        sta NUSIZ1              ; 3 24
        sta.w RESP1             ; 4 28
        lda #<DS_COLUP_HMMB     ; 2 30
        sta COLUP1              ; 3 33
        ldx DS_GRP1             ; 4 37
        lda #<DS_JUMP           ; 2 39
        sta NextKernel          ; 3 42
        lda #<DS_JUMP           ; 2 44
        sta NextKernel+1        ; 3 47
        sta HMCLR               ; 3 50 ' reset missile/ball HMOVE
        lda #<DS_HMP0           ; 2 52
        sta HMP0                ; 3 55
        lda #<DS_HMP1           ; 2 57
        sta HMP1                ; 3 60
        lda #<DS_GRP0           ; 2 62
        sta GRP0                ; 3 65 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 67 <- on VDEL, for next scanline
        sta.w ENABL             ; 4 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76
        
Resp1Strobe33:                  ;   19
        lda #<DS_NUSIZ_COLUPF   ; 2 21 
        sta NUSIZ1              ; 3 24
        lda #<DS_COLUP_HMMB     ; 2 26
        sta COLUP1              ; 3 29
        sta.w RESP1             ; 4 33
        ldx DS_GRP1             ; 4 37
        lda #<DS_JUMP           ; 2 39
        sta NextKernel          ; 3 42
        lda #<DS_JUMP           ; 2 44
        sta NextKernel+1        ; 3 47
        sta HMCLR               ; 3 50 ' reset missile/ball HMOVE
        lda #<DS_HMP0           ; 2 52
        sta HMP0                ; 3 55
        lda #<DS_HMP1           ; 2 57
        sta HMP1                ; 3 60
        lda #<DS_GRP0           ; 2 62
        sta GRP0                ; 3 65 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 67 <- on VDEL, for next scanline
        sta.w ENABL             ; 4 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76

Resp1Strobe38:                  ;   19
        lda #<DS_NUSIZ_COLUPF   ; 2 21 
        sta NUSIZ1              ; 3 24
        lda #<DS_COLUP_HMMB     ; 2 26
        sta COLUP1              ; 3 29
        ldx DS_GRP1             ; 4 33
        lda #<DS_JUMP           ; 2 35
        sta RESP1               ; 3 38
        sta NextKernel          ; 3 41
        lda #<DS_JUMP           ; 2 43
        sta NextKernel+1        ; 3 46
        sta HMCLR               ; 3 49 ' reset missile/ball HMOVE
        lda #<DS_HMP0           ; 2 51
        sta HMP0                ; 3 54
        lda #<DS_HMP1           ; 2 56
        sta HMP1                ; 3 59
        lda #<DS_GRP0           ; 2 61
        sta GRP0                ; 3 64 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 66 <- on VDEL, for next scanline
        sta ENABL               ; 3 69
        lsr                     ; 2 71
        SLEEP 2                 ; 2 73
        jmp R76                 ; 3 76

Resp1Strobe43:                  ;   19
        lda #<DS_NUSIZ_COLUPF   ; 2 21 
        sta NUSIZ1              ; 3 24
        lda #<DS_COLUP_HMMB     ; 2 26
        sta COLUP1              ; 3 29
        ldx DS_GRP1             ; 4 33
        lda #<DS_JUMP           ; 2 35
        sta NextKernel          ; 3 38
        lda #<DS_JUMP           ; 2 40
        sta RESP1               ; 3 43
        sta NextKernel+1        ; 3 46
        sta HMCLR               ; 3 49 ' reset missile/ball HMOVE
        lda #<DS_HMP0           ; 2 51
        sta HMP0                ; 3 54
        lda #<DS_HMP1           ; 2 56
        sta HMP1                ; 3 59
        lda #<DS_GRP0           ; 2 61
        sta GRP0                ; 3 64 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 66 <- on VDEL, for next scanline
        sta ENABL               ; 3 69
        lsr                     ; 2 71
        SLEEP 2                 ; 2 73
        jmp R76                 ; 3 76
        
Resp1Strobe48:                  ;   19
        lda #<DS_NUSIZ_COLUPF   ; 2 21 
        sta NUSIZ1              ; 3 24
        lda #<DS_COLUP_HMMB     ; 2 26
        sta COLUP1              ; 3 29
        ldx DS_GRP1             ; 4 33
        lda #<DS_JUMP           ; 2 35
        sta NextKernel          ; 3 38
        lda #<DS_JUMP           ; 2 40
        sta NextKernel+1        ; 3 43
        lda #<DS_HMP0           ; 2 45
        sta RESP1               ; 3 48
        sta HMCLR               ; 3 51
        sta HMP0                ; 3 54
        lda #<DS_HMP1           ; 2 56
        sta HMP1                ; 3 59
        lda #<DS_GRP0           ; 2 61
        sta GRP0                ; 3 64 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 66 <- on VDEL, for next scanline
        sta ENABL               ; 3 69
        lsr                     ; 2 71
        SLEEP 2                 ; 2 73
        jmp R76                 ; 3 76
        
Resp1Strobe53:                  ;   19
        lda #<DS_NUSIZ_COLUPF   ; 2 21 
        sta NUSIZ1              ; 3 24
        lda #<DS_COLUP_HMMB     ; 2 26
        sta COLUP1              ; 3 29
        ldx DS_GRP1             ; 4 33
        lda #<DS_JUMP           ; 2 35
        sta NextKernel          ; 3 38
        lda #<DS_JUMP           ; 2 40
        sta NextKernel+1        ; 3 43
        sta HMCLR               ; 3 46
        SLEEP 2                 ; 2 48
        lda #<DS_HMP0           ; 2 50
        sta RESP1               ; 3 53
        sta HMP0                ; 3 56
        lda #<DS_HMP1           ; 2 58
        sta HMP1                ; 3 61
        lda #<DS_GRP0           ; 2 63
        sta GRP0                ; 3 66 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 68 <- on VDEL, for next scanline
        sta ENABL               ; 3 71
        lsr                     ; 2 73
        jmp R76                 ; 3 76
        
Resp1Strobe58:                  ;   19
        lda #<DS_NUSIZ_COLUPF   ; 2 21 
        sta NUSIZ1              ; 3 24
        lda #<DS_COLUP_HMMB     ; 2 26
        sta COLUP1              ; 3 29
        ldx DS_GRP1             ; 4 33
        lda #<DS_JUMP           ; 2 35
        sta NextKernel          ; 3 38
        lda #<DS_JUMP           ; 2 40
        sta NextKernel+1        ; 3 43
        sta HMCLR               ; 3 46
        SLEEP 2                 ; 2 48
        lda #<DS_HMP0           ; 2 50
        sta HMP0                ; 3 53
        lda #<DS_HMP1           ; 2 55
        sta RESP1               ; 3 58        
        sta HMP1                ; 3 61
        lda #<DS_GRP0           ; 2 63
        sta GRP0                ; 3 66 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 68 <- on VDEL, for next scanline
        sta ENABL               ; 3 71
        lsr                     ; 2 73
        jmp R76                 ; 3 76
        
Resp1Strobe63:                  ;   19
        lda #<DS_NUSIZ_COLUPF   ; 2 21 
        sta NUSIZ1              ; 3 24
        lda #<DS_COLUP_HMMB     ; 2 26
        sta COLUP1              ; 3 29
        ldx DS_GRP1             ; 4 33
        lda #<DS_JUMP           ; 2 35
        sta NextKernel          ; 3 38
        lda #<DS_JUMP           ; 2 40
        sta NextKernel+1        ; 3 43
        sta HMCLR               ; 3 46
        SLEEP 2                 ; 2 48
        lda #<DS_HMP0           ; 2 50
        sta HMP0                ; 3 53
        lda #<DS_HMP1           ; 2 55
        sta HMP1                ; 3 58
        lda #<DS_GRP0           ; 2 60
        sta RESP1               ; 3 63        
        sta GRP0                ; 3 66
        lda #<DS_M1M0BL         ; 2 68
        sta ENABL               ; 3 71
        lsr                     ; 2 73
        jmp R76                 ; 3 76
        
Resp1Strobe68:                  ;   19
        lda #<DS_NUSIZ_COLUPF   ; 2 21 
        sta NUSIZ1              ; 3 24
        lda #<DS_COLUP_HMMB     ; 2 26
        sta COLUP1              ; 3 29
        ldx DS_GRP1             ; 4 33
        lda #<DS_JUMP           ; 2 35
        sta NextKernel          ; 3 38
        lda #<DS_JUMP           ; 2 40
        sta NextKernel+1        ; 3 43
        sta HMCLR               ; 3 46
        SLEEP 2                 ; 2 48
        lda #<DS_HMP0           ; 2 50
        sta HMP0                ; 3 53
        lda #<DS_HMP1           ; 2 55
        sta HMP1                ; 3 58
        lda #<DS_GRP0           ; 2 60
        sta GRP0                ; 3 63
        lda #<DS_M1M0BL         ; 2 65
        sta RESP1               ; 3 68
        sta ENABL               ; 3 71
        lsr                     ; 2 73
        jmp R76                 ; 3 76

Resp1Strobe73:                  ;   19
        lda #<DS_NUSIZ_COLUPF   ; 2 21 
        sta NUSIZ1              ; 3 24
        lda #<DS_COLUP_HMMB     ; 2 26
        sta COLUP1              ; 3 29
        ldx DS_GRP1             ; 4 33
        lda #<DS_JUMP           ; 2 35
        sta NextKernel          ; 3 38
        lda #<DS_JUMP           ; 2 40
        sta NextKernel+1        ; 3 43
        sta HMCLR               ; 3 46
        SLEEP 2                 ; 2 48
        lda #<DS_HMP0           ; 2 50
        sta HMP0                ; 3 53
        lda #<DS_HMP1           ; 2 55
        sta HMP1                ; 3 58
        lda #<DS_GRP0           ; 2 60
        sta GRP0                ; 3 63
        lda #<DS_M1M0BL         ; 2 65
        sta ENABL               ; 3 68
        lsr                     ; 2 70
        sta RESP1               ; 3 73
        jmp R76                 ; 3 76        
        
;----------------------------------------
; Reposition Missile0 Kernels
;   all must be consecutive with
;   __Strobe23 first and __Strobe73 last
; MUST FOLLOW Player1 Kernels
;----------------------------------------
Resm0Strobe23:                  ;   19
        sta.w RESM0             ; 4 23
        lda #<DS_JUMP           ; 2 25
        sta NextKernel          ; 3 28
        lda #<DS_JUMP           ; 2 30
        sta NextKernel+1        ; 3 33
        lda #<DS_NUSIZ_COLUPF   ; 2 35
        sta NUSIZ0              ; 3 38
        sta HMCLR               ; 3 41 ' reset missile/ball HMOVE
        ldx DS_GRP1             ; 4 45
        lda #<DS_COLUP_HMMB     ; 2 47
        sta HMM0                ; 3 50
        lda #<DS_HMP0           ; 2 52
        sta HMP0                ; 3 55
        lda #<DS_HMP1           ; 2 57
        sta HMP1                ; 3 60
        lda #<DS_GRP0           ; 2 62
        sta GRP0                ; 3 65 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 67 <- on VDEL, for next scanline
        sta.w ENABL             ; 4 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76
        
Resm0Strobe28:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        sta.w RESM0             ; 4 28
        lda #<DS_JUMP           ; 2 30
        sta NextKernel+1        ; 3 33
        lda #<DS_NUSIZ_COLUPF   ; 2 35
        sta NUSIZ0              ; 3 38
        sta HMCLR               ; 3 41 ' reset missile/ball HMOVE
        ldx DS_GRP1             ; 4 45
        lda #<DS_COLUP_HMMB     ; 2 47
        sta HMM0                ; 3 50
        lda #<DS_HMP0           ; 2 52
        sta HMP0                ; 3 55
        lda #<DS_HMP1           ; 2 57
        sta HMP1                ; 3 60
        lda #<DS_GRP0           ; 2 62
        sta GRP0                ; 3 65 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 67 <- on VDEL, for next scanline
        sta.w ENABL             ; 4 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76

Resm0Strobe33:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        sta.w RESM0             ; 4 33
        lda #<DS_NUSIZ_COLUPF   ; 2 35
        sta NUSIZ0              ; 3 38
        sta HMCLR               ; 3 41 ' reset missile/ball HMOVE
        ldx DS_GRP1             ; 4 45
        lda #<DS_COLUP_HMMB     ; 2 47
        sta HMM0                ; 3 50
        lda #<DS_HMP0           ; 2 52
        sta HMP0                ; 3 55
        lda #<DS_HMP1           ; 2 57
        sta HMP1                ; 3 60
        lda #<DS_GRP0           ; 2 62
        sta GRP0                ; 3 65 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 67 <- on VDEL, for next scanline
        sta.w ENABL             ; 4 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76

Resm0Strobe38:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta NUSIZ0              ; 3 34
        sta.w RESM0             ; 4 38
        sta HMCLR               ; 3 41 ' reset missile/ball HMOVE
        ldx DS_GRP1             ; 4 45
        lda #<DS_COLUP_HMMB     ; 2 47
        sta HMM0                ; 3 50
        lda #<DS_HMP0           ; 2 52
        sta HMP0                ; 3 55
        lda #<DS_HMP1           ; 2 57
        sta HMP1                ; 3 60
        lda #<DS_GRP0           ; 2 62
        sta GRP0                ; 3 65 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 67 <- on VDEL, for next scanline
        sta.w ENABL             ; 4 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76

Resm0Strobe43:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta NUSIZ0              ; 3 34
        ldx DS_GRP1             ; 4 38
        SLEEP 2                 ; 2 40
        sta RESM0               ; 3 43
        sta HMCLR               ; 3 46 ' reset missile/ball HMOVE
        lda #<DS_COLUP_HMMB     ; 2 48
        sta HMM0                ; 3 51
        lda #<DS_HMP0           ; 2 53
        sta HMP0                ; 3 56
        lda #<DS_HMP1           ; 2 58
        sta HMP1                ; 3 61
        lda #<DS_GRP0           ; 2 63
        sta GRP0                ; 3 66 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 68 <- on VDEL, for next scanline
        sta ENABL               ; 3 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76
        
Resm0Strobe48:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta NUSIZ0              ; 3 34
        ldx DS_GRP1             ; 4 38
        SLEEP 2                 ; 2 40
        sta HMCLR               ; 3 43 ' reset missile/ball HMOVE
        lda #<DS_COLUP_HMMB     ; 2 45
        sta RESM0               ; 3 48
        sta HMM0                ; 3 51
        lda #<DS_HMP0           ; 2 53
        sta HMP0                ; 3 56
        lda #<DS_HMP1           ; 2 58
        sta HMP1                ; 3 61
        lda #<DS_GRP0           ; 2 63
        sta GRP0                ; 3 66 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 68 <- on VDEL, for next scanline
        sta ENABL               ; 3 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76      

Resm0Strobe53:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta NUSIZ0              ; 3 34
        ldx DS_GRP1             ; 4 38
        SLEEP 2                 ; 2 40
        sta HMCLR               ; 3 43 ' reset missile/ball HMOVE
        lda #<DS_COLUP_HMMB     ; 2 45
        sta HMM0                ; 3 48
        lda #<DS_HMP0           ; 2 50
        sta RESM0               ; 3 53
        sta HMP0                ; 3 56
        lda #<DS_HMP1           ; 2 58
        sta HMP1                ; 3 61
        lda #<DS_GRP0           ; 2 63
        sta GRP0                ; 3 66 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 68 <- on VDEL, for next scanline
        sta ENABL               ; 3 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76      
        
Resm0Strobe58:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta NUSIZ0              ; 3 34
        ldx DS_GRP1             ; 4 38
        SLEEP 2                 ; 2 40
        sta HMCLR               ; 3 43 ' reset missile/ball HMOVE
        lda #<DS_COLUP_HMMB     ; 2 45
        sta HMM0                ; 3 48
        lda #<DS_HMP0           ; 2 50
        sta HMP0                ; 3 53
        lda #<DS_HMP1           ; 2 55
        sta RESM0               ; 3 58
        sta HMP1                ; 3 61
        lda #<DS_GRP0           ; 2 63
        sta GRP0                ; 3 66 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 68 <- on VDEL, for next scanline
        sta ENABL               ; 3 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76      
        
Resm0Strobe63:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta NUSIZ0              ; 3 34
        ldx DS_GRP1             ; 4 38
        SLEEP 2                 ; 2 40
        sta HMCLR               ; 3 43 ' reset missile/ball HMOVE
        lda #<DS_COLUP_HMMB     ; 2 45
        sta HMM0                ; 3 48
        lda #<DS_HMP0           ; 2 50
        sta HMP0                ; 3 53
        lda #<DS_HMP1           ; 2 55
        sta HMP1                ; 3 58
        lda #<DS_GRP0           ; 2 60
        sta RESM0               ; 3 63
        sta GRP0                ; 3 66 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 68 <- on VDEL, for next scanline
        sta ENABL               ; 3 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76      

Resm0Strobe68:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta NUSIZ0              ; 3 34
        ldx DS_GRP1             ; 4 38
        SLEEP 2                 ; 2 40
        sta HMCLR               ; 3 43 ' reset missile/ball HMOVE
        lda #<DS_COLUP_HMMB     ; 2 45
        sta HMM0                ; 3 48
        lda #<DS_HMP0           ; 2 50
        sta HMP0                ; 3 53
        lda #<DS_HMP1           ; 2 55
        sta HMP1                ; 3 58
        lda #<DS_GRP0           ; 2 60
        sta GRP0                ; 3 63 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 65 <- on VDEL, for next scanline
        sta RESM0               ; 3 68
        sta ENABL               ; 3 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76               

Resm0Strobe73:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta NUSIZ0              ; 3 34
        ldx DS_GRP1             ; 4 38
        SLEEP 2                 ; 2 40
        sta HMCLR               ; 3 43 ' reset missile/ball HMOVE
        lda #<DS_COLUP_HMMB     ; 2 45
        sta HMM0                ; 3 48
        lda #<DS_HMP0           ; 2 50
        sta HMP0                ; 3 53
        lda #<DS_HMP1           ; 2 55
        sta HMP1                ; 3 58
        lda #<DS_GRP0           ; 2 60
        sta GRP0                ; 3 63 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 65 <- on VDEL, for next scanline
        sta ENABL               ; 3 68
        lsr                     ; 2 70         
        sta RESM0               ; 3 73
        jmp R76                 ; 3 76     

;----------------------------------------
; Reposition Missile1 Kernels
;   all must be consecutive with
;   __Strobe23 first and __Strobe73 last
; MUST FOLLOW Missile0 Kernels
;----------------------------------------
Resm1Strobe23:                  ;   19
        sta.w RESM1             ; 4 23
        lda #<DS_JUMP           ; 2 25
        sta NextKernel          ; 3 28
        lda #<DS_JUMP           ; 2 30
        sta NextKernel+1        ; 3 33
        lda #<DS_NUSIZ_COLUPF   ; 2 35
        sta NUSIZ1              ; 3 38
        sta HMCLR               ; 3 41 ' reset missile/ball HMOVE
        ldx DS_GRP1             ; 4 45
        lda #<DS_COLUP_HMMB     ; 2 47
        sta HMM1                ; 3 50
        lda #<DS_HMP0           ; 2 52
        sta HMP0                ; 3 55
        lda #<DS_HMP1           ; 2 57
        sta HMP1                ; 3 60
        lda #<DS_GRP0           ; 2 62
        sta GRP0                ; 3 65 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 67 <- on VDEL, for next scanline
        sta.w ENABL             ; 4 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76
        
Resm1Strobe28:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        sta.w RESM1             ; 4 28
        lda #<DS_JUMP           ; 2 30
        sta NextKernel+1        ; 3 33
        lda #<DS_NUSIZ_COLUPF   ; 2 35
        sta NUSIZ1              ; 3 38
        sta HMCLR               ; 3 41 ' reset missile/ball HMOVE
        ldx DS_GRP1             ; 4 45
        lda #<DS_COLUP_HMMB     ; 2 47
        sta HMM1                ; 3 50
        lda #<DS_HMP0           ; 2 52
        sta HMP0                ; 3 55
        lda #<DS_HMP1           ; 2 57
        sta HMP1                ; 3 60
        lda #<DS_GRP0           ; 2 62
        sta GRP0                ; 3 65 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 67 <- on VDEL, for next scanline
        sta.w ENABL             ; 4 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76

Resm1Strobe33:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        sta.w RESM1             ; 4 33
        lda #<DS_NUSIZ_COLUPF   ; 2 35
        sta NUSIZ1              ; 3 38
        sta HMCLR               ; 3 41 ' reset missile/ball HMOVE
        ldx DS_GRP1             ; 4 45
        lda #<DS_COLUP_HMMB     ; 2 47
        sta HMM1                ; 3 50
        lda #<DS_HMP0           ; 2 52
        sta HMP0                ; 3 55
        lda #<DS_HMP1           ; 2 57
        sta HMP1                ; 3 60
        lda #<DS_GRP0           ; 2 62
        sta GRP0                ; 3 65 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 67 <- on VDEL, for next scanline
        sta.w ENABL             ; 4 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76

Resm1Strobe38:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta NUSIZ1              ; 3 34
        sta.w RESM1             ; 4 38
        sta HMCLR               ; 3 41 ' reset missile/ball HMOVE
        ldx DS_GRP1             ; 4 45
        lda #<DS_COLUP_HMMB     ; 2 47
        sta HMM1                ; 3 50
        lda #<DS_HMP0           ; 2 52
        sta HMP0                ; 3 55
        lda #<DS_HMP1           ; 2 57
        sta HMP1                ; 3 60
        lda #<DS_GRP0           ; 2 62
        sta GRP0                ; 3 65 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 67 <- on VDEL, for next scanline
        sta.w ENABL             ; 4 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76

Resm1Strobe43:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta NUSIZ1              ; 3 34
        ldx DS_GRP1             ; 4 38
        SLEEP 2                 ; 2 40
        sta RESM1               ; 3 43
        sta HMCLR               ; 3 46 ' reset missile/ball HMOVE
        lda #<DS_COLUP_HMMB     ; 2 48
        sta HMM1                ; 3 51
        lda #<DS_HMP0           ; 2 53
        sta HMP0                ; 3 56
        lda #<DS_HMP1           ; 2 58
        sta HMP1                ; 3 61
        lda #<DS_GRP0           ; 2 63
        sta GRP0                ; 3 66 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 68 <- on VDEL, for next scanline
        sta ENABL               ; 3 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76
        
Resm1Strobe48:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta NUSIZ1              ; 3 34
        ldx DS_GRP1             ; 4 38
        SLEEP 2                 ; 2 40
        sta HMCLR               ; 3 43 ' reset missile/ball HMOVE
        lda #<DS_COLUP_HMMB     ; 2 45
        sta RESM1               ; 3 48
        sta HMM1                ; 3 51
        lda #<DS_HMP0           ; 2 53
        sta HMP0                ; 3 56
        lda #<DS_HMP1           ; 2 58
        sta HMP1                ; 3 61
        lda #<DS_GRP0           ; 2 63
        sta GRP0                ; 3 66 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 68 <- on VDEL, for next scanline
        sta ENABL               ; 3 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76      

Resm1Strobe53:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta NUSIZ1              ; 3 34
        ldx DS_GRP1             ; 4 38
        SLEEP 2                 ; 2 40
        sta HMCLR               ; 3 43 ' reset missile/ball HMOVE
        lda #<DS_COLUP_HMMB     ; 2 45
        sta HMM1                ; 3 48
        lda #<DS_HMP0           ; 2 50
        sta RESM1               ; 3 53
        sta HMP0                ; 3 56
        lda #<DS_HMP1           ; 2 58
        sta HMP1                ; 3 61
        lda #<DS_GRP0           ; 2 63
        sta GRP0                ; 3 66 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 68 <- on VDEL, for next scanline
        sta ENABL               ; 3 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76      
        
Resm1Strobe58:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta NUSIZ1              ; 3 34
        ldx DS_GRP1             ; 4 38
        SLEEP 2                 ; 2 40
        sta HMCLR               ; 3 43 ' reset missile/ball HMOVE
        lda #<DS_COLUP_HMMB     ; 2 45
        sta HMM1                ; 3 48
        lda #<DS_HMP0           ; 2 50
        sta HMP0                ; 3 53
        lda #<DS_HMP1           ; 2 55
        sta RESM1               ; 3 58
        sta HMP1                ; 3 61
        lda #<DS_GRP0           ; 2 63
        sta GRP0                ; 3 66 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 68 <- on VDEL, for next scanline
        sta ENABL               ; 3 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76      
        
Resm1Strobe63:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta NUSIZ1              ; 3 34
        ldx DS_GRP1             ; 4 38
        SLEEP 2                 ; 2 40
        sta HMCLR               ; 3 43 ' reset missile/ball HMOVE
        lda #<DS_COLUP_HMMB     ; 2 45
        sta HMM1                ; 3 48
        lda #<DS_HMP0           ; 2 50
        sta HMP0                ; 3 53
        lda #<DS_HMP1           ; 2 55
        sta HMP1                ; 3 58
        lda #<DS_GRP0           ; 2 60
        sta RESM1               ; 3 63
        sta GRP0                ; 3 66 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 68 <- on VDEL, for next scanline
        sta ENABL               ; 3 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76      

Resm1Strobe68:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta NUSIZ1              ; 3 34
        ldx DS_GRP1             ; 4 38
        SLEEP 2                 ; 2 40
        sta HMCLR               ; 3 43 ' reset missile/ball HMOVE
        lda #<DS_COLUP_HMMB     ; 2 45
        sta HMM1                ; 3 48
        lda #<DS_HMP0           ; 2 50
        sta HMP0                ; 3 53
        lda #<DS_HMP1           ; 2 55
        sta HMP1                ; 3 58
        lda #<DS_GRP0           ; 2 60
        sta GRP0                ; 3 63 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 65 <- on VDEL, for next scanline
        sta RESM1               ; 3 68
        sta ENABL               ; 3 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76               

Resm1Strobe73:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta NUSIZ1              ; 3 34
        ldx DS_GRP1             ; 4 38
        SLEEP 2                 ; 2 40
        sta HMCLR               ; 3 43 ' reset missile/ball HMOVE
        lda #<DS_COLUP_HMMB     ; 2 45
        sta HMM1                ; 3 48
        lda #<DS_HMP0           ; 2 50
        sta HMP0                ; 3 53
        lda #<DS_HMP1           ; 2 55
        sta HMP1                ; 3 58
        lda #<DS_GRP0           ; 2 60
        sta GRP0                ; 3 63 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 65 <- on VDEL, for next scanline
        sta ENABL               ; 3 68
        lsr                     ; 2 70         
        sta RESM1               ; 3 73
        jmp R76                 ; 3 76     

        
;----------------------------------------
; Reposition Ball Kernels
;----------------------------------------
ResblStrobe23:                  ;   19
        sta.w RESBL             ; 4 23
        lda #<DS_JUMP           ; 2 25
        sta NextKernel          ; 3 28
        lda #<DS_JUMP           ; 2 30
        sta NextKernel+1        ; 3 33
        lda #<DS_NUSIZ_COLUPF   ; 2 35
        sta COLUPF              ; 3 38
        sta HMCLR               ; 3 41 ' reset missile/ball HMOVE
        ldx DS_GRP1             ; 4 45
        lda #<DS_COLUP_HMMB     ; 2 47
        sta HMBL                ; 3 50
        lda #<DS_HMP0           ; 2 52
        sta HMP0                ; 3 55
        lda #<DS_HMP1           ; 2 57
        sta HMP1                ; 3 60
        lda #<DS_GRP0           ; 2 62
        sta GRP0                ; 3 65 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 67 <- on VDEL, for next scanline
        sta.w ENABL             ; 4 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76
        
ResblStrobe28:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        sta.w RESBL             ; 4 28
        lda #<DS_JUMP           ; 2 30
        sta NextKernel+1        ; 3 33
        lda #<DS_NUSIZ_COLUPF   ; 2 35
        sta COLUPF              ; 3 38
        sta HMCLR               ; 3 41 ' reset missile/ball HMOVE
        ldx DS_GRP1             ; 4 45
        lda #<DS_COLUP_HMMB     ; 2 47
        sta HMBL                ; 3 50
        lda #<DS_HMP0           ; 2 52
        sta HMP0                ; 3 55
        lda #<DS_HMP1           ; 2 57
        sta HMP1                ; 3 60
        lda #<DS_GRP0           ; 2 62
        sta GRP0                ; 3 65 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 67 <- on VDEL, for next scanline
        sta.w ENABL             ; 4 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76

ResblStrobe33:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        sta.w RESBL             ; 4 33
        lda #<DS_NUSIZ_COLUPF   ; 2 35
        sta COLUPF              ; 3 38
        sta HMCLR               ; 3 41 ' reset missile/ball HMOVE
        ldx DS_GRP1             ; 4 45
        lda #<DS_COLUP_HMMB     ; 2 47
        sta HMBL                ; 3 50
        lda #<DS_HMP0           ; 2 52
        sta HMP0                ; 3 55
        lda #<DS_HMP1           ; 2 57
        sta HMP1                ; 3 60
        lda #<DS_GRP0           ; 2 62
        sta GRP0                ; 3 65 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 67 <- on VDEL, for next scanline
        sta.w ENABL             ; 4 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76

ResblStrobe38:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta COLUPF              ; 3 34
        sta.w RESBL             ; 4 38
        sta HMCLR               ; 3 41 ' reset missile/ball HMOVE
        ldx DS_GRP1             ; 4 45
        lda #<DS_COLUP_HMMB     ; 2 47
        sta HMBL                ; 3 50
        lda #<DS_HMP0           ; 2 52
        sta HMP0                ; 3 55
        lda #<DS_HMP1           ; 2 57
        sta HMP1                ; 3 60
        lda #<DS_GRP0           ; 2 62
        sta GRP0                ; 3 65 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 67 <- on VDEL, for next scanline
        sta.w ENABL             ; 4 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76

ResblStrobe43:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta COLUPF              ; 3 34
        ldx DS_GRP1             ; 4 38
        SLEEP 2                 ; 2 40
        sta RESBL               ; 3 43
        sta HMCLR               ; 3 46 ' reset missile/ball HMOVE
        lda #<DS_COLUP_HMMB     ; 2 48
        sta HMBL                ; 3 51
        lda #<DS_HMP0           ; 2 53
        sta HMP0                ; 3 56
        lda #<DS_HMP1           ; 2 58
        sta HMP1                ; 3 61
        lda #<DS_GRP0           ; 2 63
        sta GRP0                ; 3 66 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 68 <- on VDEL, for next scanline
        sta ENABL               ; 3 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76
        
ResblStrobe48:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta COLUPF              ; 3 34
        ldx DS_GRP1             ; 4 38
        SLEEP 2                 ; 2 40
        sta HMCLR               ; 3 43 ' reset missile/ball HMOVE
        lda #<DS_COLUP_HMMB     ; 2 45
        sta RESBL               ; 3 48
        sta HMBL                ; 3 51
        lda #<DS_HMP0           ; 2 53
        sta HMP0                ; 3 56
        lda #<DS_HMP1           ; 2 58
        sta HMP1                ; 3 61
        lda #<DS_GRP0           ; 2 63
        sta GRP0                ; 3 66 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 68 <- on VDEL, for next scanline
        sta ENABL               ; 3 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76      

ResblStrobe53:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta COLUPF              ; 3 34
        ldx DS_GRP1             ; 4 38
        SLEEP 2                 ; 2 40
        sta HMCLR               ; 3 43 ' reset missile/ball HMOVE
        lda #<DS_COLUP_HMMB     ; 2 45
        sta HMBL                ; 3 48
        lda #<DS_HMP0           ; 2 50
        sta RESBL               ; 3 53
        sta HMP0                ; 3 56
        lda #<DS_HMP1           ; 2 58
        sta HMP1                ; 3 61
        lda #<DS_GRP0           ; 2 63
        sta GRP0                ; 3 66 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 68 <- on VDEL, for next scanline
        sta ENABL               ; 3 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76      
        
ResblStrobe58:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta COLUPF              ; 3 34
        ldx DS_GRP1             ; 4 38
        SLEEP 2                 ; 2 40
        sta HMCLR               ; 3 43 ' reset missile/ball HMOVE
        lda #<DS_COLUP_HMMB     ; 2 45
        sta HMBL                ; 3 48
        lda #<DS_HMP0           ; 2 50
        sta HMP0                ; 3 53
        lda #<DS_HMP1           ; 2 55
        sta RESBL               ; 3 58
        sta HMP1                ; 3 61
        lda #<DS_GRP0           ; 2 63
        sta GRP0                ; 3 66 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 68 <- on VDEL, for next scanline
        sta ENABL               ; 3 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76      
        
ResblStrobe63:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta COLUPF              ; 3 34
        ldx DS_GRP1             ; 4 38
        SLEEP 2                 ; 2 40
        sta HMCLR               ; 3 43 ' reset missile/ball HMOVE
        lda #<DS_COLUP_HMMB     ; 2 45
        sta HMBL                ; 3 48
        lda #<DS_HMP0           ; 2 50
        sta HMP0                ; 3 53
        lda #<DS_HMP1           ; 2 55
        sta HMP1                ; 3 58
        lda #<DS_GRP0           ; 2 60
        sta RESBL               ; 3 63
        sta GRP0                ; 3 66 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 68 <- on VDEL, for next scanline
        sta ENABL               ; 3 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76      

ResblStrobe68:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta COLUPF              ; 3 34
        ldx DS_GRP1             ; 4 38
        SLEEP 2                 ; 2 40
        sta HMCLR               ; 3 43 ' reset missile/ball HMOVE
        lda #<DS_COLUP_HMMB     ; 2 45
        sta HMBL                ; 3 48
        lda #<DS_HMP0           ; 2 50
        sta HMP0                ; 3 53
        lda #<DS_HMP1           ; 2 55
        sta HMP1                ; 3 58
        lda #<DS_GRP0           ; 2 60
        sta GRP0                ; 3 63 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 65 <- on VDEL, for next scanline
        sta RESBL               ; 3 68
        sta ENABL               ; 3 71
        lsr                     ; 2 73         
        jmp R76                 ; 3 76               

ResblStrobe73:                  ;   19
        lda #<DS_JUMP           ; 2 21
        sta NextKernel          ; 3 24
        lda #<DS_JUMP           ; 2 26
        sta NextKernel+1        ; 3 29
        lda #<DS_NUSIZ_COLUPF   ; 2 31
        sta COLUPF              ; 3 34
        ldx DS_GRP1             ; 4 38
        SLEEP 2                 ; 2 40
        sta HMCLR               ; 3 43 ' reset missile/ball HMOVE
        lda #<DS_COLUP_HMMB     ; 2 45
        sta HMBL                ; 3 48
        lda #<DS_HMP0           ; 2 50
        sta HMP0                ; 3 53
        lda #<DS_HMP1           ; 2 55
        sta HMP1                ; 3 58
        lda #<DS_GRP0           ; 2 60
        sta GRP0                ; 3 63 <- on VDEL, for next scanline
        lda #<DS_M1M0BL         ; 2 65 <- on VDEL, for next scanline
        sta ENABL               ; 3 68
        lsr                     ; 2 70         
        sta RESBL               ; 3 73
        jmp R76                 ; 3 76         
        
MainMenu:
        ; ldy #zz__XX__XX     ; 2  7
        ; sty ENAM0       ; 3 10
        ; sty ENAM1       ; 3 13
        ; sty ENABL       ; 3 16        
        ; sty NUSIZ0      ; 3 63
        ; sty NUSIZ1      ; 3 66
        ; sty VDELP0
        ; sty VDELP1        
        ; ldy #%00100000  ; Ball Size = 4
        ; sty CTRLPF
        
PositionForLogo:
        ldx #3
        lda Frame
        and #1
        bne paOdd
        ldy #3          ; even frame
        .byte $2c
paOdd:  ldy #7          ; odd frame
pflLoop:
        lda ObjectPosition96,y
        jsr PosObject
        dey
        dex
        bpl pflLoop
        stx WSYNC
        stx HMOVE
        
DataStreamPrep:
        ldx #7
        lda Frame
        and #1
        bne dspOdd
        ldy #7          ; even frame
        .byte $2c
dspOdd: ldy #15         ; odd frame        
dspLoop:
        lda LogoLow,y
        sta DF0LOW,x
        lda LogoHi,y
        sta DF0HI,x
        dey
        dex
        bpl dspLoop        
        
        ldy #zz__XX__XX     ; 2  7
        sty ENAM0       ; 3 10
        sty ENAM1       ; 3 13
        sty NUSIZ0      ; 3 63
        sty NUSIZ1      ; 3 66

        ldy #zzXXXXXXXX
        sty GRP0
        sty GRP1

        ldy #zr____XXXX
        sty PF0        
        ldy #zzXXXX____
        sty PF1
        ldy #zr_______X
        sty PF2
        sty VDELP0  ; turn off VDEL
        sty VDELP1  ; turn off VDEL
        sty VDELBL  ; turn off VDEL
        
        ldy #zz______XX ; D0=1 reflect playfield, D1=1 SCORE mode on
        sty CTRLPF
        
        lda #<DS_COLUBK
        sta COLUPF
                
        ldy #0
        sty COLUP0
        sty COLUP1        
        
; PositionMMobjects:
        ; ldx #3   
; pmmoLoop:   
        ; lda MMobjectPosition,x
        ; jsr PosObject
        ; dex
        ; bpl pmmoLoop
        ; sta WSYNC
        ; sta HMOVE   
        ; 
        ; ldx #<MenuOptionsDatastream
        ; stx DF0LOW
        ; ldx #>MenuOptionsDatastream
        ; stx DF0HI           
        ; 
        ; ldx #0   
        ; stx COLUP0
        ; stx COLUP1
        ; stx VDELBL
        
VblankWaitMenu:
        sta WSYNC
        lda INTIM           ; 4  4
        bpl VblankWaitMenu  ; 2  6
        sty VBLANK          ; 3  9 
        lda Frame               ; 3 12
        and #1                  ; 2 14
        beq StartEven           ; 2 16
        jsr SLEEP12             ;12 28
        jsr SLEEP12             ;12 40
        jsr SLEEP12             ;12 52
        jsr SLEEP12             ;12 64
        SLEEP 6                 ; 6 70
        sta HMCLR               ; 3 73
        jmp OddLine             ; 3 76/0
        
StartEven:        
        sta WSYNC        
EvenLine:                       ;    0
        lda #<DS_COLUBK         ; 2  2
        sta COLUBK              ; 3  5
        beq LogoDone            ; 2  7 - if color was 0 we're done showing image
        lda #<DS_GRP0A          ; 2  9
        sta GRP0                ; 3 12 - must be 0-33 
        lda #<DS_GRP1A          ; 2 14
        sta GRP1                ; 3 17 - must be 0-49
        lda #<DS_PF2            ; 2 19
        sta PF2                 ; 3 22
        SLEEP 5                 ; 5 27
        ldy #zz_______X         ; 2 29 - D0=1 reflect playfield, D1=0 SCORE mode off
        sty CTRLPF              ; 3 32 - SCORE mode off so second color shows up
        lda #<DS_GRP0B          ; 2 34
        sta GRP0                ; 3 37 - must be 36-38 
        lda #<DS_GRP0C          ; 2 39
        sta GRP0                ; 3 42 - must be 41-43        
        ldy #zzXXXXXX__         ; 2 44
        sty PF1                 ; 3 47 - mask 8 pixels on right that aren't hidden with missile
        lda #<DS_GRP1B          ; 2 49
        sta GRP1                ; 3 52 - must be 52-54 
        lda #<DS_GRP1C          ; 2 54
        sta GRP1                ; 3 57 - must be 57-59
        ldy #zz______XX         ; 2 59 - D0=1 reflect playfield, D1=1 SCORE mode on
        sty CTRLPF              ; 3 62 - SCORE mode on to hide edges of screen
        ldy #HMOVE_R8           ; 2 64
        sty HMP0                ; 3 67
        sty HMP1                ; 3 70
        sty HMM0                ; 3 73
        sty HMM1                ; 3 76
OddLine:        
        sta HMOVE               ; 3  3 shift all objects right 8 pixels
        lda #<DS_COLUBK         ; 2  5
        sta COLUBK              ; 3  8
        beq LogoDone            ; 2 10 - if color was 0 we're done showing image       
        lda #<DS_GRP0A          ; 2 12
        sta GRP0                ; 3 15 - must be 0-35
        lda #<DS_GRP1A          ; 2 17
        sta GRP1                ; 3 20 - must be 0-51
        lda #<DS_PF2            ; 2 22
        sta PF2                 ; 3 25
        ldy #zzXXXX____         ; 2 32
        sty PF1                 ; 3 35 - remove mask so player can draw 8 pixels on right        
        ldy #zz_______X         ; 2 27 - D0=1 reflect playfield, D1=0 SCORE mode off
        sty CTRLPF              ; 3 30 - SCORE mode off so second color shows up
        lda #<DS_GRP0B          ; 2 37
        sta GRP0                ; 3 40 - must be 39-41
        lda #<DS_GRP0C          ; 2 42
        sta GRP0                ; 3 45 - must be 44-46
        SLEEP 4                 ; 4 49        
        ldy #zz______XX         ; 2 51 D0=1 reflect playfield, D1=1 SCORE mode on
        lda #<DS_GRP1B          ; 2 53
        sta GRP1                ; 3 56 - must be 55-57
        lda #<DS_GRP1C          ; 2 58
        sta GRP1                ; 3 61 - must be 60-62
        sty CTRLPF              ; 3 64 - SCORE mode on to hide edges of screen
        SLEEP 3                 ; 3 67
        sta HMCLR               ; 3 70 - 27-70
        sta HMOVE               ; 3 73 - shift all objects left 8 pixels
        jmp EvenLine            ; 3 76/0
        
LogoDone:                       ;   11 (from OddLine)
        sta COLUPF              ; 3 14 A has 0 
        sta GRP0                ; 3 17
        sta GRP1                ; 3 20
        sta ENAM0               ; 3 23
        sta ENAM1               ; 3 26
        ldy #zzXXXX____
        sty PF1
        
        ldy #zz__XX__XX     ; 2  7
        sty ENAM0       ; 3 10
        sty ENAM1       ; 3 13
        sty ENABL       ; 3 16        
        sty NUSIZ0      ; 3 63
        sty NUSIZ1      ; 3 66
        sty VDELP0
        sty VDELP1        
        ldy #%00100000  ; Ball Size = 4
        sty CTRLPF
        
PositionMMobjects:
        ldx #3   
pmmoLoop:   
        lda MMobjectPosition,x
        jsr PosObject
        dex
        bpl pmmoLoop
        sta WSYNC
        sta HMOVE   
        
        ldx #<MenuOptionsDatastream
        stx DF0LOW
        ldx #>MenuOptionsDatastream
        stx DF0HI           
        
        ldx #0   
        stx COLUP0
        stx COLUP1
        stx VDELBL        

MMloop: 
        ldx #6
MMloop2:
        lda #<DF0DATA
        sta DF1LOW,x
        lda #<DF0DATA
        sta DF1HI,x
        dex
        bpl MMloop2
        lda #<DF0DATA
        sta PF0
        lda #<DF0DATA
        sta PF1
        lda #<DF0DATA
        sta PF2
        sta HMCLR
        ldx #4
        lda #<DF0DATA
        jsr PosObject
        sta WSYNC
        sta HMOVE   
        lda #<DF0DATA
        tay
        jsr ShowTwoColorGraphic
        lda #<DF0DATA
        bmi MMdone
        beq MMloop
        tay
MMloop3
        sta WSYNC
        dey
        bne MMloop3
        beq MMloop
    

MMdone: 
        sta WSYNC
        stx ENAM0    ; X=0 after ShowTwoColorGraphic
        stx ENAM1
        stx ENABL
        
        ldx #MM_LOGO_GAP
SLgap:
        sta WSYNC
        dex
        bpl SLgap
        
; show the SpiceWare logo   
ShowSpiceWareLogo:  
    ldx #5
;    ldy #5
SSWL0:
    lda SWLlo,x
    sta DF2LOW,x
    lda SWLhi,x
    sta DF2HI,x
;    dey
    dex
    bpl SSWL0
    ldx #WHITE
    stx COLUP0
    stx COLUP1
    stx COLUPF
    
    ldy #7
    sty LoopCounter
    
    ; show 1st line
    sta WSYNC
    ;dec Sleep5 ; JTZ
    lda #<DF2DATA       ; 2  7
    sta GRP0        ; 3 10
    lda #<DF3DATA       ; 2 12
    sta GRP1        ; 3 15
    lda #<DF4DATA       ; 2 17
    sta GRP0        ; 3 20
    lda #<DF5DATA       ; 2 22
    ldx DF6DATA     ; 4 26
    ldy DF7DATA     ; 4 30
    SLEEP 16
    ;jsr WasteTime16  
    ;SLEEP 4 ; JTZ
    ;PHA ; \ 7 cycles, 2 bytes ; JTZ
    ;PLA ; / ; JTZ

        
    sta GRP1        ; 3 44 <- time critical 
    stx GRP0        ; 3 47
    sty GRP1        ; 3 50
    sty GRP0        ; 3 53
    sty PF1         ; 3 56  ; Y has $FF from extended line in SW logo
    sty PF2         ; 3 59

    lda #<DF2DATA       ; 2 61  ; next line
    sta GRP0        ; 3 64
    lda #<DF3DATA       ; 2 66
    sta GRP1        ; 3 69
    ; JTZ start original code
    ; ldy #0          ; 2 71
    ; sty PF1         ; 3 74
    ; sty PF2         ; 3 77/1
    ; SLEEP 5         ; 5  6 space for audio if we need it
    ; ;dec Sleep5 - can't use dec due to beq
    ; SLEEP 6         ; 6 12
    ; JTZ end original code
    
    ; JTZ start optimized code
    iny             ; 2 71
    sty PF1         ; 3 74
    sty PF2         ; 3 77/1
    dec Sleep5      ; 5  6 space for audio if we need it
    SLEEP 4         ; 4 10
    tya             ; 2 12    
    ; JTZ end optimized code
    
    beq SSWLskip1       ; 3 15
    
SSWL1:
    sta WSYNC
    ;SLEEP 5
    dec Sleep5
    lda #<DF2DATA       ; 2  7
    sta GRP0        ; 3 10
    lda #<DF3DATA       ; 2 12
    sta GRP1        ; 3 15
SSWLskip1:
    lda #<DF4DATA       ; 2 17
    sta GRP0        ; 3 20
    lda #<DF5DATA       ; 2 22
    ldx DF6DATA     ; 4 26
    ldy DF7DATA     ; 4 30
    SLEEP 6         ; 6 36  
    dec LoopCounter  ; 5 41
    sta GRP1        ; 3 44
    stx GRP0        ; 3 47
    sty GRP1        ; 3 50
    sty GRP0        ; 3 53
    bne SSWL1       ; 3 56  
                ; 2 55 if not taken
    ; show last line
    lda #<DF2DATA       ; 2 57
    sta GRP0        ; 3 60
    ldx DF3DATA     ; 4 64
    stx GRP1        ; 3 67
    sta PF0         ; 3 70  ; A = $FF
    sta PF1         ; 3 73
    ldx #zz______XX     ; 2 75
    ;dec Sleep5 ; JTZ
    stx ENAM0       ; 3  7
    ldx #1          ; 2  9
    stx CTRLPF      ; 3 12  ; reflect playfield
    lda #<DF4DATA       ; 2 14
    sta GRP0        ; 3 17
    lda #<DF5DATA       ; 2 19
    ldx DF6DATA     ; 4 23
    
    ;jsr WasteTime19 ; JTZ - was 14
    SLEEP 19
    
    ldy DF7DATA     ; 4 40
    sta GRP1        ; 3 44 <- time critical 
    stx GRP0        ; 3 47
    sty GRP1        ; 3 50
    sty GRP0        ; 3 53
    ldy #0          ; 2 55
    sty PF1         ; 3 58
    sty PF0         ; 3 61
    ;stx GRP1        ; 3 64
    ;stx GRP0        ; 3 67
    sty ENAM0               ; 3 70
EndShowSpiceWareLogo:        
        

        jmp SetOStimer        
                

        
;        align 256
        ; had weird glitch prepping datastreams 
        ; when InitDShi wrapped to another page...
InitDSlow:  .byte #<Player0DataStream
            .byte #<Player1DataStream
            .byte #<MissileBallDataStream
            .byte #<HMP0DataStream
            .byte #<HMP1DataStream
            .byte #<ColorHMDataStream
            .byte #<SizeBallColorDataStream
            .byte #<JumpDataStream
            
InitDShi:   .byte #>Player0DataStream
            .byte #>Player1DataStream
            .byte #>MissileBallDataStream
            .byte #>HMP0DataStream
            .byte #>HMP1DataStream
            .byte #>ColorHMDataStream
            .byte #>SizeBallColorDataStream
            .byte #>JumpDataStream
            
InitPFlow:  .byte #<PF0L
            .byte #<PF1L
            .byte #<PF2L
            .byte #<PF0R
            .byte #<PF1R
            .byte #<PF2R
            .byte #<RadarControl
            
InitPFhi:   .byte #>PF0L
            .byte #>PF1L
            .byte #>PF2L
            .byte #>PF0R
            .byte #>PF1R
            .byte #>PF2R
            .byte #>RadarControl
            
MMobjectPosition:
        .byte $38   ; P0 for menu
        .byte $40   ; P1 for menu
        .byte 17    ; Missile 0 - to hide MM logo garbage
        .byte 105   ; Missile 1 - to hide MM logo garbage            

SWLlo:
        .byte <SpiceWareLogoA
        .byte <SpiceWareLogoB
        .byte <SpiceWareLogoC
        .byte <SpiceWareLogoD
        .byte <SpiceWareLogoE
        .byte <SpiceWareLogoF   
    
SWLhi:
        .byte >SpiceWareLogoA
        .byte >SpiceWareLogoB
        .byte >SpiceWareLogoC
        .byte >SpiceWareLogoD
        .byte >SpiceWareLogoE
        .byte >SpiceWareLogoF
        
ObjectPosition96:
        ; even frame
        .byte 32    ; P0 - left side of 96 pixel display
        .byte 80    ; P1 - right side of 96 pixel display
        .byte 24+1  ; M0 - hide P0 gaps on left side - missiles are 1 off from players
        .byte 72+1  ; M0 - hide P1 gaps on right side    
        ; odd frame        
        .byte 40    ; P0 - left side of 96 pixel display
        .byte 88    ; P1 - right side of 96 pixel display
        .byte 32+1  ; M0 - hide P0 gaps on left side
        .byte 80+1  ; M0 - hide P1 gaps on right side    

LogoLow:
        ; even frame
        .byte #<DraconianLogoP0Aeven
        .byte #<DraconianLogoP0Beven
        .byte #<DraconianLogoP0Ceven
        .byte #<DraconianLogoP1Aeven
        .byte #<DraconianLogoP1Beven
        .byte #<DraconianLogoP1Ceven
        .byte #<DraconianLogoColor
        .byte #<DraconianLogoPF2
        
        ; odd frame
        .byte #<DraconianLogoP0Aodd
        .byte #<DraconianLogoP0Bodd
        .byte #<DraconianLogoP0Codd
        .byte #<DraconianLogoP1Aodd
        .byte #<DraconianLogoP1Bodd
        .byte #<DraconianLogoP1Codd
        .byte #<DraconianLogoColor
        .byte #<DraconianLogoPF2
        
LogoHi:
        ; even frame
        .byte #>DraconianLogoP0Aeven
        .byte #>DraconianLogoP0Beven
        .byte #>DraconianLogoP0Ceven
        .byte #>DraconianLogoP1Aeven
        .byte #>DraconianLogoP1Beven
        .byte #>DraconianLogoP1Ceven
        .byte #>DraconianLogoColor
        .byte #>DraconianLogoPF2
        
        ; odd frame
        .byte #>DraconianLogoP0Aodd
        .byte #>DraconianLogoP0Bodd
        .byte #>DraconianLogoP0Codd
        .byte #>DraconianLogoP1Aodd
        .byte #>DraconianLogoP1Bodd
        .byte #>DraconianLogoP1Codd
        .byte #>DraconianLogoColor
        .byte #>DraconianLogoPF2        
        
InitLevel:              ; screen jitter occurs if InitLevel() is run during
        ldx #188        ; OverScan, so this is set up to mid-screen
        stx TIM64T
;        ldx #WHITE      ; for testing, make background white to show how
;        stx COLUBK      ; long the InitLevel takes to run
        ldx #5
        jsr CallArmSub
        ; Radar data is immediately after GRP0 and GRP1 data for the main
        ; display area.  Since we didn't read it, we need to reset the
        ; data fetcher pointers.
        ldx #< [Player0DataStream + DATASTREAM_SIZE]
        stx DF0LOW
        ldx #> [Player0DataStream + DATASTREAM_SIZE]
        stx DF0HI
        ldx #< [Player1DataStream + DATASTREAM_SIZE]
        stx DF1LOW
        ldx #> [Player1DataStream + DATASTREAM_SIZE]
        stx DF1HI
;        ldx #0
;        stx COLUBK
ILwait:
        sta WSYNC       ; 
        bit TIMINT      ; 4  4
        bpl ILwait      ; 2  6
        dec Sleep5      ; 5 11
        dec Sleep5      ; 5 16
        jmp KernelDone  ; 3 19
        
        
        

        echo "------",($FFF6 - *) , "bytes of BANK 5 ROM left"
                
        ORG $5FF6 + HM_DRIVER_SIZE
        RORG $FFF6
SelectBank0 .byte $00   ; can store ARM data here
SelectBank1 .byte $00   ; can store ARM data here
SelectBank2 .byte $00   ; can store ARM data here
SelectBank3 .byte $00   ; can store ARM data here
SelectBank4 .byte $00   ; can store ARM data here
SelectBank5 .byte $00   ; can store ARM data here
        .word InitSystem ; RESET
        .word InitSystem ; IRQ
        
        
;----------------------------------------
; Display Data
;----------------------------------------
; The Display Data bank is copied into RAM when DPC+ initializes the cartridge.
; This allows us to manipulate the data during run-time, but have a known
; starting state when the Atari is first turned on.
;
; Unlike normal Atari VCS/2600 sprite definitions, the sprite data in the
; Display Data bank is stored right-side-up.
;
; NOTE: The initial contents of Display Data is stored in ROM and can be
; accessed by ARM routines.  This can prove a hand ROM saving technique.
;
;----------------------------------------
        ORG $6000 + HM_DRIVER_SIZE
        RORG $0000       
        
DisplayDataRAMstart:
        SEG.U RAM_DD
        ORG DisplayDataRAMstart + $6000 + HM_DRIVER_SIZE]
        RORG DisplayDataRAMstart
        
JumpDataStream: ; must be 2 byte aligned for C code
        repeat DATASTREAM_SIZE
            .word NormalKernel
        repend
        .word KernelDone
        
Player0DataStream:
        ds DATASTREAM_SIZE
        ds 1    ; kernel done, turn off sprite
        ds RADAR_HEIGHT
        
Player1DataStream:
        ds DATASTREAM_SIZE
        ds 1    ; kernel done, turn off sprite
        ds RADAR_HEIGHT
        
MissileBallDataStream:
        ds DATASTREAM_SIZE        
        
HMP0DataStream:
        ds DATASTREAM_SIZE
        
HMP1DataStream:
        ds DATASTREAM_SIZE
        
ColorHMDataStream:        
        ds DATASTREAM_SIZE
        
SizeBallColorDataStream:
        ds DATASTREAM_SIZE
        
MissileRepositionActive:
        ds DATASTREAM_SIZE
        
PF0L:   ds RADAR_HEIGHT        
PF1L:   ds RADAR_HEIGHT
PF2L:   ds RADAR_HEIGHT
PF0R:   ds RADAR_HEIGHT
PF1R:   ds RADAR_HEIGHT
PF2R:   ds RADAR_HEIGHT

StationImages:
        ds StationHorizontalHeight * 8
StationCollisionPods:
;        ds StationHorizontalHeight * 8
StationCollisionStructure:
;        ds StationHorizontalHeight * 8
DisplayDataRAMend:      
        
; this makes the ROM overlap the RAM
    SEG ROM_DD
    ORG [DisplayDataRAMstart + $6000 + HM_DRIVER_SIZE]
    RORG DisplayDataRAMstart
MainMenuGraphics:        

DraconianLogoP0Aeven:
        .byte zzXXXXXXXX ; 0
        .byte zzXXXXXXXX ; 1
        .byte zzXXXXXXXX ; 2
        .byte zzXXXXXXXX ; 3
        .byte zzXXXXXXXX ; 4
        .byte zzXXXXXXXX ; 5
        .byte zzXXXXXXXX ; 6
        .byte zzXXXXXXXX ; 7
        .byte zzXXXXXXXX ; 8
        .byte zzXXXXXXXX ; 9
        .byte zzXXXXXXXX ; 10
        .byte zzXXXXXXXX ; 11
        .byte zzXXXXXXXX ; 12
        .byte zzXXXXXXXX ; 13
        .byte zzXXXXXXXX ; 14
        .byte zzXXXXXXXX ; 15
        .byte zzXXXXXXXX ; 16
        .byte zzXXXXXXXX ; 17
        .byte zzXXXXXXXX ; 18
        .byte zzXXXXXXXX ; 19
        .byte zzXXXXXXXX ; 20
        .byte zzXXXXXXXX ; 21
        .byte zzXXXXXXXX ; 22
        .byte zzXXXXXXXX ; 23
        .byte zzXXXXXXXX ; 24
        .byte zzXXXX____ ; 25
        .byte zz_______X ; 26
        .byte zzXXXX____ ; 27
        .byte zz________ ; 28
        .byte zz_XXX____ ; 29
        .byte zz________ ; 30
        .byte zz_XXX____ ; 31
        .byte zz____XX__ ; 32
        .byte zz__XX___X ; 33
        .byte zz____XXX_ ; 34
        .byte zz__XX___X ; 35
        .byte zz____XXX_ ; 36
        .byte zz___X____ ; 37
        .byte zz____XXX_ ; 38
        .byte zz___X____ ; 39
        .byte zz____XXX_ ; 40
        .byte zz___X___X ; 41
        .byte zz____XXX_ ; 42
        .byte zz___X___X ; 43
        .byte zz____XXX_ ; 44
        .byte zz___X___X ; 45
        .byte zz____XXX_ ; 46
        .byte zz__XX___X ; 47
        .byte zz____XXX_ ; 48
        .byte zz__XX___X ; 49
        .byte zz____XXX_ ; 50
        .byte zz__XX___X ; 51
        .byte zz____XX__ ; 52
        .byte zz_XXX___X ; 53
        .byte zz____XX__ ; 54
        .byte zz_XXX___X ; 55
        .byte zz____X___ ; 56
        .byte zzXXXX___X ; 57
        .byte zz____X___ ; 58
        .byte zzXXXX___X ; 59
        .byte zz________ ; 60
        .byte zzXXXX___X ; 61
        .byte zz_______X ; 62
        .byte zzXXXX___X ; 63
        .byte zz_______X ; 64
        .byte zzXXXX___X ; 65
        .byte zz______XX ; 66
        .byte zzXXXX__XX ; 67
        .byte zz______XX ; 68
        .byte zzXXXX_XXX ; 69
        .byte zz_____XXX ; 70
        .byte zzXXXXXXXX ; 71
        .byte zz_____XXX ; 72
        .byte zzXXXXXXXX ; 73
        .byte zz____XXXX ; 74
        .byte zzXXXXXXXX ; 75
        .byte zz____XXXX ; 76
        .byte zzXXXXXXXX ; 77
        .byte zz___XXXXX ; 78
        .byte zzXXXXXXXX ; 79
        .byte zz___XXXXX ; 80
        .byte zzXXXXXXXX ; 81
        .byte zz__XXXXXX ; 82
        .byte zzXXXXXXXX ; 83
        .byte zz__XXXXXX ; 84
        .byte zzXXXXXXXX ; 85
        .byte zz_XXXXXXX ; 86
        .byte zzXXXXXXXX ; 87
        .byte zz_XXXXXXX ; 88
        .byte zzXXXXXXXX ; 89

DraconianLogoP0Beven:
        .byte zzXXXXXXXX ; 0
        .byte zzXXXXXXXX ; 1
        .byte zzXXXXXXXX ; 2
        .byte zzXXXXXXXX ; 3
        .byte zzXXXXXXXX ; 4
        .byte zzXXXXXXXX ; 5
        .byte zzXXXXXXXX ; 6
        .byte zzXXXXXXXX ; 7
        .byte zzXXXXXXXX ; 8
        .byte zzXXXXXXXX ; 9
        .byte zzXXXXXXXX ; 10
        .byte zzXXXXXXXX ; 11
        .byte zzXXXXXXXX ; 12
        .byte zzXXXXXXXX ; 13
        .byte zzXXXXXXXX ; 14
        .byte zzXXXXXXXX ; 15
        .byte zzXXXXXXXX ; 16
        .byte zzXXXXXXX_ ; 17
        .byte zzXXXXXXXX ; 18
        .byte zzXXXXXXX_ ; 19
        .byte zzXXXXXXXX ; 20
        .byte zzXXXXXXX_ ; 21
        .byte zzXXXXXXXX ; 22
        .byte zzXXXXXXX_ ; 23
        .byte zzXXXXXXXX ; 24
        .byte zzXXXXXXX_ ; 25
        .byte zz__XXXXXX ; 26
        .byte zz___XXXX_ ; 27
        .byte zz___XXXXX ; 28
        .byte zz___XXXX_ ; 29
        .byte zz___XXXX_ ; 30
        .byte zz____XXX_ ; 31
        .byte zz___XXXX_ ; 32
        .byte zz_____XX_ ; 33
        .byte zzX__XXX__ ; 34
        .byte zz_____XX_ ; 35
        .byte zz___XXX__ ; 36
        .byte zz_____XX_ ; 37
        .byte zz__XXXX__ ; 38
        .byte zz_X___XX_ ; 39
        .byte zz___XX___ ; 40
        .byte zz_X____X_ ; 41
        .byte zz___XX___ ; 42
        .byte zzXXX___X_ ; 43
        .byte zzX___X___ ; 44
        .byte zzXXX___X_ ; 45
        .byte zzX___X___ ; 46
        .byte zz______X_ ; 47
        .byte zzX___X___ ; 48
        .byte zz______X_ ; 49
        .byte zzX___X___ ; 50
        .byte zz______X_ ; 51
        .byte zzX___X___ ; 52
        .byte zzXXX___X_ ; 53
        .byte zzX___X___ ; 54
        .byte zzXXX___X_ ; 55
        .byte zzX___X___ ; 56
        .byte zzXXX___X_ ; 57
        .byte zzX___X___ ; 58
        .byte zzXXX___X_ ; 59
        .byte zzX___X___ ; 60
        .byte zzXXX___X_ ; 61
        .byte zzX___X___ ; 62
        .byte zzXXX___X_ ; 63
        .byte zzX___X___ ; 64
        .byte zzXXXXXXXX ; 65
        .byte zzXXXXXXXX ; 66
        .byte zzXXXXXXXX ; 67
        .byte zzXXXXXXXX ; 68
        .byte zzXXXXXXXX ; 69
        .byte zzXXXXXXXX ; 70
        .byte zzXXXXXXXX ; 71
        .byte zzXXXXXXXX ; 72
        .byte zzXXXXXXXX ; 73
        .byte zzXXXXXXXX ; 74
        .byte zzXXXXXXXX ; 75
        .byte zzXXXXXXXX ; 76
        .byte zzXXXXXXXX ; 77
        .byte zzXXXXXXXX ; 78
        .byte zzXXXXXXXX ; 79
        .byte zzXXXXXXXX ; 80
        .byte zzXXXXXXXX ; 81
        .byte zzXXXXXXXX ; 82
        .byte zzXXXXXXXX ; 83
        .byte zzXXXXXXXX ; 84
        .byte zzXXXXXXXX ; 85
        .byte zzXXXXXXXX ; 86
        .byte zzXXXXXXXX ; 87
        .byte zzXXXXXXXX ; 88
        .byte zzXXXXXXXX ; 89

DraconianLogoP0Ceven:
        .byte zzXXXXXXXX ; 0
        .byte zzXXXXXXX_ ; 1
        .byte zzXXXXXXXX ; 2
        .byte zzXXXXX___ ; 3
        .byte zzXXXXXXXX ; 4
        .byte zzXXX_____ ; 5
        .byte zzXXXXXXXX ; 6
        .byte zzX_______ ; 7
        .byte zzXXXXXXXX ; 8
        .byte zz_______X ; 9
        .byte zzXXXXXX__ ; 10
        .byte zz_____XXX ; 11
        .byte zzXXXX____ ; 12
        .byte zz___XXX__ ; 13
        .byte zzXX______ ; 14
        .byte zz_XXX____ ; 15
        .byte zz________ ; 16
        .byte zzXX______ ; 17
        .byte zz______XX ; 18
        .byte zz________ ; 19
        .byte zz____XXX_ ; 20
        .byte zz______XX ; 21
        .byte zz___XX___ ; 22
        .byte zz____XXXX ; 23
        .byte zz___X____ ; 24
        .byte zz__XXXXX_ ; 25
        .byte zz___X____ ; 26
        .byte zzXXXXXXX_ ; 27
        .byte zz___X___X ; 28
        .byte zzXXXXXX__ ; 29
        .byte zz___X___X ; 30
        .byte zzXXX__XXX ; 31
        .byte zz___X___X ; 32
        .byte zzXX___XXX ; 33
        .byte zz___X___X ; 34
        .byte zzXX___X__ ; 35
        .byte zz___X___X ; 36
        .byte zzXXX__XXX ; 37
        .byte zz___X___X ; 38
        .byte zzXXXXXXX_ ; 39
        .byte zz___X___X ; 40
        .byte zzXXXXXXX_ ; 41
        .byte zz___X___X ; 42
        .byte zzXXX__XXX ; 43
        .byte zz___X___X ; 44
        .byte zzXX___X__ ; 45
        .byte zz___X___X ; 46
        .byte zzXX___XXX ; 47
        .byte zz___X___X ; 48
        .byte zzXXX__XXX ; 49
        .byte zz___X___X ; 50
        .byte zzXXXXXX__ ; 51
        .byte zz___X___X ; 52
        .byte zzXXXXXXX_ ; 53
        .byte zz___X____ ; 54
        .byte zz_XXXXXX_ ; 55
        .byte zz___X____ ; 56
        .byte zz___XXXXX ; 57
        .byte zz___X____ ; 58
        .byte zz_____XXX ; 59
        .byte zz___XXX__ ; 60
        .byte zz_______X ; 61
        .byte zz_____XXX ; 62
        .byte zzX_______ ; 63
        .byte zz_______X ; 64
        .byte zzXXX_____ ; 65
        .byte zzX_______ ; 66
        .byte zz__XXX___ ; 67
        .byte zzXXX_____ ; 68
        .byte zz____XXX_ ; 69
        .byte zzXXXXX___ ; 70
        .byte zz______XX ; 71
        .byte zzXXXXXXX_ ; 72
        .byte zz________ ; 73
        .byte zzXXXXXXXX ; 74
        .byte zzXX______ ; 75
        .byte zzXXXXXXXX ; 76
        .byte zzXXXX____ ; 77
        .byte zzXXXXXXXX ; 78
        .byte zzXXXXXX__ ; 79
        .byte zzXXXXXXXX ; 80
        .byte zzXXXXXXXX ; 81
        .byte zzXXXXXXXX ; 82
        .byte zzXXXXXXXX ; 83
        .byte zzXXXXXXXX ; 84
        .byte zzXXXXXXXX ; 85
        .byte zzXXXXXXXX ; 86
        .byte zzXXXXXXXX ; 87
        .byte zzXXXXXXXX ; 88
        .byte zzXXXXXXXX ; 89

DraconianLogoP1Aeven:
        .byte zz__XXXXXX ; 0
        .byte zzXXXXXXXX ; 1
        .byte zz____XXXX ; 2
        .byte zzXXXXXXXX ; 3
        .byte zz______XX ; 4
        .byte zzXXXXXXXX ; 5
        .byte zz________ ; 6
        .byte zz_XXXXXXX ; 7
        .byte zzXX______ ; 8
        .byte zz___XXXXX ; 9
        .byte zzXXXX____ ; 10
        .byte zz_____XXX ; 11
        .byte zz___XXX__ ; 12
        .byte zz______XX ; 13
        .byte zz_____XXX ; 14
        .byte zzX_____XX ; 15
        .byte zz_______X ; 16
        .byte zzXXX___XX ; 17
        .byte zz________ ; 18
        .byte zz__XXX_XX ; 19
        .byte zzXXX_____ ; 20
        .byte zz____XXXX ; 21
        .byte zzXXXXX___ ; 22
        .byte zz______XX ; 23
        .byte zz__XXXXX_ ; 24
        .byte zz______XX ; 25
        .byte zz__XXXXXX ; 26
        .byte zzXX____X_ ; 27
        .byte zz__XXXXXX ; 28
        .byte zzXXX___X_ ; 29
        .byte zz__XX_XXX ; 30
        .byte zzXXX___X_ ; 31
        .byte zz__XX____ ; 32
        .byte zzXXX___X_ ; 33
        .byte zz__XX__X_ ; 34
        .byte zzXXX___X_ ; 35
        .byte zz__XX__X_ ; 36
        .byte zzXXX___X_ ; 37
        .byte zz___XXXXX ; 38
        .byte zzXXX___X_ ; 39
        .byte zz_X_XXXXX ; 40
        .byte zzXXX___X_ ; 41
        .byte zz_X_XXXXX ; 42
        .byte zzXXX___X_ ; 43
        .byte zz__XX____ ; 44
        .byte zzXXX___X_ ; 45
        .byte zz__XX__X_ ; 46
        .byte zzXXX___X_ ; 47
        .byte zz__XX__X_ ; 48
        .byte zzXXX___X_ ; 49
        .byte zz__XX_XXX ; 50
        .byte zzXXX___X_ ; 51
        .byte zz__XXXXXX ; 52
        .byte zzXXX___X_ ; 53
        .byte zz__XXXXXX ; 54
        .byte zzX_____X_ ; 55
        .byte zz__XXXXXX ; 56
        .byte zz______X_ ; 57
        .byte zzXXXXXX__ ; 58
        .byte zz_____XX_ ; 59
        .byte zzXXXX____ ; 60
        .byte zz___XXXX_ ; 61
        .byte zzXX______ ; 62
        .byte zz_XXX__X_ ; 63
        .byte zz________ ; 64
        .byte zzXX____XX ; 65
        .byte zz______XX ; 66
        .byte zz______XX ; 67
        .byte zz____XXX_ ; 68
        .byte zz______XX ; 69
        .byte zz__XXX___ ; 70
        .byte zz____XXXX ; 71
        .byte zzXXX_____ ; 72
        .byte zz__XXXXXX ; 73
        .byte zz________ ; 74
        .byte zzXXXXXXXX ; 75
        .byte zz_______X ; 76
        .byte zzXXXXXXXX ; 77
        .byte zz_____XXX ; 78
        .byte zzXXXXXXXX ; 79
        .byte zz___XXXXX ; 80
        .byte zzXXXXXXXX ; 81
        .byte zzXXXXXXXX ; 82
        .byte zzXXXXXXXX ; 83
        .byte zzXXXXXXXX ; 84
        .byte zzXXXXXXXX ; 85
        .byte zzXXXXXXXX ; 86
        .byte zzXXXXXXXX ; 87
        .byte zzXXXXXXXX ; 88
        .byte zzXXXXXXXX ; 89

DraconianLogoP1Beven:
        .byte zzXXXXXXXX ; 0
        .byte zzXXXXXXXX ; 1
        .byte zzXXXXXXXX ; 2
        .byte zzXXXXXXXX ; 3
        .byte zzXXXXXXXX ; 4
        .byte zzXXXXXXXX ; 5
        .byte zzXXXXXXXX ; 6
        .byte zzXXXXXXXX ; 7
        .byte zzXXXXXXXX ; 8
        .byte zzXXXXXXXX ; 9
        .byte zzXXXXXXXX ; 10
        .byte zzXXXXXXXX ; 11
        .byte zzXXXXXXXX ; 12
        .byte zzXXXXXXXX ; 13
        .byte zzXXXXXXXX ; 14
        .byte zzXXXXXXXX ; 15
        .byte zzXXXXXXXX ; 16
        .byte zzXXXXXXXX ; 17
        .byte zzXXXXXXXX ; 18
        .byte zzXXXXXXXX ; 19
        .byte zzXXXXXXXX ; 20
        .byte zzXXXXXXXX ; 21
        .byte zzXXXXXXXX ; 22
        .byte zzXXXXXXXX ; 23
        .byte zzXXXXXXXX ; 24
        .byte zzX___XXXX ; 25
        .byte zzXXXXX___ ; 26
        .byte zzX___XXXX ; 27
        .byte zz__XXX___ ; 28
        .byte zzX___XXXX ; 29
        .byte zz___XX___ ; 30
        .byte zzX___XXX_ ; 31
        .byte zz____X___ ; 32
        .byte zzX___XX__ ; 33
        .byte zz________ ; 34
        .byte zzX___XX__ ; 35
        .byte zz________ ; 36
        .byte zzX___XX__ ; 37
        .byte zz________ ; 38
        .byte zzX___X___ ; 39
        .byte zz________ ; 40
        .byte zzX___X___ ; 41
        .byte zz__X_____ ; 42
        .byte zzX___X___ ; 43
        .byte zz__XX____ ; 44
        .byte zzX___X___ ; 45
        .byte zz__XXX___ ; 46
        .byte zzX_______ ; 47
        .byte zz__XXX___ ; 48
        .byte zzX_______ ; 49
        .byte zz__XXX___ ; 50
        .byte zzX_______ ; 51
        .byte zz__XXX___ ; 52
        .byte zzX___X___ ; 53
        .byte zz__XXX___ ; 54
        .byte zzX___X___ ; 55
        .byte zz__XXX___ ; 56
        .byte zzX___X___ ; 57
        .byte zz__XXX___ ; 58
        .byte zzX___X___ ; 59
        .byte zz__XXX___ ; 60
        .byte zzX___X___ ; 61
        .byte zz__XXX___ ; 62
        .byte zzX___X___ ; 63
        .byte zz__XXX___ ; 64
        .byte zzXXXXXXXX ; 65
        .byte zzXXXXXXXX ; 66
        .byte zzXXXXXXXX ; 67
        .byte zzXXXXXXXX ; 68
        .byte zzXXXXXXXX ; 69
        .byte zzXXXXXXXX ; 70
        .byte zzXXXXXXXX ; 71
        .byte zzXXXXXXXX ; 72
        .byte zzXXXXXXXX ; 73
        .byte zzXXXXXXXX ; 74
        .byte zzXXXXXXXX ; 75
        .byte zzXXXXXXXX ; 76
        .byte zzXXXXXXXX ; 77
        .byte zzXXXXXXXX ; 78
        .byte zzXXXXXXXX ; 79
        .byte zzXXXXXXXX ; 80
        .byte zzXXXXXXXX ; 81
        .byte zzXXXXXXXX ; 82
        .byte zzXXXXXXXX ; 83
        .byte zzXXXXXXXX ; 84
        .byte zzXXXXXXXX ; 85
        .byte zzXXXXXXXX ; 86
        .byte zzXXXXXXXX ; 87
        .byte zzXXXXXXXX ; 88
        .byte zzXXXXXXXX ; 89

DraconianLogoP1Ceven:
        .byte zzXXXXXXXX ; 0
        .byte zzXXXXXXXX ; 1
        .byte zzXXXXXXXX ; 2
        .byte zzXXXXXXXX ; 3
        .byte zzXXXXXXXX ; 4
        .byte zzXXXXXXXX ; 5
        .byte zzXXXXXXXX ; 6
        .byte zzXXXXXXXX ; 7
        .byte zzXXXXXXXX ; 8
        .byte zzXXXXXXXX ; 9
        .byte zzXXXXXXXX ; 10
        .byte zzXXXXXXXX ; 11
        .byte zzXXXXXXXX ; 12
        .byte zzXXXXXXXX ; 13
        .byte zzXXXXXXXX ; 14
        .byte zzXXXXXXXX ; 15
        .byte zzXXXXXXXX ; 16
        .byte zzXXXXXXXX ; 17
        .byte zzXXXXXXXX ; 18
        .byte zzXXXXXXXX ; 19
        .byte zzXXXXXXXX ; 20
        .byte zzXXXXXXXX ; 21
        .byte zzXXXXXXXX ; 22
        .byte zzXXXXXXXX ; 23
        .byte zzXXXXXXXX ; 24
        .byte zzXXXXX___ ; 25
        .byte zzX_XXXXX_ ; 26
        .byte zz_XXXX___ ; 27
        .byte zz___XXXX_ ; 28
        .byte zz___XX___ ; 29
        .byte zz____XXX_ ; 30
        .byte zz____X___ ; 31
        .byte zz____XXX_ ; 32
        .byte zz________ ; 33
        .byte zz_____XX_ ; 34
        .byte zz________ ; 35
        .byte zz_____XX_ ; 36
        .byte zz________ ; 37
        .byte zz_X___XX_ ; 38
        .byte zz________ ; 39
        .byte zz_X____X_ ; 40
        .byte zz________ ; 41
        .byte zzXXX___X_ ; 42
        .byte zz__XX____ ; 43
        .byte zzXXX___X_ ; 44
        .byte zz__XXX___ ; 45
        .byte zzXXX___X_ ; 46
        .byte zz__XXX___ ; 47
        .byte zz______X_ ; 48
        .byte zz__XXX___ ; 49
        .byte zz______X_ ; 50
        .byte zz__XXX___ ; 51
        .byte zzXXX___X_ ; 52
        .byte zz__XXX___ ; 53
        .byte zzXXX___X_ ; 54
        .byte zz__XXX___ ; 55
        .byte zzXXX___X_ ; 56
        .byte zz__XXX___ ; 57
        .byte zzXXX___X_ ; 58
        .byte zz__XXX___ ; 59
        .byte zzXXX___X_ ; 60
        .byte zz__XXX___ ; 61
        .byte zzXXX___X_ ; 62
        .byte zz__XXX___ ; 63
        .byte zzXXX___X_ ; 64
        .byte zzXXXXXXXX ; 65
        .byte zzXXXXXXXX ; 66
        .byte zzXXXXXXXX ; 67
        .byte zzXXXXXXXX ; 68
        .byte zzXXXXXXXX ; 69
        .byte zzXXXXXXXX ; 70
        .byte zzXXXXXXXX ; 71
        .byte zzXXXXXXXX ; 72
        .byte zzXXXXXXXX ; 73
        .byte zzXXXXXXXX ; 74
        .byte zzXXXXXXXX ; 75
        .byte zzXXXXXXXX ; 76
        .byte zzXXXXXXXX ; 77
        .byte zzXXXXXXXX ; 78
        .byte zzXXXXXXXX ; 79
        .byte zzXXXXXXXX ; 80
        .byte zzXXXXXXXX ; 81
        .byte zzXXXXXXXX ; 82
        .byte zzXXXXXXXX ; 83
        .byte zzXXXXXXXX ; 84
        .byte zzXXXXXXXX ; 85
        .byte zzXXXXXXXX ; 86
        .byte zzXXXXXXXX ; 87
        .byte zzXXXXXXXX ; 88
        .byte zzXXXXXXXX ; 89


DraconianLogoP0Aodd:
        .byte zzXXXXXXXX ; 0
        .byte zzXXXXXXXX ; 1
        .byte zzXXXXXXXX ; 2
        .byte zzXXXXXXXX ; 3
        .byte zzXXXXXXXX ; 4
        .byte zzXXXXXXXX ; 5
        .byte zzXXXXXXXX ; 6
        .byte zzXXXXXXXX ; 7
        .byte zzXXXXXXXX ; 8
        .byte zzXXXXXXXX ; 9
        .byte zzXXXXXXXX ; 10
        .byte zzXXXXXXXX ; 11
        .byte zzXXXXXXXX ; 12
        .byte zzXXXXXXXX ; 13
        .byte zzXXXXXXXX ; 14
        .byte zzXXXXXXXX ; 15
        .byte zzXXXXXXXX ; 16
        .byte zzXXXXXXXX ; 17
        .byte zzXXXXXXXX ; 18
        .byte zzXXXXXXXX ; 19
        .byte zzXXXXXXXX ; 20
        .byte zzXXXXXXXX ; 21
        .byte zzXXXXXXXX ; 22
        .byte zzXXXXXXXX ; 23
        .byte zzXXXXXXXX ; 24
        .byte zz______XX ; 25
        .byte zzXXXX____ ; 26
        .byte zz________ ; 27
        .byte zzXXXX____ ; 28
        .byte zz________ ; 29
        .byte zz_XXX____ ; 30
        .byte zz________ ; 31
        .byte zz__XX___X ; 32
        .byte zz____XX__ ; 33
        .byte zz__XX___X ; 34
        .byte zz____XXX_ ; 35
        .byte zz___X___X ; 36
        .byte zz____XXX_ ; 37
        .byte zz___X____ ; 38
        .byte zz____XXX_ ; 39
        .byte zz___X____ ; 40
        .byte zz____XXX_ ; 41
        .byte zz___X___X ; 42
        .byte zz____XXX_ ; 43
        .byte zz___X___X ; 44
        .byte zz____XXX_ ; 45
        .byte zz__XX___X ; 46
        .byte zz____XXX_ ; 47
        .byte zz__XX___X ; 48
        .byte zz____XXX_ ; 49
        .byte zz__XX___X ; 50
        .byte zz____XX__ ; 51
        .byte zz_XXX___X ; 52
        .byte zz____XX__ ; 53
        .byte zz_XXX___X ; 54
        .byte zz____X___ ; 55
        .byte zz_XXX___X ; 56
        .byte zz____X___ ; 57
        .byte zzXXXX___X ; 58
        .byte zz________ ; 59
        .byte zzXXXX___X ; 60
        .byte zz________ ; 61
        .byte zzXXXX___X ; 62
        .byte zz_______X ; 63
        .byte zzXXXX___X ; 64
        .byte zz_______X ; 65
        .byte zzXXXX__XX ; 66
        .byte zz______XX ; 67
        .byte zzXXXX_XXX ; 68
        .byte zz_____XXX ; 69
        .byte zzXXXX_XXX ; 70
        .byte zz_____XXX ; 71
        .byte zzXXXXXXXX ; 72
        .byte zz____XXXX ; 73
        .byte zzXXXXXXXX ; 74
        .byte zz____XXXX ; 75
        .byte zzXXXXXXXX ; 76
        .byte zz___XXXXX ; 77
        .byte zzXXXXXXXX ; 78
        .byte zz___XXXXX ; 79
        .byte zzXXXXXXXX ; 80
        .byte zz__XXXXXX ; 81
        .byte zzXXXXXXXX ; 82
        .byte zz__XXXXXX ; 83
        .byte zzXXXXXXXX ; 84
        .byte zz__XXXXXX ; 85
        .byte zzXXXXXXXX ; 86
        .byte zz_XXXXXXX ; 87
        .byte zzXXXXXXXX ; 88
        .byte zz_XXXXXXX ; 89

DraconianLogoP0Bodd:
        .byte zzXXXXXXXX ; 0
        .byte zzXXXXXXXX ; 1
        .byte zzXXXXXXXX ; 2
        .byte zzXXXXXXXX ; 3
        .byte zzXXXXXXXX ; 4
        .byte zzXXXXXXXX ; 5
        .byte zzXXXXXXXX ; 6
        .byte zzXXXXXXXX ; 7
        .byte zzXXXXXXXX ; 8
        .byte zzXXXXXXXX ; 9
        .byte zzXXXXXXXX ; 10
        .byte zzXXXXXXXX ; 11
        .byte zzXXXXXXXX ; 12
        .byte zzXXXXXXXX ; 13
        .byte zzXXXXXXXX ; 14
        .byte zzXXXXXXXX ; 15
        .byte zzXXXXXXXX ; 16
        .byte zzXXXXXXXX ; 17
        .byte zzXXXXXXX_ ; 18
        .byte zzXXXXXXXX ; 19
        .byte zzXXXXXXX_ ; 20
        .byte zzXXXXXXXX ; 21
        .byte zzXXXXXXX_ ; 22
        .byte zzXXXXXXXX ; 23
        .byte zzXXXXXXX_ ; 24
        .byte zz_XXXXXXX ; 25
        .byte zzX_XXXXX_ ; 26
        .byte zz__XXXXXX ; 27
        .byte zz___XXXX_ ; 28
        .byte zz___XXXXX ; 29
        .byte zz____XXX_ ; 30
        .byte zz___XXXX_ ; 31
        .byte zz____XXX_ ; 32
        .byte zzX__XXX__ ; 33
        .byte zz_____XX_ ; 34
        .byte zzX__XXX__ ; 35
        .byte zz_____XX_ ; 36
        .byte zz__XXXX__ ; 37
        .byte zz_X___XX_ ; 38
        .byte zz___XX___ ; 39
        .byte zz_X____X_ ; 40
        .byte zz___XX___ ; 41
        .byte zzXXX___X_ ; 42
        .byte zzX___X___ ; 43
        .byte zzXXX___X_ ; 44
        .byte zzX___X___ ; 45
        .byte zzXXX___X_ ; 46
        .byte zzX___X___ ; 47
        .byte zz______X_ ; 48
        .byte zzX___X___ ; 49
        .byte zz______X_ ; 50
        .byte zzX___X___ ; 51
        .byte zzXXX___X_ ; 52
        .byte zzX___X___ ; 53
        .byte zzXXX___X_ ; 54
        .byte zzX___X___ ; 55
        .byte zzXXX___X_ ; 56
        .byte zzX___X___ ; 57
        .byte zzXXX___X_ ; 58
        .byte zzX___X___ ; 59
        .byte zzXXX___X_ ; 60
        .byte zzX___X___ ; 61
        .byte zzXXX___X_ ; 62
        .byte zzX___X___ ; 63
        .byte zzXXX___X_ ; 64
        .byte zzXXXXXXXX ; 65
        .byte zzXXXXXXXX ; 66
        .byte zzXXXXXXXX ; 67
        .byte zzXXXXXXXX ; 68
        .byte zzXXXXXXXX ; 69
        .byte zzXXXXXXXX ; 70
        .byte zzXXXXXXXX ; 71
        .byte zzXXXXXXXX ; 72
        .byte zzXXXXXXXX ; 73
        .byte zzXXXXXXXX ; 74
        .byte zzXXXXXXXX ; 75
        .byte zzXXXXXXXX ; 76
        .byte zzXXXXXXXX ; 77
        .byte zzXXXXXXXX ; 78
        .byte zzXXXXXXXX ; 79
        .byte zzXXXXXXXX ; 80
        .byte zzXXXXXXXX ; 81
        .byte zzXXXXXXXX ; 82
        .byte zzXXXXXXXX ; 83
        .byte zzXXXXXXXX ; 84
        .byte zzXXXXXXXX ; 85
        .byte zzXXXXXXXX ; 86
        .byte zzXXXXXXXX ; 87
        .byte zzXXXXXXXX ; 88
        .byte zzXXXXXXXX ; 89

DraconianLogoP0Codd:
        .byte zzXXXXXXXX ; 0
        .byte zzXXXXXXXX ; 1
        .byte zzXXXXXX__ ; 2
        .byte zzXXXXXXXX ; 3
        .byte zzXXXX____ ; 4
        .byte zzXXXXXXXX ; 5
        .byte zzXX______ ; 6
        .byte zzXXXXXXXX ; 7
        .byte zz________ ; 8
        .byte zzXXXXXXX_ ; 9
        .byte zz______XX ; 10
        .byte zzXXXXX___ ; 11
        .byte zz____XXX_ ; 12
        .byte zzXXX_____ ; 13
        .byte zz__XXX___ ; 14
        .byte zzX_______ ; 15
        .byte zzXXX_____ ; 16
        .byte zz_______X ; 17
        .byte zzX_______ ; 18
        .byte zz_____XXX ; 19
        .byte zz_______X ; 20
        .byte zz___XXX__ ; 21
        .byte zz_____XXX ; 22
        .byte zz___X____ ; 23
        .byte zz___XXXXX ; 24
        .byte zz___X____ ; 25
        .byte zz_XXXXXX_ ; 26
        .byte zz___X____ ; 27
        .byte zzXXXXXXX_ ; 28
        .byte zz___X___X ; 29
        .byte zzXXXXX_XX ; 30
        .byte zz___X___X ; 31
        .byte zzXX____XX ; 32
        .byte zz___X___X ; 33
        .byte zzXX__X_XX ; 34
        .byte zz___X___X ; 35
        .byte zzXX__X_XX ; 36
        .byte zz___X___X ; 37
        .byte zzXXXXXXX_ ; 38
        .byte zz___X___X ; 39
        .byte zzXXXXXXX_ ; 40
        .byte zz___X___X ; 41
        .byte zzXXXXXXX_ ; 42
        .byte zz___X___X ; 43
        .byte zzXX____XX ; 44
        .byte zz___X___X ; 45
        .byte zzXX__X_XX ; 46
        .byte zz___X___X ; 47
        .byte zzXX__X_XX ; 48
        .byte zz___X___X ; 49
        .byte zzXXXXX_XX ; 50
        .byte zz___X___X ; 51
        .byte zzXXXXXXX_ ; 52
        .byte zz___X___X ; 53
        .byte zzXXXXXXX_ ; 54
        .byte zz___X____ ; 55
        .byte zz__XXXXXX ; 56
        .byte zz___X____ ; 57
        .byte zz____XXXX ; 58
        .byte zz___XX___ ; 59
        .byte zz______XX ; 60
        .byte zz____XXX_ ; 61
        .byte zz________ ; 62
        .byte zz______XX ; 63
        .byte zzXX______ ; 64
        .byte zz________ ; 65
        .byte zz_XXX____ ; 66
        .byte zzXX______ ; 67
        .byte zz___XXX__ ; 68
        .byte zzXXXX____ ; 69
        .byte zz_____XXX ; 70
        .byte zzXXXXXX__ ; 71
        .byte zz_______X ; 72
        .byte zzXXXXXXXX ; 73
        .byte zzX_______ ; 74
        .byte zzXXXXXXXX ; 75
        .byte zzXXX_____ ; 76
        .byte zzXXXXXXXX ; 77
        .byte zzXXXXX___ ; 78
        .byte zzXXXXXXXX ; 79
        .byte zzXXXXXXX_ ; 80
        .byte zzXXXXXXXX ; 81
        .byte zzXXXXXXXX ; 82
        .byte zzXXXXXXXX ; 83
        .byte zzXXXXXXXX ; 84
        .byte zzXXXXXXXX ; 85
        .byte zzXXXXXXXX ; 86
        .byte zzXXXXXXXX ; 87
        .byte zzXXXXXXXX ; 88
        .byte zzXXXXXXXX ; 89

DraconianLogoP1Aodd:
        .byte zzXXXXXXXX ; 0
        .byte zz___XXXXX ; 1
        .byte zzXXXXXXXX ; 2
        .byte zz_____XXX ; 3
        .byte zzXXXXXXXX ; 4
        .byte zz_______X ; 5
        .byte zzXXXXXXXX ; 6
        .byte zz________ ; 7
        .byte zz__XXXXXX ; 8
        .byte zzXXX_____ ; 9
        .byte zz____XXXX ; 10
        .byte zz__XXX___ ; 11
        .byte zz______XX ; 12
        .byte zz____XXX_ ; 13
        .byte zz______XX ; 14
        .byte zz______XX ; 15
        .byte zzXX____XX ; 16
        .byte zz________ ; 17
        .byte zz_XXX__XX ; 18
        .byte zzXX______ ; 19
        .byte zz___XXXXX ; 20
        .byte zzXXXX____ ; 21
        .byte zz_____XXX ; 22
        .byte zzXXXXXX__ ; 23
        .byte zz______XX ; 24
        .byte zz___XXXXX ; 25
        .byte zzX_____X_ ; 26
        .byte zz_X_XXXXX ; 27
        .byte zzXXX___X_ ; 28
        .byte zz_X__XXXX ; 29
        .byte zzXXX___X_ ; 30
        .byte zzXXXXX__X ; 31
        .byte zzXXX___X_ ; 32
        .byte zz__XX___X ; 33
        .byte zzXXX___X_ ; 34
        .byte zzXX_____X ; 35
        .byte zzXXX___X_ ; 36
        .byte zz__XXX__X ; 37
        .byte zzXXX___X_ ; 38
        .byte zz__XXXXXX ; 39
        .byte zzXXX___X_ ; 40
        .byte zz__XXXXXX ; 41
        .byte zzXXX___X_ ; 42
        .byte zz__XXX__X ; 43
        .byte zzXXX___X_ ; 44
        .byte zzXX_____X ; 45
        .byte zzXXX___X_ ; 46
        .byte zz__XX___X ; 47
        .byte zzXXX___X_ ; 48
        .byte zzXXXXX__X ; 49
        .byte zzXXX___X_ ; 50
        .byte zz____XXXX ; 51
        .byte zzXXX___X_ ; 52
        .byte zz_X_XXXXX ; 53
        .byte zzXX____X_ ; 54
        .byte zz_X_XXXXX ; 55
        .byte zz______X_ ; 56
        .byte zzXXXXXXX_ ; 57
        .byte zz______X_ ; 58
        .byte zzXXXXX___ ; 59
        .byte zz____XXX_ ; 60
        .byte zzXXX_____ ; 61
        .byte zz__XXX_X_ ; 62
        .byte zz________ ; 63
        .byte zzXXX___X_ ; 64
        .byte zz_______X ; 65
        .byte zzX_____XX ; 66
        .byte zz_____XXX ; 67
        .byte zz______XX ; 68
        .byte zz___XXX__ ; 69
        .byte zz_____XXX ; 70
        .byte zzXXXX____ ; 71
        .byte zz___XXXXX ; 72
        .byte zzXX______ ; 73
        .byte zz_XXXXXXX ; 74
        .byte zz________ ; 75
        .byte zzXXXXXXXX ; 76
        .byte zz______XX ; 77
        .byte zzXXXXXXXX ; 78
        .byte zz____XXXX ; 79
        .byte zzXXXXXXXX ; 80
        .byte zz__XXXXXX ; 81
        .byte zzXXXXXXXX ; 82
        .byte zzXXXXXXXX ; 83
        .byte zzXXXXXXXX ; 84
        .byte zzXXXXXXXX ; 85
        .byte zzXXXXXXXX ; 86
        .byte zzXXXXXXXX ; 87
        .byte zzXXXXXXXX ; 88
        .byte zzXXXXXXXX ; 89

DraconianLogoP1Bodd:
        .byte zzXXXXXXXX ; 0
        .byte zzXXXXXXXX ; 1
        .byte zzXXXXXXXX ; 2
        .byte zzXXXXXXXX ; 3
        .byte zzXXXXXXXX ; 4
        .byte zzXXXXXXXX ; 5
        .byte zzXXXXXXXX ; 6
        .byte zzXXXXXXXX ; 7
        .byte zzXXXXXXXX ; 8
        .byte zzXXXXXXXX ; 9
        .byte zzXXXXXXXX ; 10
        .byte zzXXXXXXXX ; 11
        .byte zzXXXXXXXX ; 12
        .byte zzXXXXXXXX ; 13
        .byte zzXXXXXXXX ; 14
        .byte zzXXXXXXXX ; 15
        .byte zzXXXXXXXX ; 16
        .byte zzXXXXXXXX ; 17
        .byte zzXXXXXXXX ; 18
        .byte zzXXXXXXXX ; 19
        .byte zzXXXXXXXX ; 20
        .byte zzXXXXXXXX ; 21
        .byte zzXXXXXXXX ; 22
        .byte zzXXXXXXXX ; 23
        .byte zzXXXXXXXX ; 24
        .byte zzXXXXX___ ; 25
        .byte zzX___XXXX ; 26
        .byte zz_XXXX___ ; 27
        .byte zzX___XXXX ; 28
        .byte zz___XX___ ; 29
        .byte zzX___XXX_ ; 30
        .byte zz____X___ ; 31
        .byte zzX___XXX_ ; 32
        .byte zz________ ; 33
        .byte zzX___XX__ ; 34
        .byte zz________ ; 35
        .byte zzX___XX__ ; 36
        .byte zz________ ; 37
        .byte zzX___XX__ ; 38
        .byte zz________ ; 39
        .byte zzX___X___ ; 40
        .byte zz________ ; 41
        .byte zzX___X___ ; 42
        .byte zz__XX____ ; 43
        .byte zzX___X___ ; 44
        .byte zz__XXX___ ; 45
        .byte zzX___X___ ; 46
        .byte zz__XXX___ ; 47
        .byte zzX_______ ; 48
        .byte zz__XXX___ ; 49
        .byte zzX_______ ; 50
        .byte zz__XXX___ ; 51
        .byte zzX_______ ; 52
        .byte zz__XXX___ ; 53
        .byte zzX___X___ ; 54
        .byte zz__XXX___ ; 55
        .byte zzX___X___ ; 56
        .byte zz__XXX___ ; 57
        .byte zzX___X___ ; 58
        .byte zz__XXX___ ; 59
        .byte zzX___X___ ; 60
        .byte zz__XXX___ ; 61
        .byte zzX___X___ ; 62
        .byte zz__XXX___ ; 63
        .byte zzX___X___ ; 64
        .byte zzXXXXXXXX ; 65
        .byte zzXXXXXXXX ; 66
        .byte zzXXXXXXXX ; 67
        .byte zzXXXXXXXX ; 68
        .byte zzXXXXXXXX ; 69
        .byte zzXXXXXXXX ; 70
        .byte zzXXXXXXXX ; 71
        .byte zzXXXXXXXX ; 72
        .byte zzXXXXXXXX ; 73
        .byte zzXXXXXXXX ; 74
        .byte zzXXXXXXXX ; 75
        .byte zzXXXXXXXX ; 76
        .byte zzXXXXXXXX ; 77
        .byte zzXXXXXXXX ; 78
        .byte zzXXXXXXXX ; 79
        .byte zzXXXXXXXX ; 80
        .byte zzXXXXXXXX ; 81
        .byte zzXXXXXXXX ; 82
        .byte zzXXXXXXXX ; 83
        .byte zzXXXXXXXX ; 84
        .byte zzXXXXXXXX ; 85
        .byte zzXXXXXXXX ; 86
        .byte zzXXXXXXXX ; 87
        .byte zzXXXXXXXX ; 88
        .byte zzXXXXXXXX ; 89

DraconianLogoP1Codd:
        .byte zzXXXXXXXX ; 0
        .byte zzXXXXXXXX ; 1
        .byte zzXXXXXXXX ; 2
        .byte zzXXXXXXXX ; 3
        .byte zzXXXXXXXX ; 4
        .byte zzXXXXXXXX ; 5
        .byte zzXXXXXXXX ; 6
        .byte zzXXXXXXXX ; 7
        .byte zzXXXXXXXX ; 8
        .byte zzXXXXXXXX ; 9
        .byte zzXXXXXXXX ; 10
        .byte zzXXXXXXXX ; 11
        .byte zzXXXXXXXX ; 12
        .byte zzXXXXXXXX ; 13
        .byte zzXXXXXXXX ; 14
        .byte zzXXXXXXXX ; 15
        .byte zzXXXXXXXX ; 16
        .byte zzXXXXXXXX ; 17
        .byte zzXXXXXXXX ; 18
        .byte zzXXXXXXXX ; 19
        .byte zzXXXXXXXX ; 20
        .byte zzXXXXXXXX ; 21
        .byte zzXXXXXXXX ; 22
        .byte zzXXXXXXXX ; 23
        .byte zzXXXXXXXX ; 24
        .byte zzXXXXXXXX ; 25
        .byte zzXXXXX___ ; 26
        .byte zz___XXXX_ ; 27
        .byte zz__XXX___ ; 28
        .byte zz___XXXX_ ; 29
        .byte zz___XX___ ; 30
        .byte zz____XXX_ ; 31
        .byte zz____X___ ; 32
        .byte zz_____XX_ ; 33
        .byte zz________ ; 34
        .byte zz_____XX_ ; 35
        .byte zz________ ; 36
        .byte zz_____XX_ ; 37
        .byte zz________ ; 38
        .byte zz_X___XX_ ; 39
        .byte zz________ ; 40
        .byte zz_X____X_ ; 41
        .byte zz__X_____ ; 42
        .byte zzXXX___X_ ; 43
        .byte zz__XX____ ; 44
        .byte zzXXX___X_ ; 45
        .byte zz__XXX___ ; 46
        .byte zz______X_ ; 47
        .byte zz__XXX___ ; 48
        .byte zz______X_ ; 49
        .byte zz__XXX___ ; 50
        .byte zz______X_ ; 51
        .byte zz__XXX___ ; 52
        .byte zzXXX___X_ ; 53
        .byte zz__XXX___ ; 54
        .byte zzXXX___X_ ; 55
        .byte zz__XXX___ ; 56
        .byte zzXXX___X_ ; 57
        .byte zz__XXX___ ; 58
        .byte zzXXX___X_ ; 59
        .byte zz__XXX___ ; 60
        .byte zzXXX___X_ ; 61
        .byte zz__XXX___ ; 62
        .byte zzXXX___X_ ; 63
        .byte zz__XXX___ ; 64
        .byte zzXXXXXXXX ; 65
        .byte zzXXXXXXXX ; 66
        .byte zzXXXXXXXX ; 67
        .byte zzXXXXXXXX ; 68
        .byte zzXXXXXXXX ; 69
        .byte zzXXXXXXXX ; 70
        .byte zzXXXXXXXX ; 71
        .byte zzXXXXXXXX ; 72
        .byte zzXXXXXXXX ; 73
        .byte zzXXXXXXXX ; 74
        .byte zzXXXXXXXX ; 75
        .byte zzXXXXXXXX ; 76
        .byte zzXXXXXXXX ; 77
        .byte zzXXXXXXXX ; 78
        .byte zzXXXXXXXX ; 79
        .byte zzXXXXXXXX ; 80
        .byte zzXXXXXXXX ; 81
        .byte zzXXXXXXXX ; 82
        .byte zzXXXXXXXX ; 83
        .byte zzXXXXXXXX ; 84
        .byte zzXXXXXXXX ; 85
        .byte zzXXXXXXXX ; 86
        .byte zzXXXXXXXX ; 87
        .byte zzXXXXXXXX ; 88
        .byte zzXXXXXXXX ; 89           
        
DraconianLogoPF2:
        ds 24,  zr________
        ds 4,   zr_______X
        ds 23,  zr______XX
        ds 6,   zr_______X
        ds 33,  zr________
        
OptionSpace:
        .byte zzXXXXXXXX 
        .byte zzXXXXXXXX 
        .byte zzXXXXXXXX 
        .byte zzXXXXXXXX 
        .byte zzXXXXXXXX 
        .byte zzXXXXXXXX 
        .byte zzXXXXXXXX 
        .byte zzXXXXXXXX     
    
LevelA:
        .byte zz_XXX___X
        .byte zz_XXX_XXX
        .byte zz_XXX_XXX
        .byte zz_XXX__XX
        .byte zz_XXX_XXX
        .byte zz_XXX_XXX
        .byte zz_XXX_XXX
        .byte zz___X___X 
LevelB:
        .byte zz_X_X___X 
        .byte zz_X_X_XXX 
        .byte zz_X_X_XXX
        .byte zz_X_X__XX
        .byte zz_X_X_XXX
        .byte zz_X_X_XXX
        .byte zz_X_X_XXX
        .byte zzX_XX___X
LevelC:
        .byte zz_XXXXXXX 
        .byte zz_XXXXXXX 
        .byte zz_XXXXXXX
        .byte zz_XXXXXXX 
        .byte zz_XXXXXXX
        .byte zz_XXXXXXX
        .byte zz_XXXXXXX
        .byte zz___XXXXX    
        
TVtypeA:
        .byte zz___X_X_X
        .byte zzX_XX_X_X 
        .byte zzX_XX_X_X
        .byte zzX_XX_X_X
        .byte zzX_XX_X_X
        .byte zzX_XX_X_X
        .byte zzX_XX_X_X
        .byte zzX_XXX_XX
        
TVtypeB:
        .byte zzXX___X_X
        .byte zzXXX_XX_X
        .byte zzXXX_XX_X
        .byte zzXXX_XX_X
        .byte zz_XX_XXX_
        .byte zzXXX_XXX_
        .byte zzXXX_XXX_
        .byte zzXXX_XXX_
TVtypeC:
        .byte zz_X__XX__
        .byte zz_X_X_X_X 
        .byte zz_X_X_X_X
        .byte zz_X__XX__
        .byte zzXX_XXX_X
        .byte zzXX_XXX_X
        .byte zzXX_XXX_X
        .byte zzXX_XXX__
TVtypeD:
        .byte zz_XXXXXXX
        .byte zzXXXXXXXX 
        .byte zzXXXXXXXX
        .byte zzXXXXXXXX
        .byte zzXXXXXXXX
        .byte zzXXXXXXXX
        .byte zzXXXXXXXX
        .byte zz_XXXXXXX      
        
OptionNTSCA:
        .byte zzX__XX___ 
        .byte zzX_X_XX_X 
        .byte zzX_X_XX_X
        .byte zzX_X_XX_X
        .byte zzX_X_XX_X
        .byte zzX_X_XX_X
        .byte zzX_X_XX_X
        .byte zzX_X_XX_X
        
OptionNTSCB:
        .byte zzXX__XX_X 
        .byte zzX_XXX_X_ 
        .byte zzX_XXX_XX
        .byte zzXX_XX_XX
        .byte zzXXX_X_XX
        .byte zzXXX_X_XX
        .byte zzXXX_X_X_
        .byte zzX__XXX_X       
        
OptionPALA:
        .byte zzXXXXX__X 
        .byte zzXXXXX_X_ 
        .byte zzXXXXX_X_
        .byte zzXXXXX__X
        .byte zzXXXXX_XX
        .byte zzXXXXX_XX
        .byte zzXXXXX_XX
        .byte zzXXXXX_XX  
OptionPALB:
        .byte zzXX_XX_XX 
        .byte zzX_X_X_XX 
        .byte zzX_X_X_XX
        .byte zzX___X_XX
        .byte zzX_X_X_XX
        .byte zzX_X_X_XX
        .byte zzX_X_X_XX
        .byte zzX_X_X___  
                
OptionSECAMA:    
        .byte zz_XXXXX__ 
        .byte zzXXXXX_XX 
        .byte zzXXXXX_XX
        .byte zzXXXXXX_X
        .byte zzXXXXXXX_
        .byte zzXXXXXXX_
        .byte zzXXXXXXX_
        .byte zz_XXXX__X
        
OptionSECAMB:    
        .byte zzX___XX_X 
        .byte zzX_XXX_X_ 
        .byte zzX_XXX_XX
        .byte zzX__XX_XX
        .byte zzX_XXX_XX
        .byte zzX_XXX_XX
        .byte zzX_XXX_X_
        .byte zzX___XX_X
        
OptionSECAMC:    
        .byte zzXX_XX_X_ 
        .byte zzX_X_X___ 
        .byte zzX_X_X_X_
        .byte zzX___X_X_
        .byte zzX_X_X_X_
        .byte zzX_X_X_X_
        .byte zzX_X_X_X_
        .byte zzX_X_X_X_        

StartA:
        .byte zzXXXXXXXX
        .byte zzXXXXXXXX
        .byte zzXXXXXXXX
        .byte zzXXXXXXXX
        .byte zzXXXXXXXX
        .byte zzXXXXXXXX
        .byte zzXXXXXXXX
        .byte zzXXXXXXXX    
StartB:   
        .byte zzXXX____X
        .byte zzXX_XXXXX
        .byte zzXX_XXXXX
        .byte zzXXX___XX
        .byte zzXXXXXX_X
        .byte zzXXXXXX_X
        .byte zzXXXXXX_X
        .byte zzXX____XX  
StartC:
        .byte zz_____XX_
        .byte zzXX_XXX_X 
        .byte zzXX_XXX_X
        .byte zzXX_XXX__
        .byte zzXX_XXX_X
        .byte zzXX_XXX_X
        .byte zzXX_XXX_X
        .byte zzXX_XXX_X
StartD:
        .byte zz__XX____
        .byte zzXX_X_XXX 
        .byte zzXX_X_XXX
        .byte zz___X____
        .byte zzXX_X_XXX
        .byte zzXX_X_XXX
        .byte zzXX_X_XXX
        .byte zzXX_X_XXX    
StartE:
        .byte zzXX_____X
        .byte zz_XXX_XXX 
        .byte zz_XXX_XXX
        .byte zzXXXX_XXX
        .byte zz_XXX_XXX
        .byte zz_XXX_XXX
        .byte zz_XXX_XXX
        .byte zz_XXX_XXX    
    
Checked:
        .byte zzXXXXXX__ 
        .byte zzXXXXXX__ 
        .byte zzXXXXX__X 
        .byte zzXXXXX_XX 
        .byte zzX_XX_XXX 
        .byte zzXX_X_XXX 
        .byte zzXXX_XXXX 
        .byte zzXXX_XXXX 
    
UnChecked:
        .byte zzX_XXXX__ 
        .byte zzXX_XX__X 
        .byte zzXXX___XX 
        .byte zzXXX__XXX 
        .byte zzXX____XX 
        .byte zzXX_XX__X 
        .byte zzX_XXXX__ 
        .byte zzX_XXXX__ 
    
MenuDigit0:
LevelDigitsA:               ; C code merges values into here for menu display
        .byte zz____XX_X
        .byte zz____X_X_
        .byte zz____X_X_
        .byte zz____X_X_
        .byte zz____X_X_
        .byte zz____X_X_
        .byte zz____X_X_
        .byte zz____XX_X
        
MenuDigit1:
LevelDigitsB:               ; C code merges values into here for menu display
        .byte zz____XXX_
        .byte zz____XX__
        .byte zz____XXX_
        .byte zz____XXX_
        .byte zz____XXX_
        .byte zz____XXX_
        .byte zz____XXX_
        .byte zz____XXX_
        
MenuDigit2:
        .byte zz____X__X
        .byte zz____XXX_
        .byte zz____XXX_
        .byte zz____XX_X
        .byte zz____X_XX
        .byte zz____X_XX
        .byte zz____X_XX
        .byte zz____X___        

MenuDigit3:
        .byte zz____X__X
        .byte zz____XXX_
        .byte zz____XXX_
        .byte zz____XX_X
        .byte zz____XXX_
        .byte zz____XXX_
        .byte zz____XXX_
        .byte zz____X__X       
        
MenuDigit4:
        .byte zz____X_X_
        .byte zz____X_X_
        .byte zz____X_X_
        .byte zz____X___
        .byte zz____XXX_
        .byte zz____XXX_
        .byte zz____XXX_
        .byte zz____XXX_
        
MenuDigit5:
        .byte zz____X___
        .byte zz____X_XX
        .byte zz____X_XX
        .byte zz____X__X
        .byte zz____XXX_
        .byte zz____XXX_
        .byte zz____XXX_
        .byte zz____X__X
        
MenuDigit6:
        .byte zz____XX__
        .byte zz____X_XX
        .byte zz____X_XX
        .byte zz____X__X
        .byte zz____X_X_
        .byte zz____X_X_
        .byte zz____X_X_
        .byte zz____XX_X
        
MenuDigit7:
        .byte zz____X___
        .byte zz____XXX_
        .byte zz____XXX_
        .byte zz____XX_X
        .byte zz____XX_X
        .byte zz____X_XX
        .byte zz____X_XX
        .byte zz____X_XX  
        
MenuDigit8:
        .byte zz____XX_X
        .byte zz____X_X_
        .byte zz____X_X_
        .byte zz____XX_X
        .byte zz____X_X_
        .byte zz____X_X_
        .byte zz____X_X_
        .byte zz____XX_X       
        
MenuDigit9:
        .byte zz____XX_X
        .byte zz____X_X_
        .byte zz____X_X_
        .byte zz____XX__
        .byte zz____XXX_
        .byte zz____XXX_
        .byte zz____XXX_
        .byte zz____X__X  
        
MenuDigitSpace:
        .byte zz____XXXX
        .byte zz____XXXX
        .byte zz____XXXX
        .byte zz____XXXX
        .byte zz____XXXX
        .byte zz____XXXX
        .byte zz____XXXX
        .byte zz____XXXX   
    
SpiceWareLogoA:
        .byte zz____XXXX
        .byte zz___X____
        .byte zz___X____
        .byte zz___X____
        .byte zz____XXXX
        .byte zz________
        .byte zz________
        .byte zz________
        .byte zzXXXXXXXX    
         
SpiceWareLogoB:
        .byte zzXXXXXXXX
        .byte zz________
        .byte zz___XX__X
        .byte zz___X_X_X
        .byte zzX__X_X_X
        .byte zz_X_XX__X
        .byte zz_X_X___X
        .byte zz_X_X___X
        .byte zzX__X___X
    
SpiceWareLogoC:
        .byte zzXXXXXXXX
        .byte zz________
        .byte zz__X__XXX
        .byte zz_X_X_X__
        .byte zz_X___X__
        .byte zz_X___XX_
        .byte zz_X___X__
        .byte zz_X_X_X__
        .byte zz__X__XXX    
         
SpiceWareLogoD:
        .byte zzXX_____X
        .byte zz_X_____X
        .byte zz_XX___XX
        .byte zz__X_X_X_
        .byte zz__X_X_X_
        .byte zz__XXXXX_
        .byte zz___X_X__
        .byte zz___X_X__
        .byte zz___X_X__    

SpiceWareLogoE:
        .byte zzXXXXXXXX
        .byte zz________
        .byte zz__X__XX_
        .byte zz_X_X_X_X
        .byte zz_X_X_X_X
        .byte zz_XXX_XX_
        .byte zz_X_X_X_X
        .byte zz_X_X_X_X
        .byte zz_X_X_X_X    
    
SpiceWareLogoF:
        .byte zzXXXXXXXX
        .byte zz________
        .byte zz_XXX____
        .byte zz_X______
        .byte zz_X______
        .byte zz_XX_____
        .byte zz_X______
        .byte zz_X______
        .byte zz_XXX____    
    
    align 2 
MenuOptionsDatastream:
MENU_FIRST_OPTION_ID = (* - MenuOptionsDatastream) / 20
MENU_LEVEL_ID = (* - MenuOptionsDatastream) / 20
        .word LevelDigitsB
        .word LevelDigitsA
        .word OptionSpace
        .word LevelC
        .word LevelB
        .word LevelA
        .word MenuOptionGreen
        .byte zr____XXXX; PF0
        .byte zzXX______; PF1
        .byte zr_______X; PF2
        .byte 0; ball x location
        .byte 8; rows to show   
        .byte MM_OPTION_GAP; scanline padding   
        
MENU_TV_TYPE_ID = (* - MenuOptionsDatastream) / 20 
        .word OptionNTSCB
        .word OptionNTSCA
        .word TVtypeD
        .word TVtypeC
        .word TVtypeB
        .word TVtypeA
        .word MenuOptionGreen
        .byte zr_____XXX; PF0
        .byte zzXX______; PF1
        .byte zr________; PF2
        .byte 0; ball x location
        .byte 8; rows to show   
        .byte MM_START_GAP; scanline padding            
    
MENU_START_ID = (* - MenuOptionsDatastream) / 20        
        .word StartA
        .word StartE
        .word StartD
        .word StartC
        .word StartB
        .word StartA
        .word StartColor
        .byte 0; PF0
        .byte 0; PF1
        .byte 0; PF2
        .byte 0; ball x location
        .byte 8; rows to show
    
        .byte $80 ; no more menu options    
    
MainMenuColors:
DraconianLogoColor:
        .byte GREEN + 4 ; PF color, the station inside the O
        ds 31,  VIOLET + 2
        ds 6,   PURPLE + 2
        ds 6,   RED + 4
        ds 6,   RED + 6
        ds 6,   RED + 8
        ds 6,   RED_ORANGE + 8
        ds 6,   RED_ORANGE + 10
        ds 6,   RED_ORANGE + 12
        ds 6,   YELLOW + 12
        ds 11,  YELLOW + 14
        .byte 0
        
MenuOptionGreen:
        .byte GREEN + 8, BLUE + 8
        .byte GREEN +10, BLUE +10
        .byte GREEN +12, BLUE +12
        .byte GREEN +14, BLUE +14
        .byte GREEN +14, BLUE +14
        .byte GREEN +12, BLUE +12
        .byte GREEN +10, BLUE +10
        .byte GREEN + 8, BLUE + 8

MenuOptionRed:
        .byte RED + 8, BLUE + 8
        .byte RED +10, BLUE +10
        .byte RED +12, BLUE +12
        .byte RED +14, BLUE +14
        .byte RED +14, BLUE +14
        .byte RED +12, BLUE +12
        .byte RED +10, BLUE +10
        .byte RED + 8, BLUE + 8
    
MenuOptionGrey:
        .byte GREY+4, GREY+4 ;  0
        .byte GREY+4, GREY+4 ;  1
        .byte GREY+4, GREY+4 ;  2
        .byte GREY+4, GREY+4 ;  3
        .byte GREY+4, GREY+4 ;  4
        .byte GREY+4, GREY+4 ;  5
        .byte GREY+4, GREY+4 ;  6
        .byte GREY+4, GREY+4 ;  7
        
MenuSelectedColor:
        .byte GREY+6, GREY+ 8;  0
        .byte GREY+6, GREY+10;  1
        .byte GREY+6, GREY+12;  2
        .byte GREY+6, GREY+14;  3
        .byte GREY+6, GREY+14;  4
        .byte GREY+6, GREY+12;  5
        .byte GREY+6, GREY+10;  6
        .byte GREY+6, GREY+ 8;  7
    
StartColor:
        .byte WHITE, WHITE
        .byte WHITE, WHITE
        .byte WHITE, WHITE
        .byte WHITE, WHITE
        .byte WHITE, WHITE
        .byte WHITE, WHITE
        .byte WHITE, WHITE
        .byte WHITE, WHITE
        .byte WHITE, WHITE
        .byte 0, 0
        
MainMenuColorsSize = * - MainMenuColors
MainMenuGraphicsSize = * - MainMenuGraphics     

DisplayDataROMend:
; add extra space if the RAM usage is greater than the ROM usage
 if (DisplayDataRAMend > DisplayDataROMend)
  echo "------",( DisplayDataRAMend - DisplayDataROMend) , "bytes of Display Data RAM/ROM overlap left for ROM"
  ds ( DisplayDataRAMend - DisplayDataROMend)
 else
  echo "------",( DisplayDataROMend - DisplayDataRAMend) , "bytes of Display Data RAM/ROM overlap left for RAM" 
 endif      

RadarControl:
        .byte $C8       ; condition color
        .byte HMOVE_0   ; BallXadjust
        .byte $48       ; player0, enemy color 
        .byte $C8       ; player1, station color
; must directly follow RadarControl so radar colors get updated based on TV-Type
ObjectColors:   
        .byte $28       ; asteroid - orange
        .byte $88       ; mine - blue
        .byte $0c       ; ship - white, subtract 2 due to lumaboost
        .byte $58       ; itype - pink
        .byte $b8       ; ptype - cyan
        .byte $38       ; etype - reddish orange
        .byte $18       ; spy - yellow
        .byte $c8       ; station -green
        .byte $c4       ; condition green
        .byte $1A       ; condition yellow
        .byte $44       ; condition red        
ObjectColorSize = * - ObjectColors


StarX:      ds MAX_STARS
StarY:      ds MAX_STARS
StarColor:  ds MAX_STARS
        
; ---- START OF A DATASTREAM ----
BallX:          .byte 30
Missile1X:      .byte 60
Missile0X:      .byte 90
Player1X:       .byte 120
Player0X:       .byte 150
FirstNUSIZ0:    .byte 0         ; NUSIZ0
FirstNUSIZ1:    .byte 0         ; NUSIZ1
FirstCOLUP0:    .byte $4E       ; COLUP0
FirstCOLUP1:    .byte $CE       ; COLUP1
FirstCOLUPF:    .byte WHITE     ; COLUPF
; ----- END OF A DATASTREAM -----

ARMframe:          .byte 0        
LevelNumber:    ds 1
ActiveSound0:   ds 1 ; sound active in menu so these can't be in DD overlap
ActiveSound1:   ds 1
SoundDuration0: ds 1
SoundDuration1: ds 1


        
; This section is used to pass values between the 6507 and the ARM       
; ---- START OF A DATASTREAM ----
ARMsub:     ds 1    ; Which ARM subroutine to run:
            ;   0 = Initialize()
            ;   1 = OverScan()
            ;   2 = VerticalBlank()
            ;   3 = OverScanMenu() 
ARMswcha:   ds 1    ; controller input to ARM
ARMswchb:   ds 1    ; console switches to ARM
ARMinpt4:   ds 1    ; firebutton
ARMvbtime:  ds 1    ; vertical blank time remaining
ARMostime:  ds 1    ; over scan time remaining
ARMmode:    ds 1    ; 0 = show menu, 1 = show game, 2 show splash
ARMaudc0:       .byte 0
ARMaudc1:       .byte 0
ARMaudf0:       .byte 0
ARMaudf1:       .byte 0
ARMaudv0:       .byte 0
ARMaudv1:       .byte 0

; ----- END OF A DATASTREAM -----

; menu variables
MMjoystick:     .byte 0 ; joystick timer
MMfire:         .byte 0 ; fire held
MMselected:     .byte MENU_START_ID
MM_InitialLevel .byte 0;
MM_TVtype:      .byte 0 ; 0 = NTSC, 1 = PAL, 2 = SECAM

D1 = 0
D2 = 1*32
D3 = 2*32
D4 = 3*32
D5 = 4*32
D6 = 5*32
D7 = 6*32
D8 = 7*32
        
SoundFreq:  ; 0-31 0 = highest freq.
            ; upper 3 bits control duration, 0 = 1 frame, 1 = 2 frames... 7=8 frames
        .byte 0, 26, 25, 24, 23     ; Player Shoots
        .byte    22, 21, 20, 19     ; Player Shoots
        .byte    18, 17, 16, 15, 14 ; Player Shoots
        .byte 0, D2+$1b, D2+$15, D2+$1b, D2+$16 ; Asteroid Hit
        .byte    D2+$1b, D2+$17, D2+$1b, D2+$18 ; Asteroid Hit
        .byte    D2+$1b, D2+$19, D2+$1b, D2+$1a ; Asteroid Hit
        .byte    D2+$1b, D2+$1b, D2+$1b         ; Asteroid Hit      
        .byte 0, D1+$17, D1+$17, D2+$17, D2+$17 ; Mine Explosion
        .byte    D2+$17, D2+$17, D2+$17, D2+$17 ; Mine Explosion
        .byte    D2+$17, D2+$17, D2+$17, D2+$17 ; Mine Explosion             
        .byte 0, D2+$1f, D4+$1f, D4+$1f, D4+$1f         ; Player's Ship Explosion
        .byte    D4+$1f, D4+$1f, D4+$1f, D4+$1f         ; Player's Ship Explosion
        .byte    D4+$1f, D4+$1f, D4+$1f, D4+$1f, D4+$1f ; Player's Ship Explosion            
        .byte 0, D4+24 ; menu blip
        
SoundVol: ; upper nybble AUDC, lower nybble volume.   $00 = end of sound effect.
    .byte 0, $40, $41, $42, $43        ; Player Shoots
    .byte    $44, $45, $46, $47              ; Player Shoots
    .byte    $48, $49, $4a, $4b, $4c ; Player Shoots
SoundPlayerShoot: EQU * - SoundVol - 1

    .byte 0, $81, $62, $83, $64 ; Asteroid Hit
    .byte    $85, $66, $87, $68 ; Asteroid Hit
    .byte    $89, $6a, $8b, $6c ; Asteroid Hit
    .byte    $8d, $6e, $8f      ; Asteroid Hit
SoundAsteroidHit: EQU * - SoundVol - 1

    .byte 0, $81, $82, $83, $85 ; Mine Explosion
    .byte    $86, $88, $89, $8b ; Mine Explosion
    .byte    $8c, $8d, $8e, $8f ; Mine Explosion
SoundMineHit: EQU * - SoundVol - 1

    .byte 0, $81, $82, $83, $84         ; Player's Ship Explosion
    .byte    $85, $87, $88, $8a         ; Player's Ship Explosion
    .byte    $8b, $8c, $8d, $8e, $8f    ; Player's Ship Explosion
SoundPlayerHit: EQU * - SoundVol - 1

    .byte 0, $58 ; menu blip
SoundMenuBlip: EQU * - SoundVol - 1 
        
        
        ; last 1K of Display Data is allocated to C variable space
        echo "------",(($1000 - $400) - *) , "bytes of Display Data RAM left"
        

        if (* <= $1000)
          echo "------",($1000 - *) , "bytes of Display Data ROM left"
          ds ($1000 - *)        ; pad out remaining space in Display Data ROM

        else
          echo "FATAL ERROR - Display Data exceeds 4K"
          err
        endif   

        include DPC_frequencies.h
        
        ; when echoing out data for use in the C code, wrapping the variables in
        ; []d will cause them to be output in decimal instead of hexadecimal.
        ; this is handy as C requires a prefix of 0x for hex values, but echo outputs $.
        echo ""
        echo ""
        echo ""
        echo ""
        echo "#define FRAME                      (*(unsigned char *)(0x40000C00 +",[ARMframe]d,"))"
        echo ""
        echo "// data from 6507 to ARM"
        echo "#define SUB                        (*(unsigned char *)(0x40000C00 +",[ARMsub]d,"))"
        echo "#define SWCHA                      (*(unsigned char *)(0x40000C00 +",[ARMswcha]d,"))"
        echo "#define SWCHB                      (*(unsigned char *)(0x40000C00 +",[ARMswchb]d,"))"
        echo "#define INPT4                      (*(unsigned char *)(0x40000C00 +",[ARMinpt4]d,"))"
        echo "#define VB_TIME                    (*(unsigned char *)(0x40000C00 +",[ARMvbtime]d,"))"
        echo "#define OS_TIME                    (*(unsigned char *)(0x40000C00 +",[ARMostime]d,"))"
        
        echo ""
        echo "// data from ARM to 6507 - from overscan call"
        echo "#define MODE                       (*(unsigned char *)(0x40000C00 +",[ARMmode]d,"))"
        echo "#define AUDV                       ((unsigned char *)(0x40000C00 +",[ARMaudv0]d,"))"
        echo "#define AUDF                       ((unsigned char *)(0x40000C00 +",[ARMaudf0]d,"))"
        echo "#define AUDC                       ((unsigned char *)(0x40000C00 +",[ARMaudc0]d,"))"        
        echo "#define AUDV0                      (*(unsigned char *)(0x40000C00 +",[ARMaudv0]d,"))"
        echo "#define AUDF0                      (*(unsigned char *)(0x40000C00 +",[ARMaudf0]d,"))"
        echo "#define AUDC0                      (*(unsigned char *)(0x40000C00 +",[ARMaudc0]d,"))"
        echo "#define AUDV1                      (*(unsigned char *)(0x40000C00 +",[ARMaudv1]d,"))"
        echo "#define AUDF1                      (*(unsigned char *)(0x40000C00 +",[ARMaudf1]d,"))"
        echo "#define AUDC1                      (*(unsigned char *)(0x40000C00 +",[ARMaudc1]d,"))"
        echo "#define PLAYER_X                   ((unsigned char *)(0x40000C00 +",[Player1X]d,"))"
        echo "#define PLAYER_0_X                 (*(unsigned char *)(0x40000C00 +",[Player0X]d,"))"
        echo "#define PLAYER_1_X                 (*(unsigned char *)(0x40000C00 +",[Player1X]d,"))"
        echo "#define MISSILE_X                  ((unsigned char *)(0x40000C00 +",[Missile1X]d,"))"
        echo "#define MISSILE_0_X                (*(unsigned char *)(0x40000C00 +",[Missile0X]d,"))"
        echo "#define MISSILE_1_X                (*(unsigned char *)(0x40000C00 +",[Missile1X]d,"))"
        echo "#define BALL_X                     (*(unsigned char *)(0x40000C00 +",[BallX]d,"))"
        echo "#define FIRST_NUSIZ                ((unsigned char *)(0x40000C00 +",[FirstNUSIZ0]d,"))"
        echo "#define FIRST_NUSIZ0               (*(unsigned char *)(0x40000C00 +",[FirstNUSIZ0]d,"))"
        echo "#define FIRST_NUSIZ1               (*(unsigned char *)(0x40000C00 +",[FirstNUSIZ1]d,"))"
        echo "#define FIRST_COLUP                ((unsigned char *)(0x40000C00 +",[FirstCOLUP0]d,"))"
        echo "#define FIRST_COLUP0               (*(unsigned char *)(0x40000C00 +",[FirstCOLUP0]d,"))"
        echo "#define FIRST_COLUP1               (*(unsigned char *)(0x40000C00 +",[FirstCOLUP1]d,"))"
        echo "#define FIRST_COLUPF               (*(unsigned char *)(0x40000C00 +",[FirstCOLUPF]d,"))"

        echo ""        
        echo "// kernel datastreams"
        echo "#define DATASTREAM_SIZE            ", [DATASTREAM_SIZE]d
        echo "#define P0_DATASTREAM              ", [Player0DataStream]d
        echo "#define P1_DATASTREAM              ", [Player1DataStream]d
        echo "#define MISSILE_BALL_DATASTREAM    ", [MissileBallDataStream]d
        echo "#define HMP0_DATASTREAM            ", [HMP0DataStream]d
        echo "#define HMP1_DATASTREAM            ", [HMP1DataStream]d
        echo "#define COLOR_HM_DATASTREAM        ", [ColorHMDataStream]d
        echo "#define SIZE_BALL_COLOR_DATASTREAM ", [SizeBallColorDataStream]d
        echo "#define JUMP_DATASTREAM            ", [JumpDataStream]d
        echo "#define MISSILE_REPOSITION_ACTIVE  ((unsigned char *)(0x40000C00 +",[MissileRepositionActive]d,"))"        
        echo ""        
        echo "// jump data"
        echo "#define NORMAL_KERNEL              ", [NormalKernel]d
        echo "#define KERNEL_DONE                ", [KernelDone]d
        echo "#define FIRST_P0_JUMP              ", [Resp0Strobe23]d
        echo "#define LAST_P0_JUMP               ", [Resp0Strobe73]d
        echo "#define FIRST_P1_JUMP              ", [Resp1Strobe23]d
        echo "#define LAST_P1_JUMP               ", [Resp1Strobe73]d
        echo "#define FIRST_M0_JUMP              ", [Resm0Strobe23]d
        echo "#define LAST_M0_JUMP               ", [Resm0Strobe73]d
        echo "#define FIRST_M1_JUMP              ", [Resm1Strobe23]d
        echo "#define LAST_M1_JUMP               ", [Resm1Strobe73]d
        echo "#define RESP0_STROBE_TABLE         ((unsigned short int *)(0x0C00 +",[Resp0StrobeTable]d,"))"
        echo "#define RESP1_STROBE_TABLE         ((unsigned short int *)(0x0C00 +",[Resp1StrobeTable]d,"))"
        echo "#define RESM0_STROBE_TABLE         ((unsigned short int *)(0x0C00 +",[Resm0StrobeTable]d,"))"
        echo "#define RESM1_STROBE_TABLE         ((unsigned short int *)(0x0C00 +",[Resm1StrobeTable]d,"))"
        echo "#define RESBL_STROBE_TABLE         ((unsigned short int *)(0x0C00 +",[ResblStrobeTable]d,"))"
        echo "#define INIT_LEVEL_KERNEL          ", [InitLevel]d
                
        echo ""
        echo "// Image Data"
        echo "#define IMAGE_HEIGHTS              ", [ImageHeights]d
        echo "#define IMAGE_SHIFT                ", [ImageShift]d
        echo "#define IMAGE_GRAPHICS             ", [ImageGraphics]d
        echo "#define STATION_IMAGES             ", [StationImages]d
        echo "#define STATION_HEIGHT             ", [StationHorizontalHeight]d
        echo "#define OBJECT_COLORS              ", [ObjectColors]d
        echo "#define OBJECT_COLOR_SIZE          ", [ObjectColorSize]d
        echo ""
        echo "// Image IDs"
        echo "#define QUAD_SPRITES               ", [QuadSprites]d
        echo "#define DOUBLE_SPRITES             ", [DoubleSprites]d
        echo "#define SHIFTING_SPRITES           ", [ShiftingSprites]d
        echo "#define STATION_HORIZONTAL_ID      ", [StationHorizontalID]d
        echo "#define STATION_VERTICAL_ID        ", [StationVerticalID]d
        echo "#define ASTEROID_ID                ", [AsteroidID]d
        echo "#define MINE_ID                    ", [MineID]d
        echo "#define SHIP_ID                    ", [ShipID]d
        echo "#define SHIP_HEIGHT                ", [Ship0Height]d
        echo "#define ETYPE_ID                   ", [ETypeID]d
        echo "#define ITYPE_ID                   ", [ITypeID]d
        echo "#define PTYPE_ID                   ", [PTypeID]d
        echo "#define SPY_ID                     ", [SpyID]d
        echo "#define SHOT_HORIZONTAL_ID         ",[ShotHorizontalID]d
        echo "#define SHOT_DIAGONAL_ID           ",[ShotDiagonalID]d
        echo "#define SHOT_VERTICAL_ID           ",[ShotVerticalID]d
        echo "#define SHOT_STATION_ID            ",[ShotStationID]d
        echo "#define EXPLOSION_ID               ",[ExplosionID]d
        echo "#define LAST_EXPLOSION_ID          ",[LastExplosionID]d
        echo "#define POD_ID                     ",[PodID]d
        echo "#define HORIZONTAL_ENERGY_POD_ID   ",[HorizontalEnergyPodID]d
        echo "#define VERTICAL_ENERGY_POD_ID     ",[VerticalEnergyPodID]d
        
        echo ""        
        echo "// starfield"
        echo "#define MAX_STARS                  ", [MAX_STARS]d
        echo "#define STARFIELD_X                ((unsigned char *)(0x40000C00 +",[StarX]d,"))"
        echo "#define STARFIELD_Y                ((unsigned char *)(0x40000C00 +",[StarY]d,"))"
        echo "#define STARFIELD_COLOR            ((unsigned char *)(0x40000C00 +",[StarColor]d,"))"
        echo ""
        echo "// score, radar, status"
        echo "#define RADAR_HEIGHT               ", [RADAR_HEIGHT]d
        echo "#define PLAYFIELD_DATA             ", [PF0L]d
        echo "#define PF0L                       ", [PF0L-PF0L]d
        echo "#define PF1L                       ", [PF1L-PF0L]d
        echo "#define PF2L                       ", [PF2L-PF0L]d
        echo "#define PF0R                       ", [PF0R-PF0L]d
        echo "#define PF1R                       ", [PF1R-PF0L]d
        echo "#define PF2R                       ", [PF2R-PF0L]d
        echo "#define PF_DIGIT0                  ((unsigned char *)(0x0C00 +",[PFdigit0]d,"))"
        echo "#define PF_DIGIT_HEIGHT            ", [PFdigitHeight]d
        echo "#define PF_DIGIT_X                 ", [[PFdigitX - PFdigit0] / PFdigitHeight]d
        echo "#define PF_DIGIT_G                 ", [[PFdigitG - PFdigit0] / PFdigitHeight]d
        echo "#define PF_DIGIT_Y                 ", [[PFdigitY - PFdigit0] / PFdigitHeight]d
        echo "#define PF_DIGIT_R                 ", [[PFdigitR - PFdigit0] / PFdigitHeight]d
        echo "#define PF_DIGIT_V                 ", [[PFdigitV - PFdigit0] / PFdigitHeight]d
        echo "#define PF_DIGIT_O                 ", [[PFdigitO - PFdigit0] / PFdigitHeight]d
        echo "#define RADAR_CONTROL              ((unsigned char *)(0x40000C00 +",[RadarControl]d,"))"
        echo ""
        echo "// Sound Effects"
        echo "#define ACTIVE_SOUND               ((unsigned char *)(0x40000C00 +",[ActiveSound0]d,"))"
        echo "#define ACTIVE_SOUND_0             (*(unsigned char *)(0x40000C00 +",[ActiveSound0]d,"))"
        echo "#define ACTIVE_SOUND_1             (*(unsigned char *)(0x40000C00 +",[ActiveSound1]d,"))"
        echo "#define SOUND_DURATION             ((unsigned char *)(0x40000C00 +",[SoundDuration0]d,"))"
        echo "#define SOUND_DURATION_0           (*(unsigned char *)(0x40000C00 +",[SoundDuration0]d,"))"
        echo "#define SOUND_DURATION_1           (*(unsigned char *)(0x40000C00 +",[SoundDuration1]d,"))"
        echo "#define SOUND_FREQ                 ",[[SoundFreq & $fff] + $6000]d
        echo "#define SOUND_VOL                  ",[[SoundVol & $fff] + $6000]d
        echo "#define SOUND_PLAYER_SHOOTS_ID     ",[SoundPlayerShoot]d
        echo "#define SOUND_ASTEROID_ID          ",[SoundAsteroidHit]d
        echo "#define SOUND_MINE_ID              ",[SoundMineHit]d
        echo "#define SOUND_PLAYER_HIT_ID        ",[SoundPlayerHit]d
        echo "#define SOUND_MENU_BLIP_ID         ",[SoundMenuBlip]d

        echo ""
        echo "// Main Menu"
        echo "#define MENU_OPTIONS_DS            ", [MenuOptionsDatastream]d
        echo "#define MAIN_MENU_GRAPHICS         ", [MainMenuGraphics]d
        echo "#define MAIN_MENU_GFX_SIZE         ", [MainMenuGraphicsSize]d
        echo "#define MAIN_MENU_COLORS           ", [MainMenuColors]d
        echo "#define MAIN_MENU_CLR_SIZE         ", [MainMenuColorsSize]d        
        echo "#define MENU_SELECTED_COLOR        ", [MenuSelectedColor]d
        echo "#define MENU_FIRST_OPTION_ID       ", [MENU_FIRST_OPTION_ID]d
        echo "#define MENU_LEVEL_ID              ", [MENU_LEVEL_ID]d
        echo "#define MENU_TV_TYPE_ID            ", [MENU_TV_TYPE_ID]d
        echo "#define MENU_START_ID              ", [MENU_START_ID]d
        echo "#define OPTION_PAL_A               ", [OptionPALA]d
        echo "#define OPTION_PAL_B               ", [OptionPALB]d
        echo "#define OPTION_SECAM_A             ", [OptionSECAMA]d
        echo "#define OPTION_SECAM_B             ", [OptionSECAMB]d
        echo "#define OPTION_SECAM_C             ", [OptionSECAMC]d
        
        echo "#define MM_JOYSTICK_TIMER          (*(unsigned char *)(0x40000C00 +",[MMjoystick]d,"))"
        echo "#define MM_FIRE_DOWN               (*(unsigned char *)(0x40000C00 +",[MMfire]d,"))"
        echo "#define MM_SELECTED_OPTION         (*(unsigned char *)(0x40000C00 +",[MMselected]d,"))"
        echo "#define MM_INITIAL_LEVEL           (*(unsigned char *)(0x40000C00 +",[MM_InitialLevel]d,"))"
        echo "#define MM_TV_TYPE                 (*(unsigned char *)(0x40000C00 +",[MM_TVtype]d,"))"
        echo "#define MENU_DIGITS                ((unsigned char *)(0x0C00 +",[MenuDigit0 + $6000]d,"))"
        echo "#define LEVEL_DIGITS_A             ((unsigned char *)(0x40000C00 +",[LevelDigitsA]d,"))"
        echo "#define LEVEL_DIGITS_B             ((unsigned char *)(0x40000C00 +",[LevelDigitsB]d,"))"

